#ifndef GCC_HICOVEC_H
#define GCC_HICOVEC_H

/* User defines */
#define STORE_REG_NUMBER 0
#define X_REG_NUMBER 1
#define BP_REG_NUMBER 2
#define SP_REG_NUMBER 3

#define LOCAL_LABEL_PREFIX "L"  

/* GCC defines */

/*****************************************************************************/
/* RUN-TIME TARGET SPECIFICATION					     */
/*****************************************************************************/

#ifndef TARGET_CPU_CPP_BUILTINS
#define TARGET_CPU_CPP_BUILTINS()				\
do {								\
     builtin_define("__HICOVEC__");				\
} while (0)
#endif

#define INITIAL_FRAME_POINTER_OFFSET(DEPTH_VAR) 0


/*****************************************************************************/
/* GENERATING CODE FOR PROFILING - NOT IMPLEMENTED			     */
/*****************************************************************************/

#undef  FUNCTION_PROFILER
#define FUNCTION_PROFILER(STREAM, LABELNO)	\
{						\
    sorry ("Profiler support for Hicovec");	\
}

#ifndef CUMULATIVE_ARGS
struct cumulative_args
{
  int ints;
};

#define CUMULATIVE_ARGS struct cumulative_args
#endif

/* REGISTERS */

#define FIRST_PSEUDO_REGISTER 4

/* A: 0
   X: 1
   Y/BP: 2 
   SP: 3 */
#define FIXED_REGISTERS \
  {0, 0, 1, 1}
  
#define CALL_USED_REGISTERS \
  {1, 1, 1, 1}
  
#define HARD_REGNO_NREGS(REGNO, MODE)            \
  ((GET_MODE_SIZE (MODE) + UNITS_PER_WORD - 1)  \
    / UNITS_PER_WORD)

enum reg_class
{
  NO_REGS,  
  GENERAL_REGS,
  ARITH_REGS,
  SP_REGS,
  ALL_REGS,  
  LIM_REG_CLASSES
};

#define N_REG_CLASSES (int) LIM_REG_CLASSES

#define REG_CLASS_NAMES \
{			\
    "NO_REGS",		\
    "GENERAL_REGS",	\
    "ARITH_REGS",       \
    "SP_REGS",          \
    "ALL_REGS"		\
}


/* #define SMALL_REGISTER_CLASSES 1 */

/* Don't change these - It might be useful for optimizing to prefer
   register A, but it breaks the register allocator because of too few
   registers */
#define PREFERRED_RELOAD_CLASS(X,CLASS) CLASS
#define PREFERRED_OUTPUT_RELOAD_CLASS(X,CLASS) CLASS

#define REG_CLASS_CONTENTS				\
  {							\
    {0x00000000}, /* NO_REGS			*/	\
    {0x00000001}, /* GENERAL_REGS 		*/      \
    {0x00000007}, /* ARITH_REGS   	 	*/	\
    {0x00000008}, /* SP_REGS                    */      \
    {0x0000000F}  /* ALL_REGS   	 	*/	\
  }
  
#define REGNO_REG_CLASS(REGNO)  hicovec_regno_reg_class(REGNO)
  
#define BASE_REG_CLASS		ARITH_REGS

#define INDEX_REG_CLASS		ARITH_REGS
 
 #define REG_CLASS_FROM_LETTER(C)	\
  ((C) == 'a' ? ARITH_REGS : 		\
  ((C) == 'R' ? SP_REGS : 		\
  NO_REGS)

  
#define REG_CLASS_FROM_CONSTRAINT(CHAR, STR) hicovec_reg_class_from_constraint(CHAR)  
  

#define MAX_REGS_PER_ADDRESS 1

/*****************************************************************************/
/* STACK LAYOUT AND CALLING CONVENTIONS.				     */
/*****************************************************************************/

#define STACK_GROWS_DOWNWARD 1

#define FRAME_GROWS_DOWNWARD 1

#undef ARGS_GROW_DOWNWARD

/* The Stack pointer points to the last valid var */
#define STACK_POINTER_OFFSET 0

#define STARTING_FRAME_OFFSET -4

#define FIRST_PARM_OFFSET(FNDECL)  8

#define FRAME_POINTER_REQUIRED 1

#define FUNCTION_ARG(CUM, MODE, TYPE, NAMED) 0

#define FUNCTION_ARG_ADVANCE(CUM, MODE, TYPE, NAMED) 0

#define	STACK_POINTER_REGNUM	SP_REG_NUMBER

#define	FRAME_POINTER_REGNUM	BP_REG_NUMBER

#define	ARG_POINTER_REGNUM	BP_REG_NUMBER

#define PUSH_ARGS 0

#define AVOID_CCMODE_COPIES

/*****************************************************************************/
/* MISCELLANEOUS PARAMETERS						     */
/*****************************************************************************/

#define CASE_VECTOR_MODE  SImode

#define MOVE_MAX 4

#define TRULY_NOOP_TRUNCATION(OUTPREC, INPREC)  1

#define FUNCTION_ARG_REGNO_P(x) 0 /*No values can be passed in registers*/

#define FUNCTION_VALUE(VALTYPE, FUNC) \
	gen_rtx_REG(TYPE_MODE (VALTYPE), 0)

#define FUNCTION_VALUE_REGNO_P(N)	((N) == STORE_REG_NUMBER)

#define RETURN_POPS_ARGS(FNDECL, FUNTYPE, SIZE)   0

#define INIT_CUMULATIVE_ARGS(CUM, FNTYPE, LIBNAME, FNDECL, N_NAMED_ARGS) 0

#define HARD_REGNO_MODE_OK(REGNO, MODE) hicovec_hard_regno_mode_ok(REGNO, MODE)

#define MODES_TIEABLE_P(MODE1, MODE2)  0

#define LIBCALL_VALUE(MODE)	gen_rtx_REG (MODE, STORE_REG_NUMBER)
  
/*****************************************************************************/
/* TRAMPOLINES FOR NESTED FUNCTIONS - NOT SUPPORTED      		     */
/*****************************************************************************/

#define TRAMPOLINE_SIZE	32

#define INITIALIZE_TRAMPOLINE(addr, fnaddr, static_chain)	\
{								\
    sorry ("Trampoline support for HiCoVec");			\
}

#define SLOW_BYTE_ACCESS  1

#define NOTICE_UPDATE_CC(EXT, INSN) hicovec_notice_update_cc(EXT,INSN) 

#define CLASS_MAX_NREGS(CLASS, MODE) 1

#define CONST_DOUBLE_OK_FOR_CONSTRAINT_P(VALUE, C, STR) 0

#define CONST_OK_FOR_CONSTRAINT_P(VALUE, C, STR) 0

/*****************************************************************************/
/* ADDRESSING MODES							     */
/*****************************************************************************/

#define CONSTANT_ADDRESS_P(X) CONSTANT_P(X)
#define HAVE_POST_INCREMENT  0
#define HAVE_POST_DECREMENT  0
#define HAVE_POST_MODIFY_DISP 0
#define HAVE_POST_MODIFY_REG 0

#define REGNO_OK_FOR_BASE_P(REGNO) ((REGNO) < 3)
#define REGNO_OK_FOR_INDEX_P(REGNO)	   REGNO_OK_FOR_BASE_P(REGNO)

#ifdef REG_OK_STRICT
#define REG_OK_FOR_BASE_P(X)	REGNO_OK_FOR_BASE_P (REGNO (X))
#define REG_OK_FOR_INDEX_P(X)	REGNO_OK_FOR_INDEX_P (REGNO (X))
#else
#define REG_OK_FOR_BASE_P(X)	1
#define REG_OK_FOR_INDEX_P(X)	1
#endif

#ifdef REG_OK_STRICT
#define GO_IF_LEGITIMATE_ADDRESS(MODE, X, LABEL)			\
{									\
  if (hicovec_legitimate_address_p (MODE, X, 1))			\
      goto LABEL;							\
}
#else
#define GO_IF_LEGITIMATE_ADDRESS(MODE, X, LABEL)			\
{									\
  if (hicovec_legitimate_address_p (MODE, X, 0))			\
      goto LABEL;							\
}
#endif /* REG_OK_STRICT */

#define LEGITIMATE_CONSTANT_P(X)  ((INTVAL(X) <0x8000) && (INTVAL(X) >= -0x8000))

#define GO_IF_MODE_DEPENDENT_ADDRESS(ADDR, LABEL)

/*****************************************************************************/
/* LAYOUT OF SOURCE LANGUAGE DATA TYPES					     */
/*****************************************************************************/

#define Pmode		SImode

#define FUNCTION_MODE	SImode

#define BITS_BIG_ENDIAN  0

#define BYTES_BIG_ENDIAN 0

#define WORDS_BIG_ENDIAN 0

#define UNITS_PER_WORD 4

#define BITS_PER_UNIT 8

#define POINTER_SIZE 32

#define PARM_BOUNDARY 32

#define STACK_BOUNDARY 32

#define FUNCTION_BOUNDARY 32

#define STRUCTURE_SIZE_BOUNDARY 32

#define BIGGEST_ALIGNMENT 32

#define BASIC_ALIGNMENT 32

#define STRICT_ALIGNMENT 1

#define CHAR_TYPE_SIZE		32

#define SHORT_TYPE_SIZE		32

#define INT_TYPE_SIZE		32

#define LONG_TYPE_SIZE		32

#define LONG_LONG_TYPE_SIZE	32

#define FLOAT_TYPE_SIZE 	32

#define DOUBLE_TYPE_SIZE 	32

#define LONG_DOUBLE_TYPE_SIZE   32

#define DEFAULT_SIGNED_CHAR	1

#define SIZE_TYPE		"unsigned int"

#define PTRDIFF_TYPE		"unsigned int"

/*****************************************************************************/
/* ASSEMBLER FORMAT							     */
/*****************************************************************************/

#define GLOBAL_ASM_OP ";\t.globl\t"

#undef	ASM_APP_ON
#define ASM_APP_ON   ";APP\n"

#undef	ASM_APP_OFF
#define ASM_APP_OFF  ";NO_APP\n"

#define ASM_OUTPUT_ALIGN(STREAM, POWER) \
  asm_fprintf ((STREAM), "\n")


#define ASM_OUTPUT_SKIP(STREAM, NBYTES) \
{                                       \
int i;                                  \
for(i=0;i<(NBYTES);i++)                 \
   fprintf((STREAM),".dc 0");           \
}

#define ASM_OUTPUT_COMMON(FILE, NAME, SIZE, ROUNDED)  \
  do {                                                                  \
    int i;                                                              \
    fputs ("\n;.comm \n", (FILE));                                          \
    assemble_name ((FILE), (NAME));                                     \
    fprintf ((FILE), ":\n");                                            \
    for(i = 0; i < ROUNDED; i++)                                        \
      fprintf ((FILE), ".dc 0\n");                                      \
  } while(0)

#define ASM_OUTPUT_LOCAL(FILE, NAME, SIZE, ROUNDED)  \
  do {                                                                  \
    int i;                                                              \
    fputs ("\n;.lcomm \n", (FILE));                                          \
    assemble_name ((FILE), (NAME));                                     \
    fprintf ((FILE), ":\n");                                            \
    for(i = 0; i < ROUNDED; i++)                                        \
      fprintf ((FILE), ".dc 0\n");                                      \
  } while(0)

#define TEXT_SECTION_ASM_OP	""
				 
#define DATA_SECTION_ASM_OP	""
				 
#define BSS_SECTION_ASM_OP	""

#define ASM_OUTPUT_ADDR_VEC_ELT(STREAM, VALUE) \
  asm_fprintf ((STREAM), "\t.dc\t.0x%x\n", (VALUE))


#define ASM_OUTPUT_ASCII(STREAM, PTR, LEN)             \
  do {                                                  \
    int i;                                              \
    for(i = 0; i < LEN; i++)                            \
      fprintf(STREAM, ".dc 0x%x\n", PTR[i] & 0xff);     \
    fprintf(STREAM, ".dc 0\n");                         \
  } while (0)

#define ASM_GENERATE_INTERNAL_LABEL(LABEL,PREFIX,NUM)	\
  sprintf (LABEL, "*%s%s%ld", LOCAL_LABEL_PREFIX, PREFIX, (long)(NUM))


/*****************************************************************************/
/* INSTRUCTION OUTPUT							     */
/*****************************************************************************/

#define REGISTER_NAMES \
  { \
    "A",  "X",  "Y",  "SP" \
  }

#define PRINT_OPERAND(STREAM, X, CODE) hicovec_print_operand(STREAM, X, CODE)

#define PRINT_OPERAND_ADDRESS(STREAM, ADDR) hicovec_print_operand_address(STREAM, ADDR)

/*****************************************************************************/
/* OUTPUT OF DISPATCH TABLES						     */
/*****************************************************************************/

#include "hicovec-protos.h"

#if defined(__STDC__) || defined(ALMOST_STDC)
#define AS2(a,b,c) #a " " #b "," #c
#define AS2C(b,c) " " #b "," #c
#define AS3(a,b,c,d) #a " " #b "," #c "," #d
#define AS1(a,b) #a " " #b
#else
#define AS1(a,b) "a	b"
#define AS2(a,b,c) "a	b,c"
#define AS2C(b,c) " b,c"
#define AS3(a,b,c,d) "a	b,c,d"
#endif
#define OUT_AS1(a,b) output_asm_insn (AS1(a,b), operands)
#define OUT_AS2(a,b,c) output_asm_insn (AS2(a,b,c), operands)
#define CR_TAB "\n\t"

/* Taken from Alpha - Doesn't work for us, as we can't swap immediates. Feel free to fix this */
/* #define CANONICALIZE_COMPARISON(CODE,OP0,OP1) \ */
/*   do {									\ */
/*     if (((CODE) == LE || (CODE) == GT || (CODE) == LEU || (CODE) == GTU)) \ */
/*       {									\ */
/* 	rtx tem = (OP0);						\ */
/* 	(OP0) = (OP1);							\ */
/* 	(OP1) = tem;							\ */
/* 	(CODE) = reverse_condition (CODE);					\ */
/*       }									\ */
/*   } while (0) */
/* 	/\* && (GET_CODE (OP1) == REG || (OP1) == const0_rtx))		\ *\/ */

const char * ret_cond_branch (rtx x, int reverse);

#endif
