package hicosim.components.backend.io;


import hicosim.components.backend.processor.IMemory;
import hicosim.control.utils.BitUtils;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Vector;

/**
 * ObjectFileReader - Class for reading in and interpreting 
 * an objectfile assembled with the scotchas assembler.
 * 
 * comment-, info-, register-, breakpoint, and linesection are ignored.
 * 
 * @author Michael Wager, 16th May 2009
 */
public class ObjectfileReader {

	private DataInputStream dataInStream = null;
	private File file=null;
	
	private Vector<Integer> startAddressIndexesCode32=null;
	private Vector<Integer> endAddressIndexesCode32=null;
	
	//fuer programCounter:
	private int startAddressProgCounter=0;
	
	//fuer die Labels:
	private int startAddressIndexLabels=0;
	private int endAddressIndexLabels=0;
	
	private int programCounter=0;
	
	private hicosim.components.backend.processor.Labels labels = null;
	
	private IMemory memory = null;	
	
	/** 
	 * Instanciate a vector for start-/ endAddresses of hicovec code section.
     */
	public ObjectfileReader(IMemory memory ) {
		this.startAddressIndexesCode32 = new Vector<Integer>();	
		this.endAddressIndexesCode32 = new Vector<Integer>();
		
		this.labels = new hicosim.components.backend.processor.Labels(); //zB an adresse 255 = LABEL1
		this.memory = memory;
	}
	
	/**
	 * set up a new DataInputStream
	 * @param pathToFile
	 */
	public int setUp(String pathToFile) throws Exception {
		if(!pathToFile.endsWith(".o") ) {
			throw new Exception("FILENAME MUST END WITH '.o'");
		}
		try { 
			dataInStream = new DataInputStream(new FileInputStream(new File(pathToFile)));
			return 0;
		}
		catch (Exception e) {
			return -1;	//file not found
		}	
	}	
	
	/**
	 * close the streams
	 */
	public void closeStreams() {
		try {
			dataInStream.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 *	This method reads in byte per byte and fills the "instructionRegister"
	 *	with the 32 bit instructions which are needed to simulate the HiCoVec processor.
	 *  It also checks the labels and the programCounter. 
	 */
	public void readUnsignedBytes() {
		int intRead=0;
		try {
			Vector<Integer> bytesRead = new Vector<Integer>();
			
			int count=0, size=dataInStream.available();
//			System.out.println("SIZE: " + size);
			
//LESEN               =================================================================================EINLESEN:
			intRead = dataInStream.readUnsignedByte(); // read in one byte!!! 
			while(count < size ) { 					  
				bytesRead.add(intRead);
				
				try {
					intRead = dataInStream.readUnsignedByte();  
				}
				catch (java.io.EOFException e) {
//					nothing to do here!									
				}					
				count++;
			}
	
			//========================================================================================32 bit befehle:				
			
//								NOW THE IMPORTANT 32bit INSTRUCTIONS:			
			
			int q=1, instruction=0;
			
			//immer 4 bytes werden zu einem Integer konvertiert welcher dann einen 32bit Befehl darstellt
			int intArray[] = new int[4]; 
			
			//fuer alle startaddressen(also jedes .org) die je 32 bits in den vector fuellen
			for(int j=0; j < this.startAddressIndexesCode32.size(); j++) {
				//ADRESSE DER CODESEKTION AUS $ BYTES BERECHNEN:
				int a1 = bytesRead.get(this.startAddressIndexesCode32.get(j) + 0 );
				int a2 = bytesRead.get(this.startAddressIndexesCode32.get(j) + 1 );
				int a3 = bytesRead.get(this.startAddressIndexesCode32.get(j) + 2 );
				int a4 = bytesRead.get(this.startAddressIndexesCode32.get(j) + 3 );
				int address = BitUtils.fourBytesToOneInteger(a1, a2, a3, a4);
//				System.out.println("Codesektion #"+ (j+1) + "  AB ADRESSE: " +  address);
				//fuer jede Codesektion: VON START- BIS ZUR END-ADRESSE DURCHGEHN !!!
				//sind immer: erst 32 bit Adresse / dann die 32 bit instructions...
				// 
				Vector<Integer> currentInstructions = new Vector<Integer>();
				for(int i=this.startAddressIndexesCode32.get(j)+4; i < this.endAddressIndexesCode32.get(j)+1; i++) {  
					
					intArray[q-1] = bytesRead.get(i);																								
					
					if ( (q % 4) == 0) { 
						q=0;
						//HIER WIRD KONVERTIERT:
						instruction = BitUtils.fourBytesToOneInteger(intArray[0], intArray[1], intArray[2], intArray[3]);
//						System.out.println(instruction + ": " + Integer.toBinaryString(instruction));
						currentInstructions.add(instruction);																						
					}
					q++;					
				}
				//FUELLEN:
				//System.out.println("Befehle an Adresse " + address + ": " + r.getInstructionsAt(address));
				for(int i=0; i<currentInstructions.size(); i++) {
					this.memory.writeTo(address++, currentInstructions.get(i));
				}
												
			}																			
					
			//========================================================================================Programm Zaehler:
//			System.out.println("\nREGISTER: Programm Counter(32bit) und Inhalt von A,X,Y(je 32bit):");
			
			//Program counter == 32 bit also die ersten 4 bytes als eine Zahl interpretieren.....!!!					
			intArray = new int[4];//fuer programmZaehler!
						
			int i=this.startAddressProgCounter;
			this.programCounter	= BitUtils.fourBytesToOneInteger(bytesRead.get(i),bytesRead.get(i+1),bytesRead.get(i+2),bytesRead.get(i+3) );																	
			
			//================================================================================================LABELS:						
			
			//ERST DIE FUER LABELS RELEVANTEN BYTES HOLEN und temporaer speichern:
			Vector<Integer> intVec = new Vector<Integer>();
			for(int h=this.startAddressIndexLabels; h<this.endAddressIndexLabels+1; h++) {
//				System.out.print(bytesRead.get(h) + " ");
				intVec.add(bytesRead.get(h));
			}			
			
			//Anzahl Labels:   [ ---> 16 bit: (deshalb hier zwei bytes zu einer 16 bit Zahl) ]
			//--> sind die ersten zwei byte:
			try {
				int ANZAHL_LABELS = (short) ((intVec.get(0) << 8) | (intVec.get(0+1)) & 0xff) ;								
			
				int los=2;			
				//fuer JEDES LABEL:	
				for(int l=0; l<ANZAHL_LABELS; l++)  {
					//1: ERST IMMER LAENGE DES LABELS(16 bit):
					int laengeEinesLabels =  ( intVec.get(los) << 8 )  |  ( intVec.get(los+1)  & 0xff );
//					System.out.println("Labellaenge: " +  laengeEinesLabels);
			
					//2: DANN DESSEN SPEICHERADRESSE (32bit):
					int speicheradresse = BitUtils.fourBytesToOneInteger(
									intVec.get(los+2),
									intVec.get(los+3),
									intVec.get(los+4),
									intVec.get(los+5) 
								);
							
				
//				System.out.println("Speicheradresse: " +  speicheradresse);
				los += 6;

				//3: ASCII-Name (8 Bit,wenn ungerade Zeichenanzahl ein Nullbyte anhängen)
				String label = "";
				for (int k = los; k < (los+laengeEinesLabels); k++) {										
					label  +=  (char)intVec.get(k).intValue();
				}
//				System.out.println(label);
				//wenn Labellaenge ungerade: Nullbyte ignorieren!
				if(!(laengeEinesLabels%2 == 0)) 
					los += laengeEinesLabels+1;
				else
					los += laengeEinesLabels;
				
				//HASH MAP MIT LABELS FUELLEN: (dann abholen ueber getLabels() )
				this.labels.putLabelAt(speicheradresse, label);																	
			}	
		}
		catch(Exception e) {
			// dann gibts keine Labels....
		}
						
		}
		
		catch (Exception e) {
			System.err.println("ERROR WHILE TRYING TO READ THE FILE! --> ObjectfileReader.readUnsignedBytes()");
			e.printStackTrace();
			System.out.println("MESSAGE:" + e.getMessage());
			return;
		}	
	}
	
	/**
	 * This method reads in short per short(a 16 bit), calls getMemoryAndCalculateAddresses()
	 * and returns the read in memory.
	 * 
	 * @return SortedMap<Integer, Integer> memory
	 */
	public void readUnsignedShortsAndGetMemory() {	
		
		int intRead=0;
		
		try {
			Vector<Integer> shortsRead = new Vector<Integer>();
//			int []intArray = new int[1024];
			//int convertedIntegers [] = new int [1024];
			
			int count=0, size=dataInStream.available();
			//System.out.println("SIZE: " + size +" bzw. "+size/2 + "");
			
			//=================================================================================EINLESEN:
			intRead = dataInStream.readUnsignedShort(); // 2 byte einlesen!!!
			while(count < (size/2)+1 ) {
				
				shortsRead.add(intRead); //vector fuellen!!!
				
				try {
					intRead = dataInStream.readUnsignedShort(); //2 byte einlesen!!! 					
				}
				catch (java.io.EOFException e) {
					//System.err.println("REACHED THE END OF THE FILE! at readShorts()");									
				}
				count++;				
			}
			
			//#################################################ALLE SHORTS AUSGEBEN:
//			for (int j = 1; j <= count; j++) {
//				System.out.print(shortsRead.get(j-1) + "\t");	
//				if(j%8==0)
//					System.out.println();
//			}
			
			
			
			//Speicher holen und Adressen der HiCoVec Befehle berechnen
			SortedMap<Integer, Integer> mem = getMemoryAndCalculateAddresses(shortsRead);
			Object [] adds = mem.keySet().toArray();
			for(int i=0; i<adds.length; i++) {
				int add = new Integer((Integer)adds[i]);
				this.memory.writeTo(add, mem.get(add));
			}
		}
		
		catch (Exception e) {
			System.err.println("ERROR WHILE TRYING TO READ THE OBJECTFILE! --> ObjectfileReader.readUnsignedShortsAndGetMemory()");
			e.printStackTrace();
			System.out.println("MESSAGE:" + e.getMessage());
			return;
		}
	}
//================================================================================================CALC ADDRESSES AND GET MEMORY!:
	
	/**
	 * This method checks the sections, calculates the memory and the addresses,
	 * which are used in readUnsignedBytes().
	 * 
	 * Section-Types: (Code: 0x11, Labels: 0x12, Lines: 0x13,Breakpoints: 0x4, 
	 * Comments: 0x5, Info 0x6,Register: 0x17)	
	 * @return SortedMap<Integer, Integer> outMemory			
	 */
	private SortedMap<Integer, Integer> getMemoryAndCalculateAddresses(Vector<Integer> shortsRead) {
				
		SortedMap<Integer, Integer> outMemory = new TreeMap<Integer, Integer>();
		
		//ignore assembler version
//		int magnum = shortsRead.get(0);  // muss/sollte 61454 (0xf00e) sein!
		int sections=0;
		
//		System.out.println("checking Objectfile " + file.getAbsolutePath() +" ...");
		
		//ignore magicnumber(=Assembler Version)...
//		if (! (magnum == 61454 ) ) {
//			System.out.println("\nWrong magic nr...");
//			sections= shortsRead.get(2);
//		}
//		
//		else {
//			sections= shortsRead.get(2);
////			System.out.println("\nRIGTH MAGIC NR and " + sections + " Sections.");						
//		}
		sections= shortsRead.get(2);
		
		int idx=0, secType, secStartHi, secStart,  secEndHi, secEnd;						
		
		//GO THROUGH ALL SECTIONS!!!
//		System.out.println("checking sections. . .");
		for (int secNum=0; secNum<sections; secNum++) {
			//calculate correct secNum-startadress: one sectiondir-entry is 5*16Bit, therefor 5*SecNum and 3 for the header ... 
			 idx = 3 + (5 * secNum);
			 secType    = shortsRead.get(idx);
			 secStartHi = shortsRead.get(idx+1);
			 secStart   = shortsRead.get(idx+2);
			 secEndHi   = shortsRead.get(idx+3);
			 secEnd     = shortsRead.get(idx+4);
			 
			 //bring adress in high-word
			 secStart += (secStartHi << 16);
			 secEnd += (secEndHi << 16);
//			 System.out.println( "SectionType = " + secType + ", Start = " +secStart + ", End =  "+secEnd);
			 			 
			 int address, n, lineNo;			
			 
			 //CHECKING SEC_TYPES: IGNORING VISCY......
			 
//			 if (secType == 1) {   // 'Code 16 BIT VISCY'
//				 System.out.println("Code 16 BIT VISCY");
//				 address = shortsRead.get(secStart);
			 
	 /*else*/ if (secType ==  17 ){ // 'Code 32 BIT HICOVEC'  (17 = 0x11 !!!)
//				 System.out.println("Code 32 BIT HICOVEC");
				 address = shortsRead.get(secStart);
				 address = address << 16;
				 address += shortsRead.get(secStart+1);
				 n = secEnd - (secStart +1);
//		     	 print( "-> Code: 0x%04x .. 0x%04x" , (address, address + n - 1));
//				 System.out.println( "-> Code 32 bit HICOVEC: " + address +" ... " + (address+n-1) );
				 
				 //JEDE start/end-adresse speichern!!!
				 this.startAddressIndexesCode32.add(secStart*2); //also ab byteArray[secStart] gehts los fuer den HiCoVec!
				 this.endAddressIndexesCode32.add(secEnd*2+1);// +1 ?? // * 2 weil hier is laenge in 16 bit brauch aber fuer 8 !!!
//				 System.out.println("===>Stored start/end_address");
				 
				 int z = 0;
				 for (int i=1; i<n; i+=2) {		
					 outMemory.put(address+z , shortsRead.get(secStart + 1 + i) ); 						 					  
					 outMemory.put(address+z , (outMemory.get(address+z) << 16) ); 
					 outMemory.put(address+z ,  (outMemory.get(address+z)) + shortsRead.get(secStart + 2 + i) );					                               
					 z = z+1;
		      }
			//print "%04x" % objData[secStart + i]				                       
			}
			 else if (secType == 19) {                // 'Zeilen 32 BIT HICOVEC' 19 = 0x13
//				 System.out.println("Lines 32 BIT HICOVEC");
				 n = (secEnd - secStart + 1) / 2;
//				 System.out.println("-> HICOVEC-Line info: "+n+" source lines!");				 
				 
//				 for (int i=0; i<n; i++) {
//					 lineNo = shortsRead.get(secStart + 2 * i);
//					 address= shortsRead.get(secStart + 2 * i + 2);
//
//				 }
		        			        		
			 }
			 else if (secType == 23) {  // 23 == 0x17
//				 System.out.println("REGISTER SECTION ?");
				 
				 //hier steht ProgrammZaehler , die ersten 4 byte
				 this.startAddressProgCounter = (secStart*2);
//				 System.out.println("===>Stored start/end_address of Program Counter...");
			 }
	 
			 else if (secType == 18) {  // 18 == 0x12
//				 System.out.println("LABEL SECTION ?");								 
				 this.startAddressIndexLabels=(secStart*2);
				 this.endAddressIndexLabels=(secEnd*2+1); 
//				 System.out.println("===>Stored start/end_address of Labels...");
			 }			 
			 
			 else {
//		        	System.out.println("NOTHING/No Section?");
		        }
		 }//END FOR (ALL SECTIONS)
		
		return outMemory;
	}
	
	/**
	 * This method returns the read in programCounter.
	 * @return int programCounter
	 */
	public int getProgramCounter() {
		return this.programCounter;
	}
	/**
	 * This method returns the read in labels.
	 * @return
	 */
	public hicosim.components.backend.processor.Labels getLabels() {
		return this.labels;
	}
}