package hicosim.components.backend.io;

import hicosim.control.Engine;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.TreeMap;


/**
 * Class for reading in and writing to a sourcefile.
 * @author Wager, 16th May 2009
 */
public class Sourcefile {
	
//	read / write src file 
	private BufferedReader reader=null;
	private BufferedWriter writer=null;
	
	private TreeMap<Integer, Integer> lines_addresses=null;
	private TreeMap<Integer, Integer> addresses_lines=null;
	
	private int address=0;
	private boolean org=false;
	//to store the lines with linenumbers:
	private TreeMap<Integer, String> sourcecodeLines=null;
	//for string representation:
	private StringBuffer lines=null;	
	private String srcCodePATH=null;
	
	private int numberOfLines=0;
	
	private Engine engine=null;
	
	public Sourcefile(Engine e) {
		this.engine=e;
		sourcecodeLines = new TreeMap<Integer, String>();
		lines_addresses=new TreeMap<Integer, Integer>();
		addresses_lines=new TreeMap<Integer, Integer>();
	}
	public void reset() {
		this.sourcecodeLines = new TreeMap<Integer, String>();
		this.lines_addresses=new TreeMap<Integer, Integer>();
		this.addresses_lines=new TreeMap<Integer, Integer>();
		
		this.lines=null;
		this.address=0;
		org=false;
		this.srcCodePATH=null;
		this.numberOfLines=0;
	}
	/**
	 * This method reads in a sourcefile(only .asm) 
	 * and stores the lines in a stringbuffer;
	 * 
	 * @throws Exception
	 */
	public void read(String PATH) throws Exception {
		if(!PATH.contains(".asm"))
			throw new Exception("READ: FILE MUST END WITH .asm");
		
		srcCodePATH=PATH;
		
		// ! immer wenn neue gelesen wird die alte reseten:
		reset();
		
		try {
		this.reader = new BufferedReader(new InputStreamReader(
	 			new FileInputStream(						 		
	 					new File(PATH) )));
		}
		catch (FileNotFoundException e) {
			throw new FileNotFoundException("FILE NOT FOUND "+ PATH);
		}

		this.lines = new StringBuffer();
		String line=null;
		
		line = this.reader.readLine();
		this.lines.append(line  +"\n");
		numberOfLines++;
		this.putLine(numberOfLines, line);
		
		while(line != null ) {
			line = this.reader.readLine();
			if(line != null) {
				this.lines.append(line + "\n");
				numberOfLines++;
				this.putLine(numberOfLines, line);
			}
		}
		this.reader.close();
	}
	/**
	 * This method writes to a sourcefile(only .asm).
	 * 
	 * @throws Exception
	 */
	public void write(String PATH, String text) throws Exception{
		if(!PATH.contains(".asm"))
			throw new Exception("WRITE: FILE MUST END WITH .asm");
		
		this.writer = new BufferedWriter(
	 			new FileWriter(
	 					new File(PATH)));
		
		this.writer.write( text );
		this.writer.flush();
		this.writer.close();
	}
	/**
	 * @return String lines - all lines in the sourcefile.
	 */
	public String getLines() {
		if(this.lines==null)
			return null;
		return this.lines.toString();
	}
	/**
	 * @return int numberOfLines - number of lines in the sourcefile.
	 */
	public int getNumberOfLines() {
		return this.numberOfLines;
	}
	/**
	 * @return String srcCodePATH - path to the sourcefile.
	 */
	public String getPathOfSourcefile() {
		return this.srcCodePATH;
	}
	/**
	 * This method is used to put a line in a Map
	 * to get the line by the linenumber.
	 * @param linenumber - int
	 * @param line - String
	 */
	public void putLine(int linenumber, String line) {
		/* 
		 * Anhand der .org Derektiven werden die der Befehls-zeilen
		 * zugehoerigen Adressen gespeichert.
		 */
		
		//erstmal kommentare bzw alles irrelevante weg:
		char[] arr=line.toCharArray();
		line="";
		for (int i = 0; i < arr.length; i++) {
			//nur das mitnehmen wo kein ; vorher steht bw bis newLine kommt
			if (arr[i] == ';' || arr[i] == '\n' || arr[i] == '\r' )
				break;
			else
				line += arr[i];
		}		
		while(line.contains(" ") || line.contains("\t")) {
			line=line.replace(" ", "");
			line=line.replace("\t", "");
		}
		
		//ganz normal dann fuellen 
		if (!(line.equals("\t") || line.equals("")) )
			this.sourcecodeLines.put(linenumber, line);	
		else {
			lines_addresses.put(linenumber, -1);
			addresses_lines.put(-1, linenumber);
			return;
		}
		
		//ein .org in der zeile gibt die Startadresse 
		//der folgenden Codesektion an
		if(line.contains(".org")) {
			//.org und alle leerzeichen weg
			line=line.replace(".org ", "");line=line.replace(".org", "");
			while(line.contains(" ") || line.contains("\t")) {
				line=line.replace(" ", "");
				line=line.replace("\t", "");
			}
				//im scotchas kann man .org Adressen nur hex angeben:
				if(line.contains("0x"))
					line = line.replace("0x", "");
				else if(line.contains("0X"))
					line = line.replace("0X", "");
				else if(line.contains("$"))
					line = line.replace("$", "");
				
				org=true;
		
				//JETZT DIE ADDRESSE DER ZEILE HOLEN:
				address=Integer.parseInt(line, 16);
				lines_addresses.put(linenumber, -1); //alles was kein befehl ist, bekommt al Adresse -1
				return;
		}
		//wenn .start, "" --> -1 schreiben, da diese zeile dann keine adresse hat
		else if(line.contains(".start") || line.equals("") || line.equals("\t") ) {
			lines_addresses.put(linenumber, -1);
			addresses_lines.put(-1, linenumber);
			return;
		}
		
		//ein Label?
		if( line.contains(":")) { //zB LABEL2: ADD A, X, Y
			String [] lineArr = line.split(":"); //-> Array dann [LABEL2, ADD A, X, Y]
			if(lineArr.length==1) {
				//labelzeile bekommt auch adresse
				lines_addresses.put(linenumber, address);
				addresses_lines.put(address, linenumber);
				//DANACH ADRESSE NICHT ERHOEHEN, DA NUR LABEL!
				return;
			}
			else {
				lines_addresses.put(linenumber, address);
				addresses_lines.put(address, linenumber);
				//sonst adresse auch erhoehen! hier haben Befehl u Label die gleiche Adresse!
				address++;
				return;
				
			}
		}
		
		//sobald einmal ein .org gefunden wurde:
		//hier kommt man nur hin, wenn ein Befehl mit Adresse erkannt wurde
		if(org) {					
			lines_addresses.put(linenumber, address);
			addresses_lines.put(address, linenumber);
			address++;
		}
		else {
			lines_addresses.put(linenumber, -1);
			addresses_lines.put(-1, linenumber);
		}
			
	}
	
//	public void setBreakpointAtLine(int linenumber)throws Exception {
//		if(linenumber<=0 || linenumber>numberOfLines) 
//			throw new Exception("CANNOT SET A BREAKPOINT AT LINE " + linenumber + " BECAUSE THIS LINE DOES NOT EXIST IN SOURCEFILE " + this.srcCodePATH);
//		
//		for(int i=0; i<this.sourcecodeLines.size(); i++) {
//			
//		}
//		
//	}
	/**
	 * This method returns the address at a line.
	 * @param linenum
	 * @return
	 */
	public int getAddressAtLine(int linenum) {
		return this.lines_addresses.get(linenum);
	}
	/**
	 * This method returns the linenumber at a address.
	 * @return
	 */
	public int getLineNumberAtAddress(int address) {
		return this.addresses_lines.get(address);
	}
}