package hicosim.components.backend.processor;

import hicosim.control.Engine;
import java.util.Map;
import java.util.TreeMap;

/**
 * Class to disassemble hicovec instructions.
 * @author  Michael Wager, Luca Calchera, 16th May 2009
 */
public class Disassembler {

	private IMemory memoryI=null;
	
	private static Engine engine= null;
	
	private Map<Integer, String> disassembledInstructions=null;
	private TreeMap<Integer, Integer> addresses_lines=null;
	
	private int numberOfLines=0;
	
	public Disassembler( IMemory mem,Engine ei) {
		this.memoryI = mem;
		engine=ei;
		
		this.disassembledInstructions = new TreeMap<Integer, String>();
		this.addresses_lines=new TreeMap<Integer, Integer>();
	}
	
	/**
	 *  This method disassembles all read in hicovec instructions using 
	 *  the method "decodeInstruction()" from class processor stores them 
	 *  in a vector and returns them as string.
	 */
	public String disas() {
		
		numberOfLines=0; // !
		StringBuffer buf = new StringBuffer();

		//zuerst die Adressen holen an welche schon geschrieben wurde:
		Object [] adds = this.memoryI.getAllUsedAddresses().toArray();
		//alle Werte an den "benutzten" Adressen disassemblieren
		for(int i=0; i<adds.length; i++) {
			int add = new Integer((Integer)adds[i]);
		
			try { //AUCH LABELS ANZEIGEN WENN AN AKTUELLER ADDRESSE "add" VORHANDEN
				String lab = this.memoryI.getLabelAt(add);
				if(lab != null) {
					buf.append("    " + lab + ":\n");
					lab = null;
					numberOfLines++; //label bekommt immer eigene zeile also einfach numOfLines++
				}				
			}
			catch (Exception e) {
				//ignore
			}
			buf.append("        0x"+Integer.toHexString(add) + ": ");
			this.addresses_lines.put(add, numberOfLines);
			
			try {
				String disassembledInstr = engine.getProcessor().decodeInstruction(this.memoryI.readValueAt(add), true,false, add);
				
				putInstr(add, disassembledInstr);
				
				buf.append (disassembledInstr + "\n");	
				add++;
				numberOfLines++;
			}
			catch (Exception e) {
				engine.displayConsole("ERROR IN DISAS() " + e.getMessage() + " linenumber: " + numberOfLines, true);
				numberOfLines++;
			}
		}
		
		buf.append("\n");
		
	
		/* Plugins refreshen */
		this.memoryI.getPluginManager().reFreshPlugins();
		return buf.toString();
	}
	
	/**
	 *  This method stores the disassembled Instruction at the given address.
	 */ 
	public void putInstr(int add, String disasInstr) {
		disassembledInstructions.put(add, disasInstr);
	}
	
	/**
	 *  This method returns the disassembled Instruction at the given address.
	 */  
	public String getDisassembledInstructionAt(int address)throws Exception {
		return disassembledInstructions.get(address);
	}
	
	/**
	 *  This method disassembles one single Instruction at the given address.
	 */  
	public static String disasOneSingleInstruction(int add,int instr) throws Exception{
		String disassembledInstr = engine.getProcessor().decodeInstruction(instr, true,false, add);
		return disassembledInstr;
	}
	
	/**
	 * This method returns the linenumberindex at an address.
	 * @param - int address
	 * @return - int linenumberindex
	 */
	public int getLineOfAddress(int address) {
		return this.addresses_lines.get(address);
	}
}
