package hicosim.components.backend.processor;

import hicosim.control.ProcessorConfig;

import java.util.Vector;

/**
 * @author Andreas Weber, Wager, Calchera
 * @version 04-07-2009
 * 
 * Interface, welches vom Prozessor implementiert wird.
 * Sinn: Die Engine soll nur das Interface instanziieren, nicht den Prozessor. Alle Methoden in diesem Interface
 * stellen die Methoden da, mit denen die Engine auf das Hardwaresystem zugreifen kann.
 */

public interface IProcessor {

	public void setUp(IMemory memory, int progCounter);
	
	public boolean isObjectFileLoaded();
	public void setOFileNotLoaded();
	public void setIsRunning(boolean isRunning);
	public boolean isRunning();
	public void setStepping(boolean stepping) ;
	
	
	public void reset();
	
	public String getRegs();
	public void setIP(int newIP);
	public int getIP();
	
	public void step();
	
	public void deleteBreakpoint(int address);
	public void resetBr();
	public void setBreakpoint(int address) throws Exception;
	public Vector<Integer> getBreakpoints();
	
	public void interrupt() throws Exception;
	
	public void setScalarValue(char reg, int value);
	public void setVRegsValue(int i,int j,int value);
	public int getVRegsValue(int i,int j);
	public void setStatusRegs(boolean zero, int carry);
	
	public void setHALT(boolean HALT);
	public boolean isHALT();
	
	public void setConfig(ProcessorConfig newConfig);
		
	
	/* Abfragen der Prozessor-Konfiguration - Rückgabewert als Ganzes
	 * (z.B. Vector mit ['ADD'][3]) oder Befehle einzeln abfragbar, z.b.
	 * getStatsADD usw.?
	 * 
	 * edit Andi: Denke fuer stats wuerde sich auf jeden Fall auch eine eigene Klasse lohnen.
	 * Muessen v.a. darauf schauen dass man die Statistiken fuer neue Befehle erweiterbar
	 * halten
	 */
	public String getConfiguration();
	//public String getStats();
	
	public int getZeroFlag();
	public int getCarryFlag();
	
	public int[][]getVectorRegisters();
	public int getRegA();
	public int getRegX();
	public int getRegY();
}
