package hicosim.components.backend.processor;

import java.util.ArrayList;

import hicosim.components.backend.processor.IMemory;
import hicosim.components.plugins.IPlugin;


/**
 * The class PluginManager connects the hicovec-memory with plugins.
 * All plugins have to register themselves with the pluginmanager in order to get access to
 * 	memory. The pluginmanager is responsible to call the notify-method on
 * plugins. 
 * 
 * @author Weber
 * 
 */
public class PluginManager implements IPluginManager {

	
	private IMemory memory;
	
	ArrayList<Plugin> pluginList = new ArrayList<Plugin>();
	
	
	public PluginManager(IMemory newMemory){
		this.memory = newMemory;
	}
		
	
	public void addPlugin(IPlugin newPlugin, int startAt, int endAt){
	/**
	 * To deny access to special memory areas, adding can be disabled here for
	 * specified addresses
	 */
		if(startAt >= endAt){ 
			System.err.println("Wrong memory range! Plugin was not added!");
			return; }
		pluginList.add(new Plugin(newPlugin, startAt, endAt));
	}
	
	public void delPlugin(IPlugin oldPlugin){
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.equals(oldPlugin)){
				pluginList.remove(i);
			}
		}
	}
	
	public int pluginGetMemory(IPlugin plugin, int address) throws Exception {
		
		/* Check if plugin is authorized to access given memory address */
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.equals(plugin))			
			{	
				if((pluginList.get(i).startAt <= address) && (pluginList.get(i).endAt >= address)){
						return memory.readValueAt(address);
					} else {
						return -1;
					}
			}
		}
		System.err.println("Memory access not allowed!");
		return -1;
	}
	
	
	public void pluginSetMemory(IPlugin plugin, int address, int value){
		
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.equals(plugin))			
			{	
				if((pluginList.get(i).startAt <= address) && (pluginList.get(i).endAt >= address)){
						this.memory.writeTo(address, value);
					} else {
						System.err.println("Memory access not allowed!!");
					}
			}
		}
		
	}
	
	public void notifyPlugins(int adr, int value){
		
		/* Iterate the pluginlist.
		 * If plugin is an observer an adr is in the plugins observed memory
		 * range: call notify-method on plugin.
		 */
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.isObserver()){		
				if((pluginList.get(i).startAt <= adr) && (pluginList.get(i).endAt >= adr)){
					pluginList.get(i).plugin.notifyPlugin(adr, value);
				}
			}
		}
	}
	
	public void reFreshPlugins(){
		for(int i = 0; i < pluginList.size(); i++){
			pluginList.get(i).plugin.reFresh();
		}
	}
	
	
	public boolean checkPlugin(IPlugin plugin){
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.equals(plugin)){
				return true;
			}
		}
		return false;
	}
	
	public void setNewRange(IPlugin plugin, int newStart, int newEnd){
		for(int i = 0; i < pluginList.size(); i++){
			if(pluginList.get(i).plugin.equals(plugin)){
				pluginList.get(i).startAt = newStart;
				pluginList.get(i).endAt = newEnd;
			}
		}
	}
	
	
	
	
	
	/* Inner class for plugins */
	class Plugin {
		private IPlugin plugin;
		private int startAt;
		private int endAt;
		
		public Plugin(IPlugin plugin, int startAt, int endAt){
			this.plugin = plugin;
			this.startAt = startAt;
			this.endAt = endAt;
		}
	}
	
	
}
