package hicosim.components.backend.processor;

import hicosim.control.Engine;
import hicosim.control.ProcessorConfig;
import hicosim.control.utils.BitUtils;
import hicosim.control.utils.IStatistic;
import hicosim.control.utils.Numeratives;
import hicosim.components.backend.processor.enums.*;
import java.util.Vector;

/**
 * This class simulates the HICOVEC processor.
 * 
 * @author Michael Wager, Andreas Weber, Luca Calchera , 16th May 2009
 */
public class Processor implements hicosim.components.backend.processor.IProcessor  {
	
	/**
	 * Enumaration of a ScalarRegister.
	 */
	public enum ScalarRegister {
		A(0), X(0), Y(0),
		ssTmp(0),ttTmp(0),ddTmp(0);//temporary "register" 
		private int value;

		ScalarRegister(int value) {
			this.value = value;
		}
		public int getValue() {
			return value;
		}
		/**
		 * This method is used to set the value of a scalarRegister
		 * and show that change in GUI if GUI App is launched.
		 * @param value
		 */
		public void setValue(int value) {
			this.value = value;			
			//NUR CALLBACK, WENN EINS DER 3 ScalarRegs!(A,X,Y) veraendert wurde(nicht wenn zB sstmp), sonst einfach return
			String str=this.toString();
			if( (str.equals(ddTmp.toString())) || (str.equals(ssTmp.toString())) || (str.equals(ttTmp.toString())) )
				return;
			
			Processor.engine.checkNumerativeFlagAndDisplay(str.charAt(0), value );
			
//				Processor.engine.displayScalarRegChanges
//					(ScalarRegister.A.getValue(), ScalarRegister.X.getValue(), ScalarRegister.Y.getValue());
		}
	}	
	private IMemory memory=null;

	private int ip = 0; //program counter
	private int startIP = 0;
	
	
	private boolean zeroFlag=false;
	private int carryFlag=0; //carry flag auf int damit es direkt addieren kann

	private int y=0;//vshuf
    private int x=0;//vshuf

	private ProcessorConfig config;

	/* Vektorregister ist 2-Dim. int-Array */
	private int[][] vRegs;
   	
	private hicosim.control.utils.IStatistic stats;

	//BREAKPOINTS:
	private boolean breakpoint=false;
	
	private Vector<Integer> breakAddresses = null; //muss -1 sein! solange kein breakpoint gesetzt ist!

	private Decoder decoder=null;
	
	//MUSS PUBLIC static SEIN WEGEN ScalarReg enum
	public static Engine engine=null;

	private boolean isRunning=false;
	private boolean HALT=false;

	//um waehrend dem steppen die next auszufuehrenden Befehle anzeigen zu koennen
	private static boolean stepping=false;
	
    private boolean oFileLoaded=false;
    
    private int shufWidth;
    private int vectorWidth;
    private boolean isScalarMul;
    private boolean isVectorMul;
    
    /**
     * This method gets the config
     */
    private void getConfigValues(){
    	shufWidth=this.config.getShuffle_width();
    	vectorWidth=this.config.getVectorshift_width();
    	isScalarMul=this.config.isScalar_mul();
    	isVectorMul=this.config.isVector_mul();
    }
   
    /**
     * 
     * @param newConfig
     * @param engine1
     */
	public Processor(ProcessorConfig newConfig, Engine engine1){
		this.config = newConfig;
		this.getConfigValues();
		this.decoder = new Decoder();
		
		this.oFileLoaded=false;
		this.breakAddresses=new Vector<Integer>();

		this.stats = new hicosim.control.utils.Statistics();
		engine = engine1;
		
		/* Vector-Register initialisieren */ 
		//nicht erst in setUp denn es kann sein dass man vor Laden des oFiles schon regs gesetzt haben will			
		vRegs = new int[config.getN()][config.getK()];
		for(int N=0; N<config.getN(); N++) {						
			for(int K=0; K<config.getK() ; K++) {
				this.setVRegsValue(N, K, 0);
			}
		}
	}
	
	/**
	 * Everytime when a new objectfile was loaded, call setUp()
	 */
	public void setUp(IMemory memory, int progCounter) {
		this.memory = memory;
		this.startIP = progCounter; //fuer reset
		
		this.oFileLoaded=true;		
		
		//hier disassemble einmalig:
		engine.getMemory().getDisassembler().disas();
		
		this.reset();
	}

	/**
	 * This method resets the progCounter, all breakpoints, registers and the statistics. 
	 */
	public void reset() {
//		reset instruction pointer:
//		System.out.println("Prozessorreset!!!--> call setIP(startIP)");
		this.setIP(this.startIP);
		this.isRunning = false; 
		
		this.getConfigValues();
		
		//reset flags!:
		this.zeroFlag=false;
		this.carryFlag=0;

		//reset counters !!!
		this.stats.reset();
		
		stepping=false;
		this.HALT=false;

		/* Register auf 0 setzen */
		ScalarRegister.A.setValue(0);
		ScalarRegister.X.setValue(0);
		ScalarRegister.Y.setValue(0);
		/* Vector-Register reset */			
		vRegs = new int[config.getN()][config.getK()];
		for(int N=0; N<config.getN(); N++) {						
			for(int K=0; K<config.getK() ; K++) {
				this.setVRegsValue(N, K, 0);
			}
		}
		this.setStatusRegs(false, 0);
		
		if(engine.getListener()!=null) {
			engine.displayScalarRegChanges(getRegA(), getRegX(), getRegY());
			engine.displayInstructionPointer(ip);
			engine.displayStatusRegister(getCarryFlag(), getZeroFlag()==0 ? false:true);
			engine.displayVectorRegChanges(getVectorRegisters());
		}
	}

	public IStatistic getStatistics(){
		return this.stats;
	}
		
	
	/**
	 * This method is used so set a value in the vectorregister
	 * and show that change in GUI if GUI-App is launched.
	 * @param i
	 * @param j
	 * @param value
	 */
    public void setVRegsValue(int i,int j,int value){
	    	try {
	    		this.vRegs[i][j]=value;
//	    		if(Processor.stepping) {
	    			if(engine.getListener()!=null) {
	    				engine.displaySingleVecValue(i, j, value);	
	    		}
	    	}
	    	catch (Exception e) {
				engine.displayConsole("ERROR WHILE TRYING TO WRITE TO VECTOR REGISTER R" + i + " , AT " + j +"\nStopping processor.", true);
//				this.reset();
				this.isRunning=false;
			}
	    }
    /**
     * This method is used to get the content of a vecReg.
     * @param i
     * @param j
     * @return - int  VRegsValue at (i,j)
     */
    public int getVRegsValue(int i,int j){
	   try {
	   		return this.vRegs[i][j];
	   	}
	   	catch (Exception e) {
			engine.displayConsole("ERROR WHILE TRYING TO READ VECTOR REGISTER R" + i + " , AT " + j + "\nStopping processor.", true);
//			this.reset();
			this.isRunning=false;
			return -1;
		}
	}
    /**
     * To set the new value of the statusregister.
     * @param zero
     * @param carry
     */
	public void setStatusRegs(boolean zero, int carry) {
		this.zeroFlag=zero;
	 	this.carryFlag=carry;
//	 	if(Processor.stepping)
	 		engine.displayStatusRegister(carry, zero);
	}
	public void setIP(int newIP){
//		System.out.println("SET NEW IP !!!!!!!!! new: " + newIP);
		this.ip = newIP;
//		engine.checkNumerativeFlagAndDisplay('i', newIP);
		if(engine.getListener()!=null /*&& stepping*/)
			engine.getListener().displayInstructionPointer(this.ip);
	}
	 public void incIP(){
//		 System.out.println("INC IP !!!!!!!!! aktuell: " + ip);
	    	this.ip++;
//			engine.checkNumerativeFlagAndDisplay('i', newIP);
	    	if(engine.getListener()!=null /*&& stepping */)
				engine.getListener().displayInstructionPointer(this.ip);
	 }
	public void setScalarValue(char reg, int value) {
		switch (reg) {
		case 'A':
			ScalarRegister.A.setValue(value);
			break;
		case 'X':
			ScalarRegister.X.setValue(value);
			break;
		case 'Y':
			ScalarRegister.Y.setValue(value);
			break;
		}
	}
	    
	public void setIsRunning(boolean isRunning){
    	this.isRunning=isRunning;
    }
	public boolean isRunning() {
		return this.isRunning;
	}

    public void setOFileNotLoaded(){
    	this.oFileLoaded=false;
    }
    public boolean isObjectFileLoaded(){
    	return this.oFileLoaded;
    }
	public void setConfig(ProcessorConfig newConfig) {
		this.config = newConfig;
		vRegs = new int[config.getN()][config.getK()];
		for(int N=0; N<config.getN(); N++) {						
			for(int K=0; K<config.getK() ; K++) {
				this.setVRegsValue(N, K, 0);
			}
		}
	}
	public void setStepping(boolean stepping) {
		this.stepping = stepping;
	}

	public void interrupt() throws Exception {
		this.isRunning=false;
	}

	/**
	 * String representation of the Registers.
	 */
	public String getRegs() {
		String ret = "REG / DECIMAL / HEX / BINARY\n";
		ret += "A:\t "+ ScalarRegister.A.getValue() + "\t" +  "0x" + Integer.toHexString(ScalarRegister.A.getValue()) +
		"\t" + BitUtils.convert(new Integer(ScalarRegister.A.getValue()), hicosim.control.utils.Numeratives.BINARY, true) + "\n";
		ret += "X:\t "+ ScalarRegister.X.getValue() + "\t" +  "0x" +Integer.toHexString(ScalarRegister.X.getValue()) +
		"\t" + BitUtils.convert(new Integer(ScalarRegister.X.getValue()), hicosim.control.utils.Numeratives.BINARY, true) + "\n";
		ret += "Y:\t "+ ScalarRegister.Y.getValue() + "\t" +  "0x" +Integer.toHexString(ScalarRegister.Y.getValue()) +
		"\t" + BitUtils.convert(new Integer(ScalarRegister.Y.getValue()), hicosim.control.utils.Numeratives.BINARY, true) + "\n";
		//VECTOR REGS:
		ret += "VECTOR_REGISTERS: \n";
		
		for(int i=0; i<config.getN(); i++) {
			ret += "R"+i;
			 ret+=" ";
			ret +=":[";
			for(int j=0; j<config.getK(); j++) {
				ret += this.getVRegsValue(i, j);
				if(j < config.getK()-1)
					ret += ",";
			}
			ret += "]\n";			
		}
		ret += "\nIP: " + this.ip + "\n";
		return ret;
	}
	public int getRegA(){
		return ScalarRegister.A.getValue();
	}
	public int getRegX(){
		return ScalarRegister.X.getValue();
	}
	public int getRegY(){
		return ScalarRegister.Y.getValue();
	}
	public int[][]getVectorRegisters() {
		return this.vRegs;
	}
	public int getZeroFlag(){
		return zeroFlag ? 1 : 0;
	}
	public int getCarryFlag(){
		return carryFlag;
	}
	public int getIP(){
		return this.ip;
	}
	public Vector<Integer> getBreakpoints() { 
		return this.breakAddresses;
	}
	/**
	 * this method deletes ALL breakpoints
	 */
	public void resetBr(){
		this.breakAddresses=new Vector<Integer>();
	}
	/**
	 * this method deletes a breakpoint
	 */
	public void deleteBreakpoint(int address){
		for (int i=0;i<this.breakAddresses.size();i++){
			if (address==this.breakAddresses.get(i)){
				this.breakAddresses.set(i, -1);
			}
		}
	}
	public void setHALT(boolean HALT) {
		this.HALT = HALT;
	}
	public boolean isHALT() {
		return this.HALT;
	}
	
	public String getConfiguration(){
		return this.config.toString();
	}
	
	/**
	 * setBreakpoint - only if the address is available/used in the instructionRegister
	 */
	public void setBreakpoint(int address)throws Exception {
		this.breakAddresses.add(address);		
	}
	/**
	 * This method is used to check if a breakpoint was set.
	 * @return boolean
	 */
	private boolean checkBreakpoints() {
		for (int i=0; i<this.breakAddresses.size(); i++) {
			if(this.breakAddresses.get(i) == this.ip)
				return true;				
		}
		return false;
	}
//	===================================================================================ABLAUF EINES BEFEHLS:
	
	/**
	 * execute one single instruction
	 */
	public void step() {
		this.fetchAndDecodeInstruction();//macht jetzt EINE instruction!
	}
	
	/**
	 * This method is used to fetch the instructions from the instrRegister,
	 * decodes and executes it. It also checks breakpoints.
	 * 
	 */
	private void fetchAndDecodeInstruction() {
		
		 if ( this.checkBreakpoints() && !this.breakpoint) {							    	  
	    	  this.breakpoint=true;
		      this.isRunning = false;
		      String disas = null;
		      try {
		    	   disas = this.memory.getDisassembler().getDisassembledInstructionAt(this.ip);	  
		      }
		      catch (Exception e) {
				disas = "-> 0x" + Integer.toHexString(this.memory.readValueAt(ip));
			}
		  
//		     //Highlight lines
		      try {
		    	  if(engine.getSourcefile()!=null) 
		    		  engine.highlightSourcecodeline(engine.getSourcefile().getLineNumberAtAddress(ip)-1);
		      }catch (Exception e) {/*ignore if fails*/}
		      try {
		    	  engine.highlightDisascodeline(engine.getMemory().getDisassembler().getLineOfAddress(ip));
		      }catch (Exception e) {/*ignore if fails*/}
		     
		      
		      engine.displayConsole( ("### Stopped at address 0x" + Integer.toHexString(this.ip) +
						", next instruction to execute: "+ disas), false);
		      

		      return;
		 }
		 else {
			 try {						
					int instruction = this.memory.readValueAt(this.ip);
					this.decodeInstruction(instruction,false,stepping,this.ip);
					this.breakpoint=false; //-> jedes Mal wenn decoded wurde					
				}
				catch (Exception e) {
					engine.displayConsole("REACHED UNINITIALISED MEMORY OR UNNOWN INSTRUCTION AT 0x"+Integer.toHexString(ip)+". STOPPING PROCESSOR.\n" + e.getMessage()  , true);
					this.isRunning = false;
					engine.setProcessorRunning(false);
					return;
				}
		 }
	}

// ##################### DECODING #######################################################################################
	/**
	 * This method decodes the given 32 bit instruction "instr" and executes the decoded command.
	 * If this method is used by the Disassembler, it just returns the disassembled 
	 * string-representation of the instruction.
	 * 
	 * @param instr			- the instruction to decode
	 * @param isdisas		- because this method is used by the disassembler, and the processor
	 * @param stepping		- if breakpoints are set, the user should see the executed instructions
	 * @param progCounter	- programCounter(because this is used also by Disassembler)
	 * @return				- String if used by the Disassembler
	 * @throws Exception
	 */
	public String decodeInstruction(int instr,boolean isdisas,boolean stepping, int progCounter) throws Exception {
		String kooperationsbefehl=null;
		boolean coop=false;
		
		int dd=0;
		int ss=0;
		int tt=0;
		ScalarRegister targetReg = null; // Zielregister
		ScalarRegister sourceReg1 = null; // Quellregister 1
		ScalarRegister sourceReg2 = null; // Quellregister 2

//		################################### 1. SKALAR BEFEHLE: ###################################
  
		/* ALS ALLER ERSTES: Bei HALT-Befehl: Sofort abbruch */
		//		###00101... ############################################################################################## HALT:
		if(!BitUtils.testBit(instr, 31)  && !BitUtils.testBit(instr , 30)&& 
				BitUtils.testBit(instr, 29) && !BitUtils.testBit(instr, 28) && BitUtils.testBit(instr, 27)) {
			if (isdisas)
				return ("HALT");
			engine.displayConsole("HALT instruction reached. Stopping processor." , false);
			
			engine.displayInstructionPointer(engine.getIP());
			engine.displayStatusRegister(engine.getProcessor().getCarryFlag(), engine.getProcessor().getZeroFlag()==0 ? false:true);
			
			engine.checkNumerativeFlagAndDisplayAll();
			
			setIsRunning(false);
			setHALT(true);
			return "";
		}

		//DANN DIE ERSTEN 2 BITS CHECKEN:
		int commandType = instr>>>30;

		int command = instr << 2; //erst type weg, dann den rest:
		command = command >>> 28;

		//UND DIE CODIERTEN SKALARREGISTER:
		dd = instr << 6;
		dd = dd >>> 30;
		targetReg = this.decoder.decipherScalarRegister(dd);

		ss = instr << 8;
		ss = ss >>> 30;
		sourceReg1 = this.decoder.decipherScalarRegister(ss);

		tt = instr << 10;
		tt = tt >>> 30;
		sourceReg2 = this.decoder.decipherScalarRegister(tt);
		
		int vecCommandType = (instr>>>18) & 3; //3==11 - bit 18/19 von rechts bei 0 beginnend-->  00000000 00001100 00000000 00000000
		
		/**
		 * --------------------------------------- HIER MOV UND MOVA DECODIEREN UM KONFLIKTE ZU VERMEIDEN!
		 */
		int maybeMOV = instr >>> 26;
		if(maybeMOV == 46) { //   "101110"  MOV1: Kopiere aus Skalar- in Vektoreinheit
		
			int reg= instr << 20;  //DOKUMENTATION RICHTIG !
			reg = reg >>> 28;
            if (isdisas) {
//            	return  ("--->MOV1: Kopiere aus Skalar- in Vektoreinheit -") + ("INHALT VON REGISTER "+ decipherScalarRegister(ss) 
//            			+" soll an Wort "+decipherScalarRegister(tt)  +" in R" + reg + " KOPIERT WERDEN..");
            	return "MOV R"+reg + "("+sourceReg2+"), " + sourceReg1; 
            }
            if(stepping)
            	engine.displayConsole( "MOV R"+reg + "("+sourceReg2+"), " + sourceReg1, false);
            
            int val = 0;
            if(sourceReg1 != null) 
            {
            	val = sourceReg1.getValue();
            }
            
            setVRegsValue(reg, val, sourceReg2.getValue() ); // changed order of tt and ss, for new move
            
            /* DONK: no evil usage of excetions!
            try { 
                setVRegsValue(reg, val, sourceReg2.getValue() ); // changed order of tt and ss, for new move 
            }
            catch (Exception e) {
            	//ignore, already displays errormessage
			}*/
			
			this.incIP();
			this.stats.updateStatistics("Mov");
			return "";
		}
		else if(maybeMOV == 47) { //   "101111ss--tt" MOV2: Kopiere aus Vektor- in Skalareinheit
			
//			System.out.println(this.bitUtil.convert(new Integer(instr), Numeratives.BINARY, true));
						
			int reg= instr << 24;	  //DOKUMENTATION FALSCH !?
			reg = reg >>> 28;
            if (isdisas) {
//            	return "MOV2: Kopiere aus Vektor- in Skalareinheit " + 
//            	("Wort "+decipherScalarRegister(tt)+ " in REGISTER R"+reg+ " SOLL IN REGISTER " + 
//            			decipherScalarRegister(dd) +" KOPIERT WERDEN..");
            	return "MOV " + targetReg + ", R" +reg+"("+ sourceReg2 + ")";
            }
            if(stepping)
            	engine.displayConsole( "MOV " + targetReg + ", R" +reg+"("+sourceReg2 + ")", false);             		            				

	    int val = 0;
            if(sourceReg2 != null) 
            {
            	val = sourceReg2.getValue();
            }
            
            targetReg.setValue(this.getVRegsValue(reg, val)); // changed order of tt and ss, for new move
			
	    /* DONK: no evil usage of excetions! 
            try {
            	targetReg.setValue(this.getVRegsValue(reg, sourceReg2.getValue()));
            }
            catch (Exception e) {
            	//ignore, already displays errormessage
			}*/
            
			this.incIP();
			this.stats.updateStatistics("Mov");
			
			return "";
		}
		else if(maybeMOV == 45) { //   "101101"   MOVA: Ein SkalarReg in jedes Wort von einem vecReg kopieren

			int reg = instr << 20;
			reg = reg >>> 28;
			
			if (isdisas) {
				if(sourceReg1==null) 
					return ("MOVA R"+reg + ", 0" );
				else
					return ("MOVA R"+reg + ", " + sourceReg1);				
			}	
			
			if(stepping)
				engine.displayConsole(this.memory.getDisassembler().getDisassembledInstructionAt(progCounter), false);

			try {
				for (int i=0; i<config.getK(); i++) {
					if(sourceReg2==null)
						this.setVRegsValue(reg,i,0);
					else
						this.setVRegsValue(reg,i,sourceReg2.getValue());
				}
            }
            catch (Exception e) {
            	//ignore, already displays errormessage
			}
			
			this.incIP();
			this.stats.updateStatistics("Mova");
			return "";
		}
		/**
		 * DANN HIER VMOV / VMOR / VMOL   (------------ 001 .....)
		 */
	
		int maybeVMOV = (instr>>>17) & 0x7; //     -> 00000000 00001110 00000000 00000000
		int VMOV_TYPE = (instr>>>12) & 0xF;  //    -> 00000000 00000000 11110000 00000000
		
		if(maybeVMOV == 1) { //koennte jetzt VMOV, VMOL oder VMOR sein
			
			int rrrr = (instr>>>8) & 0xF; //Ziel   & 00000000 00000000 00001111 vvvvwwww
			int vvvv = (instr>>>4) & 0xF; //Quelle  & 00000000 00000000 0000rrrr 1111wwww
			
			if(VMOV_TYPE==1) { //1.VMOV '0001'
				
				if (isdisas)
					return ("VMOV R" + rrrr + ", R"+vvvv );
				
				if(stepping)
					engine.displayConsole("VMOV R" + rrrr + ", R"+vvvv , false );
				
				 try {
					 for(int i=0; i<config.getK(); i++) {
							this.setVRegsValue(rrrr,i, this.getVRegsValue(vvvv,i)); 
						}
		            }
		            catch (Exception e) {
		            	//ignore, already displays errormessage
				}		       
				
				this.incIP();
				this.stats.updateStatistics("Vmov");
				return ""; 
			}
			else if(VMOV_TYPE==2) { //2.VMOV '0010'   --> 8 bit fuer direktwert in 12 skalarBits
				
				int srcReg = (instr>>>20) & 0xFF; //8 bit Direktwert 000-nnnnnnnn ...
//				int srcReg = (instr << 4) >>> 24;
				
				if (isdisas)
					return ("VMOV R"+rrrr + ", R<" + srcReg + ">" );
				
				if(stepping)
					engine.displayConsole("VMOV R"+rrrr + ", R<" + srcReg + ">" , false );
				
				 try {
					 for(int i=0; i<config.getK(); i++) {
							this.setVRegsValue(rrrr,i,this.getVRegsValue(srcReg,i)); 
						}
		            }
		            catch (Exception e) {
		            	//ignore, already displays errormessage
					}
								
				this.incIP();
				this.stats.updateStatistics("Vmov");
				return "";
			}
			else if(VMOV_TYPE==3) { //3.VMOV '0011'   --> 8 bit fuer direktwert in 12 skalarBits
				
				int destReg = (instr>>>20) & 0xFF;  //8 bit Direktwert 000-nnnnnnnn ...
				
				if (isdisas) 
					return ("VMOV R<"+destReg + ">, R" + vvvv );
				
				if(stepping)
					engine.displayConsole("VMOV R<"+destReg + ">, R" + vvvv , false);
				
				try {
					for(int i=0; i<config.getK(); i++) {
						this.setVRegsValue(destReg,i,this.getVRegsValue(vvvv,i)); 
					}
				}
				catch (Exception e) {
					//ignore, already displays errormessage
				}
				
				this.incIP();
				this.stats.updateStatistics("Vmov");
				return "";
			}
			else if(VMOV_TYPE==8) { // VMOL '1000' 00000000 00000010 10000000 00000000				
				if (isdisas)
					return ("VMOL R"+rrrr+", R"+ vvvv);	
				
				if(stepping)
					engine.displayConsole("VMOL R"+rrrr+", R"+ vvvv, false);
				
				try {
					int val=0, tmp=0, store=0;
					for(int i=0; i<config.getK(); i++) {
						val = this.getVRegsValue(vvvv, i);
						
						if(i==0) {								
							store = this.getVRegsValue(vvvv, config.getK()-1);
							this.setVRegsValue(rrrr, config.getK()-1, val); //nachlinks schieben, erstes faellt raus und kommt hier hinten(K-1) rein
						}
						else {
							if(i==(config.getK()-1))
								this.setVRegsValue(rrrr, tmp, store);
							else
								this.setVRegsValue(rrrr, tmp,val );
							tmp++;							
						}												 
					}
				}
				catch (Exception e) {
					//ignore, already displays errormessage
				}
				
				this.incIP();
				this.stats.updateStatistics("Vmol");
				return "";
			}
			else if(VMOV_TYPE==12) { // VMOR '1100' 00000000 00000010 11000001 00000000
				if (isdisas)
					return ("VMOR R"+rrrr+", R"+ vvvv);	
				
				if(stepping)
					engine.displayConsole("VMOR R"+rrrr+", R"+ vvvv, false);				
				
				try {
					int K = config.getK();
					int val=0, tmp=K-1, store=0;
					for(int i=K-1; i>=0; i--) {
						val = this.getVRegsValue(vvvv, i);
						
						if(i==K-1) {								
							store = this.getVRegsValue(vvvv, 0);
							this.setVRegsValue(rrrr, 0, val); //nachrechts schieben, letztes faellt raus und kommt hier vorne(0)rein
						}
						else {
							if(i==0) //dann is tmp=1
								this.setVRegsValue(rrrr, tmp, store);
							else
								this.setVRegsValue(rrrr, tmp,val );
							tmp--;							
						}												 
					}
				}
				catch (Exception e) {
					//ignore, already displays errormessage
				}
				
				this.incIP();
				this.stats.updateStatistics("Vmor");
				return "";
			}
		}
		
		switch (this.decoder.decipherType(commandType)) {

	    case LOAD_STORE: //HIER AUCH VEKTOR LOAD_STORE!
	    	command = instr>>>28;
	    	//--------------------------------------------------------------------------------DISAS: LOAD STORE VLOAD VSTORE
	    	int directly = instr & 0x1ffff;
	    	if (isdisas) {
	    		switch(this.decoder.decipherLOAD_STORECommand(command)) {
	    			case LOAD:
	    				if(this.memory.getLabelAt(directly)!=null && sourceReg1==null)  
    						return ("LD " + targetReg + ", [0 + "+  this.memory.getLabelAt(directly) + "]");
	    				
    					else if (this.memory.getLabelAt(directly)!=null && sourceReg1!=null) 
    						return ("LD " + targetReg + ", [" +sourceReg1 +  " + "+ this.memory.getLabelAt(directly) + "]");
	    				
    					else if(sourceReg1==null && sourceReg2!=null)
	    					return ("LD " + targetReg + ", [0 + "+  sourceReg2 + "]");
	    				else if (sourceReg1!=null && sourceReg2==null)
	    					return ("LD " + targetReg + ", ["+sourceReg1 + " + " +  directly +  "]");	    					    						    							    					
	    				else if(sourceReg1!=null && sourceReg2!=null)
	    						return ("LD " + targetReg + ", ["+sourceReg1 + " + " + sourceReg2 +  "]");
	    				else 
	    					return ("LD " + targetReg + ", [0 + "+ directly + "]");
	    				
	    			case STORE:
	    				directly = instr & 0x1ffff;
	    				
	    				if(this.memory.getLabelAt(directly)!=null && sourceReg1==null)  
	    					return ("ST [0 + " +  this.memory.getLabelAt(directly) + "], A");
	    				
    					else if (this.memory.getLabelAt(directly)!=null && sourceReg1!=null) 
    						return ("ST [" +sourceReg1 +  " + "+ this.memory.getLabelAt(directly)+ "], A");
	    				
    					else if(sourceReg1==null && sourceReg2!=null)
    						return ("ST [0 + " +  sourceReg2 + "], A");
	    				else if (sourceReg1!=null && sourceReg2==null)
	    					return ("ST ["+sourceReg1 + " + " +directly+ "], A");	    					    						    							    					
	    				else if(sourceReg1!=null && sourceReg2!=null)
	    					return ("ST ["+sourceReg1 + " + " +sourceReg2+ "], A");
	    				else 
	    					return ("ST [0 + " +directly+ "], A");
	    					    					    			
	    			case VLOAD:	    				
	    				int rrrr =  (instr>>>8) & 0xF; //Ziel   & 00000000 00000000 00001111 vvvvwwww
	    				if(sourceReg1==null && sourceReg2!=null) 
	    					return ("VLD R"+rrrr+ ", [0 + " +  sourceReg2 + "]");	    					
	    				else if(sourceReg1!=null && sourceReg2!=null) 
		    				return ("VLD R"+rrrr+ ", ["+sourceReg1 + " + "+ sourceReg2 + "]");	    						    			
		    			
	    			case VSTORE:
	    				int vvvv = (instr>>>4) & 0xF; //Quelle  & 00000000 00000000 0000rrrr 1111wwww
	    				if(sourceReg1==null && sourceReg2!=null) 
	    					return ("VST [0 + " +  sourceReg2 + "], R"+vvvv);	    					
	    				else if(sourceReg1!=null && sourceReg2!=null) 
	    					return ("VST ["+sourceReg1 + " + "+ sourceReg2 + "], R"+vvvv);		    				
	    		}
	    		
	    	}
	    	
	    	//--------------------------------------------------------------------------------DISAS
	    	
	    	if(stepping)
	    		engine.displayConsole(this.memory.getDisassembler().getDisassembledInstructionAt(progCounter), false);

			//MUSS IN RICHTIGER REIHENFOLGE UBERG. WERDEN!!!
			this.executeLOAD_STOREOperation(targetReg, sourceReg1, sourceReg2, this.decoder.decipherLOAD_STORECommand(command), instr);
			this.incIP();
			return null; //kann kein vecOp mehr sein
		case ALU:
            if (isdisas) {
            	/*
            	 * WENN DER VEC-TYPE = 1 DANN IST EINE VEC-ALU INSTRUCTION
            	 * EBENFALLS IN DIESEM 32 BIT BEFEHL, DARUM BEIM DISASSEMBLIEREN
            	 * NUR EINEN STRING NUTZEN WELCHER DANN IM CASE VEC-ALU ERWEITERT
            	 * UND DORT RETURNT WIRD. BEI NORMALER AUSFUEHRUNG(isdisas=false) WIRD
            	 * DER VEC-ALU BEFEHL EINFACH NACH DIESEM HIER AUSGEFUEHRT.
            	 * ALSO IN "EINEM TAKT".
            	 */            	
            	if(vecCommandType==1) { //dann steht ein Kooperationsbefehl auch mit drin!!!
            		int directValue = instr & 0x1ffff;
            		try {
            		if(this.decoder.decipherALUCommand(command).toString().equals("INC"))
            			return(this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg)) + ", " + (sourceReg1==null?"0":sourceReg1);
            		}catch (Exception e) {
						return "NONE";
					}
            		//wird dann beim decoden des VALU Befehls genutzt            		
            		if(sourceReg1!=null && sourceReg2!=null)
            			kooperationsbefehl = (this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + sourceReg1 + ", " + sourceReg2); 
            		else if(sourceReg1!=null &&sourceReg2==null)
            			kooperationsbefehl = (this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + sourceReg1 + ", " + directValue);
            		else if(sourceReg1==null && sourceReg2!=null)
            			kooperationsbefehl = (this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " + sourceReg2);
            		else {
            			if(this.memory.getLabelAt(directValue)!=null)
            				kooperationsbefehl = (this.decoder.decipherALUCommand(command)) +" "+ ( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " + this.memory.getLabelAt(directValue));
            			else{//DIREKTWERTFALL!            				
            				kooperationsbefehl = (this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " +  directValue);
            			}            				
            		}
            		 kooperationsbefehl += " # ";
            		 break; //!!!
            	}
            	else { //sonst kein koop-befehl, also nur den ALU befehl disassembliert und return 
            		int directValue = instr & 0x1ffff;
            		if(this.decoder.decipherALUCommand(command).toString().equals("INC"))
            			return(this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg)) + ", " + (sourceReg1==null?"0":sourceReg1);
            		
            		if(sourceReg1!=null && sourceReg2!=null)
            			return(this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + sourceReg1 + ", " + sourceReg2); 
            		else if(sourceReg1!=null &&sourceReg2==null)
            			return(this.decoder.decipherALUCommand(command)) +" "+ ( (targetReg==null?" 0":targetReg) + ", " + sourceReg1 + ", " + directValue);
            		else if(sourceReg1==null && sourceReg2!=null)
            			return(this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " + sourceReg2);
            		else {
            			if(this.memory.getLabelAt(directValue)!=null)
            				return (this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " + this.memory.getLabelAt(directValue));
            			else { //DIREKTWERTFALL!            				
            				return(this.decoder.decipherALUCommand(command)) + " "+( (targetReg==null?" 0":targetReg) + ", " + 0 + ", " + directValue);
            			}            			
            		}
            	}
            }             
            
			if(stepping)
				engine.displayConsole(this.memory.getDisassembler().getDisassembledInstructionAt(progCounter), false); 
			
			if(vecCommandType==1) 
				coop = true;

			//MUSS IN RICHTIGER REIHENFOLGE UBERG. WERDEN!!!
			this.switchScalarALUOperation(instr, targetReg, sourceReg1, sourceReg2, this.decoder.decipherALUCommand(command));
			
			if(!coop) //ip nur erhoehen wenn kein kooperationsbefehl, sonst wird er nach VALU auch erhoeht
				this.incIP();

			break;
		case JMP_NOP:

			/*
			 * HIER KOENNTEN AUCH VEKTORBEFEHLE SEIN: zB: "VADD.DW R2, R1, R0": 000000000000 01100000001000010000
			 */
			if(vecCommandType != 0) { 
				break;
			}
			
			int nop = instr>>>29;
			int vnop = (instr>>>17) & 7; // & 00000000 00001110 00000000 00000000
            if (isdisas){
            	if (nop==0 && command==0)
        			return "NOP";
            	if(vnop==0 && command==0) 
            		return "VNOP";
            	if( ! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JMP")&& //Bei JMP und JAL ist auch 000.. am Anfang!
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JNZ") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JZ") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JNC") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JC"))  {
            		
            		if (nop==0)
            			return "NOP";
                	if(vnop==0) 
                		return "VNOP";
//                	if(decipherJMP_NOP_DCCommand(command).toString().equals("DC")) //DC GIBTS NICHT, BZW HIER NICHT RELEVANT
//        				return "DC " + memoryI.readValueAt(add);
            	}
            	 	
//            	System.out.println(this.bitUtil.convert(new Integer(instr), Numeratives.BINARY, true));
            	
            	int directValue = instr & 0x1ffff;
            	String test = this.decoder.decipherJMP_NOPCommand(command).toString();
            	
            	if(test.equals("JMP") && targetReg!=null) {//######### --> JAL            		
            		if(this.memory.getLabelAt(directValue) != null && sourceReg1!=null)
            			return "JAL " + targetReg + ", ["+sourceReg1 + " + "+ this.memory.getLabelAt(directValue) + "]";
            		else if(this.memory.getLabelAt(directValue) != null && sourceReg1==null)
            			return "JAL " + targetReg + ", [0 + "+ this.memory.getLabelAt(directValue) + "]";            		
            		else if(sourceReg1==null && sourceReg2!=null)
            			return "JAL " + targetReg + ", [0 + " + sourceReg2 + "]";  
            		else if(sourceReg1!=null && sourceReg2==null)
            			return "JAL " + targetReg + ", ["+sourceReg1 + " + " +directValue + "]";
            		else if(sourceReg1!=null && sourceReg2!=null)
            			return "JAL " + targetReg + ", ["+sourceReg1 + " + " +sourceReg2 + "]";  
            		else if(sourceReg1==null && sourceReg2==null)
            			return "JAL " + targetReg + ", [0 + " + directValue + "]";  
            	} //end JAL
            	
            	else if(test.equals("JMP") && targetReg==null) {//######### --> JMP
            		if(this.memory.getLabelAt(directValue) != null && sourceReg1!=null)
            			return "JMP ["+sourceReg1 + " + "+ this.memory.getLabelAt(directValue) + "]";
            		else if(this.memory.getLabelAt(directValue) != null && sourceReg1==null)
            			return "JMP [0 + "+ this.memory.getLabelAt(directValue) + "]";       		
            		else if(sourceReg1==null && sourceReg2!=null)
            			return "JMP [0 + "+sourceReg2 + "]";  
            		else if(sourceReg1!=null && sourceReg2==null)
            			return "JMP ["+sourceReg1 + " + "+directValue + "]";              			
            		else if(sourceReg1!=null && sourceReg2!=null)
            			return "JMP ["+sourceReg1 + " + "+sourceReg2 + "]";  
            		else if(sourceReg1==null && sourceReg2==null)
            			return "JMP [0 + " + directValue + "]";  
            	}//end JMP
            	
            	else {//######### --> all other Jumps:
            		
            		if(this.memory.getLabelAt(directValue) != null && sourceReg1!=null)
            			return this.decoder.decipherJMP_NOPCommand(command)+  " ["+sourceReg1 + " + "+ this.memory.getLabelAt(directValue) + "]";            		
            		else if(this.memory.getLabelAt(directValue) != null && sourceReg1==null && targetReg==null)            			
            			return this.decoder.decipherJMP_NOPCommand(command)+ " [0 + "+ this.memory.getLabelAt(directValue) + "]";  
            		else if(this.memory.getLabelAt(directValue) != null && targetReg!=null)            			
            			return this.decoder.decipherJMP_NOPCommand(command)+ " [" +targetReg + " + " +this.memory.getLabelAt(directValue) + "]";
            		
            		else if(targetReg!=null && sourceReg2!=null)
            			return this.decoder.decipherJMP_NOPCommand(command)+ " ["+targetReg + " + "+sourceReg2 + "]";  
            		else if(targetReg!=null && sourceReg2==null)
            			return this.decoder.decipherJMP_NOPCommand(command)+ "["+targetReg + " + "+directValue + "]";              			
            		else if(targetReg!=null && sourceReg2!=null)
            			return this.decoder.decipherJMP_NOPCommand(command)+ " ["+targetReg + " + "+sourceReg2 + "]";
            		else if(targetReg==null && sourceReg1==null && sourceReg2==null)
            			return this.decoder.decipherJMP_NOPCommand(command)+ " [0 + "+directValue + "]";
            		else if(targetReg==null && sourceReg1==null && sourceReg2!=null)
            			return this.decoder.decipherJMP_NOPCommand(command)+ " [0 + "+sourceReg2 + "]";
            	}//end all other Jumps	            	
            } //END DISAS -----------------------------------------------
          
            //hier sollte der disas nie hinkommen
            if(isdisas)
        	  return "!!! ERROR: DISAS - processor.decode... -->  'case JMP_NOP' ";
            
            if(command==0) {
            	if (nop==0) {
    				this.incIP();
    				this.stats.updateStatistics("Nop");
    				if(stepping)	
    					engine.displayConsole("NOP", false);
    				return"";
    			}    			
    			if (vnop==0) {
    				this.incIP();
    				this.stats.updateStatistics("Nop");
    				if(stepping)
    					engine.displayConsole("VNOP", false);
    				return"";
    			}
            }
            
            
            if( ! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JMP") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JNZ") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JZ") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JNC") &&
            		! this.decoder.decipherJMP_NOPCommand(command).toString().equals("JC")) { //Bei JMP und JAL ist auch 000.. am Anfang!
            	if (nop==0) {
    				this.incIP();
    				this.stats.updateStatistics("Nop");
    				if(stepping)
    					engine.displayConsole("NOP", false);
    				return"";
    			}
    			
    			if (vnop==0) {
    				this.incIP();
    				this.stats.updateStatistics("Nop");
    				if(stepping)
    					engine.displayConsole("VNOP", false);
    				return"";
    			}
            }                  
   
            if(stepping) {
            	engine.displayConsole(this.memory.getDisassembler().getDisassembledInstructionAt(progCounter), false);
            }
           
            if(!isdisas)
            	this.executeJMP_NOPOperation(instr, targetReg, sourceReg1, sourceReg2, this.decoder.decipherJMP_NOPCommand(command));
			return null; //kann kein vecOp mehr sein
		case FLAG:
			//KOMMANDO "DIASASSEMBLIERT" AUSGEBEN:
			command = instr << 8;
			command = command >>> 28;
			if (isdisas) {
				return (this.decoder.decipherFLAGCommand(command)==null ? "NONE" :
					this.decoder.decipherFLAGCommand(command).toString());
			}
			Instruction flag=null;
			try {
				flag = decoder.decipherFLAGCommand(command);
				if(flag==null) {
					this.incIP();
					return null;
				}
			}
			catch (Exception e) {
				//ignore if command is not recognized, then not known
				return null;
			}
			if(stepping)
				engine.displayConsole(flag.toString(), false);
			
			if(!isdisas) {
				 this.executeFLAGOperation(flag);
				 this.incIP();
			}
			return null; //kann kein vecOp mehr sein
		default:
			if (isdisas) 
				return"";
			this.incIP();
			break;
		}

		int rrrr = (instr>>>8) & 0xF;  //Ziel     -> 00000000 00000000 00001111 vvvvwwww
		int vvvv = (instr>>>4) & 0xF;  //Quelle1  -> 00000000 00000000 0000rrrr 1111wwww
		int wwww = instr & 0xF;  	   //Quelle2  -> 00000000 00000000 0000rrrr vvvv1111
		
//		################################### 2. Vektor BEFEHLE: ######################################################## VECTOR
		try {
			//VEKTORTYPE, WORD? und VECTOR_OPERATION
		
			int vecCommand=instr<<16;   //xxxx
			vecCommand=vecCommand>>>28;
			
			int word=instr<<14; //bb
			word=word>>>30;
			switch(this.decoder.decipherVectorType(vecCommandType)) { //-> wenn null, einfach ignorieren

				case VALU:
					
					if (isdisas) {
						/**
						 * Fall: Kooperationsbefehl
						 * siehe case ALU oben
						 */
						if(kooperationsbefehl!=null) {
							return kooperationsbefehl + ("V"+this.decoder.decipherALUCommand(vecCommand)) + 
							("."+ this.decoder.decipherWord(word) + " R"+rrrr + ", R" + vvvv + ", R"+wwww);
						}
						else
							return("V"+this.decoder.decipherALUCommand(vecCommand)) + 
							("."+ this.decoder.decipherWord(word) + " R"+rrrr + ", R" + vvvv + ", R"+wwww);
					}
					
					if(stepping && !coop)
						engine.displayConsole("V"+this.decoder.decipherALUCommand(vecCommand) +					
							("."+ this.decoder.decipherWord(word) + " R"+rrrr + ", R" + vvvv + ", R"+wwww), false);
					
					 if(!isdisas) {
						 this.switchVectorALUOperation(this.decoder.decipherWord(word),this.decoder.decipherALUCommand(vecCommand), rrrr, vvvv, wwww);
						 this.incIP();
					 }
					break;

				case VSHUF:
					if (isdisas)
						return ("VSHUF ")+(" - WORD:"+ this.decoder.decipherWord(word) + ", DEST: R"+rrrr + ", SRC1: R" + vvvv + ", SRC2: R"+wwww);;
					
					if(stepping)
						engine.displayConsole("Vector-SHUFFLE Instruction: "+
								(" - WORD:"+ this.decoder.decipherWord(word) + ", DEST: R"+rrrr + ", SRC1: R" + vvvv + ", SRC2: R"+wwww), false);
					
					int perm=(instr<<4)>>>24;
					int regSelect=(instr<<16)>>>28;
					
					if(!isdisas) {
						this.executeVSHUF_Instruction(this.decoder.decipherWord(word).getDiv(),this.decoder.decipherWord(word).getValue(),rrrr, vvvv, wwww,perm,regSelect);
						this.incIP();
						this.stats.updateStatistics("VShuffle");
					}
					break;
				}
		}
		catch (NullPointerException e) {
			//ignore...this means no vectorInstruction
		}
		
        if (isdisas) 
        	return instr + " ---> NOCH NICHT DISASSEMBLIERT!!!";
        
        return ""; //irrelevant, if used by processor

	} //#################### end method "decodeInstruction()"
	
	
	
	//###########################################################################################################
	// switch_and_execute - methods
	//###########################################################################################################
	
	
	
	/**
	 * This method is used to execute a Load or a Store Operation.
	 */
	private void executeLOAD_STOREOperation(ScalarRegister dd, ScalarRegister ss, ScalarRegister tt, Instruction instruction, int instr) throws Exception{

		int value=0; //value = das Ergebnis
                int direktwert = instr & 0xffff;
                if ((direktwert & 0x8000) != 0){
                    direktwert |= 0xffff0000; // sign extend
                }
                if (tt == null) {// DIREKT_WERT_FALL
                    tt = ScalarRegister.ttTmp;
                    tt.setValue(direktwert);
                }


		switch (instruction) {
		case LOAD:
			this.stats.updateStatistics("Load");
			try {
				if(ss!=null)
					value += ss.getValue();

                                value += tt.getValue(); 

                                /*############ SET ZERO IF "LD" LOADS 0 #############*/				
                                int val = this.memory.readValueAt(value);
                                this.setStatusRegs(((int)val) == 0,  this.carryFlag); 
				if (dd!=null) {
                                    dd.setValue(val);
				}
			}
			catch(Exception e) {
				engine.displayConsole("LOAD: ERROR! address: " + value + " IP: " + ip + " , processor.executeLOAD_STOREOperation()", true);
				return;
			}
			
			break;
		case STORE:
			try { //Adresse, an welcher der Inhalt von A gespeichert wird, holen:					
				if(ss!=null) 
					value += ss.getValue(); //FALL: ST [0+X], A
                                value += tt.getValue();

				this.memory.writeTo(value, ScalarRegister.A.getValue()); //NUR A !
			}
			catch(Exception e) {
				engine.displayConsole("STORE: ERROR! address: " + value + " IP: " + ip + " , processor.executeLOAD_STOREOperation()", true);
				return;
			}
			this.stats.updateStatistics("Store");
			break;

		case VLOAD: //Vektor-Register mit Inhalt ab Adresse A+X laden
			this.stats.updateStatistics("Vload");
			try {
				if(ss!=null)
					value += ss.getValue(); //FALL: VLD Ri, [reg+reg] ...
                                value += tt.getValue();
			}
			catch(Exception e) {
				//should not happen..
				engine.displayConsole("VLOAD: ERROR! address: " + value + " IP: " + ip + " , processor.executeLOAD_STOREOperation()", true);
				return;
			}

			//DECODIERTES VEKTORREGISTER !AB! GELESENENER ADRESSE FUELLEN
			int reg = instr >> 8 & 0x0f; //von 0 bis N

			for(int i=0; i<config.getK(); i++) {
				try {
					this.setVRegsValue(reg, i,this.memory.readValueAt(value));
				}
				catch (java.lang.NullPointerException e) {       															
					engine.displayConsole("VSTORE: ERROR! address: " + value + " IP: " + ip + " , processor.executeLOAD_STOREOperation()", true);
					return;
				}
				value++;
			}
			break;
		case VSTORE:		//Befehl dient dazu Vektorregister in voller Breite ab einer bestimmten Adresse abzuspeichern
			try {
				if(ss!=null) value += ss.getValue(); //FALL: VST [reg+reg], Ri
                                value += tt.getValue();

				int src_reg = instr >> 4 & 0x0f; //von 0 bis N

				for(int i=0; i<config.getK(); i++) {
					this.memory.writeTo(value, this.getVRegsValue(src_reg, i));
					value++;
				}
			}
			catch(Exception e) {
				engine.displayConsole("VST: ERROR WHILE TRYING TO WRITE AT " + value +" ?", true);
				return;
			}
			this.stats.updateStatistics("Vstore");
			break;
		}
    }
	/**
	 * This method is used to check which ScalarALU Operation has to be executed.
	 * @param instruction - the real 32 bit opcode
	 * @param dd - destination register
	 * @param ss - src reg 1
	 * @param tt - src reg 2
	 * @param instr - instruction to execute
	 * @throws Exception
	 */
	private void switchScalarALUOperation(int instruction, ScalarRegister dd, ScalarRegister ss, ScalarRegister tt, Instruction instr) throws Exception{
		
		long value=0; //value = das Ergebnis
		switch (instr) {

		case ADD:
			executeScalarALUOperation(instruction, dd,ss,tt,'+');
			this.stats.updateStatistics("Add");
	        break;
		case ADC:
			executeScalarALUOperation(instruction, dd,ss,tt,'A');
			this.stats.updateStatistics("Adc");
			break;
		case SUB:
			executeScalarALUOperation(instruction, dd,ss,tt,'-');
			this.stats.updateStatistics("Sub");
			break;

		case SBC:
			executeScalarALUOperation(instruction, dd,ss,tt,'S');
			this.stats.updateStatistics("Sbc");
			break;

		case INC:
			if(dd != null) {
				if(ss==null) {
					ss=ScalarRegister.ssTmp;
					ss.setValue(0);
				}
				
			value = (ss.getValue() & 0xFFFFFFFFL) + 1;
			dd.setValue((int)value);
			
			}
			this.setStatusRegs(((int)value) == 0,   (int) ((value >>> 32) & 1) );
			
			this.stats.updateStatistics("Inc");
			break;

		case DEC:
			if(dd != null) {
				if(ss==null) {
					ss=ScalarRegister.ssTmp;
					ss.setValue(0);
				}
				
			value = ss.getValue() + 0xffffffffL;
			
			if((value&0xFFFFFFFFL)==0) 
				value&=0xFFFFFFFFL;
			
			dd.setValue((int)value);
			}
			
			this.setStatusRegs(((int)value) == 0,   (int) ((value >>> 32) & 1) );
				
			this.stats.updateStatistics("Dec");
			break;

		case AND:
			executeScalarALUOperation(instruction, dd,ss,tt,'&');
			this.stats.updateStatistics("And");
			break;

		case OR:
			executeScalarALUOperation(instruction, dd,ss,tt,'|');
			this.stats.updateStatistics("Or");
			break;

		case XOR:
			executeScalarALUOperation(instruction, dd,ss,tt,'^');
			this.stats.updateStatistics("Xor");
			break;

		case MUL:
			if (config.isScalar_mul()) {
				executeScalarALUOperation(instruction, dd,ss,tt,'*');
				this.stats.updateStatistics("Mul");
			}	
			else {//wenn scalarMUL nicht konfiguriert ist, ergebnis = 0 und errormeldung
				if(dd!=null) {
					dd.setValue(0);
				}
				engine.displayConsole("ERROR - EXECUTED MUL WHILE MUL IS NOT CONFIGURED IN config.vhd",true);
			}
			break;

		case LSL:
			//EDIT: es geht auch LSL REG, 0, LSL 0, REG UND LSL 0, 0
			if(ss==null) {
				ss=ScalarRegister.ssTmp;
				ss.setValue(0);
			}
			if(dd==null) {
				dd=ScalarRegister.ddTmp;
				dd.setValue(0); 
			}
			long ssL = ss.getValue() & 0xFFFFFFFFL;
			value=(ssL<<1);
			dd.setValue((int)value);
			
			//zero pruefen			               //carry pruefen
			this.setStatusRegs(((int)value) == 0,  (int) ((value >>> 32) & 1)); //set zero false, everytime if zero != 0	
			
			this.stats.updateStatistics("Lsl");
			break;

		case LSR:
			//EDIT: es geht auch LSR REG, 0, LSR 0, REG UND LSR 0, 0
			if(ss==null) {
				ss=ScalarRegister.ssTmp;
				ss.setValue(0);
			}
			if(dd==null) {
				dd=ScalarRegister.ddTmp;
				dd.setValue(0); 
			}
			long ssL2 = ss.getValue() & 0xFFFFFFFFL;
			value=(ssL2>>1);
			dd.setValue((int)value);
			
			//zero pruefen			               //carry pruefen
			this.setStatusRegs(((int)value) == 0,  (int) ((value >>> 32) & 1)); //set zero false, everytime if zero != 0	
			
			this.stats.updateStatistics("Lsr");
			break;

		case ROL:
			//EDIT: es geht auch ROL REG, 0, ROL 0, REG UND ROL 0, 0
			if(ss==null) {
				ss=ScalarRegister.ssTmp;
				ss.setValue(0);
			}
			if(dd==null) {
				dd=ScalarRegister.ddTmp;
				dd.setValue(0); 
			}
			long ssL3 = ss.getValue() & 0xFFFFFFFFL;
			value=(ssL3<<1);
			
			if(carryFlag==1){ //carry rechts einfuegen
				int val = (int)value + 1;
//				System.out.println(BitUtils.convert(new Integer(val), Numeratives.BINARY, true));
				dd.setValue(val);
			}
			else
				dd.setValue((int)value);
			
			//zero pruefen			               //carry pruefen
			this.setStatusRegs(((int)value) == 0,  (int) ((value >>> 32) & 1)); //set zero false, everytime if zero != 0	
			
			this.stats.updateStatistics("Rol");
			break;

		case ROR:
			//EDIT: es geht auch ROR REG, 0, ROR 0, REG UND ROR 0, 0
			if(ss==null) {
				ss=ScalarRegister.ssTmp;
				ss.setValue(0);
			}
			if(dd==null) {
				dd=ScalarRegister.ddTmp;
				dd.setValue(0); 
			}
			
			long ssL4 = ss.getValue() & 0xFFFFFFFFL;
			value=(ssL4>>1);
			
			if(carryFlag==1){ //carry rechts einfuegen
				long tmp = 0x80000000; //BitUtils.binary2Decimal("10000000000000000000000000000000");
				int val = (int)value | (int)tmp;
//				System.out.println(BitUtils.convert(new Integer(val), Numeratives.BINARY, true));
				dd.setValue(val);
			}
			else
				dd.setValue((int)value);
			
			//zero pruefen			               //carry pruefen
			this.setStatusRegs(((int)value) == 0,  (int) ((value >>> 32) & 1)); //set zero false, everytime if zero != 0	
		
			this.stats.updateStatistics("Ror");
			break;

	}
}
	
	/**
	 * This method is used to execute a ScalarALU Operation.
	 * @param instruction - the real 32 bit opcode
	 * @param dd - destination register
	 * @param ss - src-register 1
	 * @param tt - src-register 2
	 * @param sign - ALU-instruction-identifier +==ADD,A==ADDC,
	 *        -==SUB,S==SUBC,*==MUL,&==AND,|==OR,^==XOR 
	 */
    private void executeScalarALUOperation(int instruction, ScalarRegister dd, ScalarRegister ss, ScalarRegister tt,char sign)throws Exception{
    	long value=0; //um carry Bit ueberpruefen zu koennen long
    	int direktwert = 0;
    	
    	//ZERO wird bei jedem Befehl ueberprueft
    	//carry nur bei adds, subs, shifts, rols
    	
				//HIER DARF NICHT DER SPEICHER GELESEN WERDEN, SONDERN DIE DIREKTWERT-BITS PRUEFEN!!!
				direktwert = instruction & 0xffff; //(instruction<<15) >>>15; //die ersten 16 bit "koennten" ein direktwert sein!
                                if ((direktwert & 0x8000) != 0){
                                    direktwert |= 0xffff0000; // sign extend
                                }
			
			if (ss == null) { //dann: 0+...
				ss = ScalarRegister.ssTmp;
				ss.setValue(0);
			}
			if (tt == null) {// DIREKT_WERT_FALL
				tt = ScalarRegister.ttTmp;
				tt.setValue(direktwert);
				direktwert  = 0; // FUER FALL: Instr reg,reg,0 un if abfrage geaendert
							// von !(memCon==0) hat kein sinn
				// jetzt tt==null wenn ja dann memCon=0 und wenn ss==null dann
				// ss.value==memCon==0
			}
			int tmp0 = ss.getValue();// & 0xffffffffL;
			int tmp1 = tt.getValue();// & 0xffffffffL;
		
		//---------------------------------------
		if (sign=='+') {
			long tmp2=tmp0 & 0xFFFFFFFFL;//mask to add in fully 32 bit
			long tmp3=tmp1 & 0xFFFFFFFFL;
//			System.out.println(BitUtils.convert(new Long(tmp2), Numeratives.BINARY, true));
			value =tmp2 + tmp3; 
		}
		else if (sign=='-') {
			// A - B = A + (~B +1) '~' gibt 2erkomplement
			long test = (0xFFFFFFFFL - tmp1);
			value = tmp0 + (test + 1 );
			
			// if((value&0xFFFFFFFFL)==0) 
			// 	value&=0xFFFFFFFFL;
		}
		
		else if (sign=='&') {  //AND
			value = tmp0 & tmp1;
		}
		else if (sign=='|') {  //OR
			value = tmp0 | tmp1;
		}
		else if (sign=='^') { //XOR
			value = tmp0 ^ tmp1;
		}
		else if (sign=='A'){//add carry
			long addcarry=0;
		    long ssl=(long)ss.getValue()& 0xffffffffl;
		    long ttl=(long)tt.getValue()& 0xffffffffl;
		    addcarry = ssl + ttl+this.carryFlag;
		    setStatusRegs(this.zeroFlag, 0);
		    value = (int) (addcarry & 0xffffffffl);
		    if ((addcarry>>>32)>0){
		    	setStatusRegs(this.zeroFlag, 1);
	    	}
	   
		}
		else if (sign == 'S') {// sub carry
			long subcarry = 0;
			long ssl=(long)ss.getValue()& 0xffffffffl;
		    long ttl=(long)tt.getValue()& 0xffffffffl;
			subcarry = ssl - ttl;
			if (this.carryFlag==0) subcarry-=1;
			value=(int)(subcarry & 0xffffffffl);
			if ((subcarry>>>32)<0){
				setStatusRegs(this.zeroFlag, 1);
			}
			if (value==0) 
				setStatusRegs(true,this.carryFlag);
			else if(value<0)
		    	setStatusRegs(false, 1);
			else
				setStatusRegs(false,this.carryFlag);
		}
		//MUL
		else if (sign=='*') {
			tmp0 = tmp0 & 0xffff;    //value(32bit) = ss(16bit) * tt(32bit) --> HiCoVec MUL
			tmp1 = tmp1 & 0xffff;
			value = tmp0 * tmp1;	
		}
		
		if(dd != null)
			dd.setValue((int)value);

		//carry zero IMMER pruefen			               
		this.setStatusRegs(((int)value) == 0,  (int) ((value >>> 32) & 1)); 	
    }
    
    /**
     * 
     * This method is used to check which VectorALU Operation has to be executed.
     * @param vecword - wordwidth enum
     * @param instr - instruction to execute.
     * @param rrrr - VectorDestReg: 0 means R0, 1 means R1 etc...
     * @param vvvv - VectorSrcReg1
     * @param wwww - VectorSrcReg2
     * @throws Exception
     */
	private void switchVectorALUOperation(VecWord vecword,Instruction instr, int rrrr, int vvvv, int wwww) throws Exception{
		int word=vecword.getValue();
		
		switch (instr) {

		case ADD: //vreg[rrrr][0] = vreg[wwww][0] + vReg[wwww][0] usw......
			
			executeVectorALUOperation(rrrr, vvvv, wwww,word, '+');
			this.stats.updateStatistics("Vadd");
			break;
		case SUB:
			
			executeVectorALUOperation(rrrr, vvvv, wwww,word, '-');
			this.stats.updateStatistics("VSub");
			break;
		case AND:
			
			executeVectorALUOperation(rrrr, vvvv, wwww,word, '&');
			this.stats.updateStatistics("Vand");
			break;
		case OR:
			
			executeVectorALUOperation(rrrr, vvvv, wwww,word, '|');
			this.stats.updateStatistics("Vor");
			break;

		case XOR:
			
			executeVectorALUOperation(rrrr, vvvv, wwww,word, '^');
			this.stats.updateStatistics("Vxor");
			break;

		case MUL:
			if(config.isVector_mul())
				executeVectorALUOperation(rrrr, vvvv, wwww,word, '*');
			else {//wenn vecMUL nicht aktiviert ist, alles auf 0
				for (int i = 0; i < config.getK(); i++) {
					this.setVRegsValue(rrrr, i, 0);		
				}
				engine.displayConsole("ERROR - EXECUTED VMUL WHILE VMUL IS NOT CONFIGURED IN config.vhd",true);
			}
			this.stats.updateStatistics("Vmul");
			break;
		case LSL:
			try{
			   for(int i=0; i<config.getK(); i++) {
				   if (word == 8) {
				      int v1 = ((this.getVRegsValue(vvvv, i) & 0x000000ff)<<1)&0x000000ff;
					  int v2 = ((this.getVRegsValue(vvvv, i) & 0x0000ff00)<<1)&0x0000ff00;
				   	  int v3 = ((this.getVRegsValue(vvvv, i) & 0x00ff0000)<<1)&0x00ff0000;
					  int v4 = ((this.getVRegsValue(vvvv, i) & 0xff000000)<<1)&0xff000000;
					  this.setVRegsValue(rrrr, i, v1 | v2 | v3 | v4);
				   }else if(word==16){
					   int v1 = ((this.getVRegsValue(vvvv, i) & 0x0000ffff)<<1)&0x0000ffff;
					   int v2 = ((this.getVRegsValue(vvvv, i) & 0xffff0000)<<1)&0xffff0000; 
					   this.setVRegsValue(rrrr, i, v1 | v2);
				   }else if(word==64){ //grund weil K immer gerade sein muss!!!
					   int j=i+1;
					   long v= ((this.getVRegsValue(vvvv, i)<<32)|this.getVRegsValue(vvvv, j))<<1;
					   this.setVRegsValue(rrrr, i, (int)((v&0xffffffff00000000l)>>>32));
					   this.setVRegsValue(rrrr, j, (int)(v&0x00000000ffffffffl));
					   i=j;
				   }else{
					   this.setVRegsValue(rrrr, i, this.getVRegsValue(vvvv, i)<<1);
				   }
			   }
			}catch(Exception e){
				engine.displayConsole("Error in VLSL --> Processor.switchVectorALUOperation()", true);
				return;
			}   
			break;
		case LSR:
			try{
				   for(int i=0; i<config.getK(); i++) {
					   if (word == 8) {
					      int v1 = ((this.getVRegsValue(vvvv, i) & 0x000000ff)>>>1)&0x000000ff;
						  int v2 = ((this.getVRegsValue(vvvv, i) & 0x0000ff00)>>>1)&0x0000ff00;
					   	  int v3 = ((this.getVRegsValue(vvvv, i) & 0x00ff0000)>>>1)&0x00ff0000;
						  int v4 = ((this.getVRegsValue(vvvv, i) & 0xff000000)>>>1)&0xff000000;
						  this.setVRegsValue(rrrr, i, v1 | v2 | v3 | v4);
					   }else if(word==16){
						   int v1 = ((this.getVRegsValue(vvvv, i) & 0x0000ffff)>>>1)&0x0000ffff;
						   int v2 = ((this.getVRegsValue(vvvv, i) & 0xffff0000)>>>1)&0xffff0000; 
						   this.setVRegsValue(rrrr, i, v1 | v2);
					   }else if(word==64){
						   int j=i+1;
						   long v= ((this.getVRegsValue(vvvv, i)<<32)|this.getVRegsValue(vvvv, j))>>>1;
						   this.setVRegsValue(rrrr, i, (int)((v&0xffffffff00000000l)>>>32));
						   this.setVRegsValue(rrrr, j, (int)(v&0x00000000ffffffffl));
						   i=j;
					   } else{
						   this.setVRegsValue(rrrr, i, this.getVRegsValue(vvvv, i)>>>1);
					   }
				   }
				}catch(Exception e){
					engine.displayConsole("Error in VLSR --> Processor.switchVectorALUOperation()", true);
					return;
				}   
				break;
		}
		
	}
	
	 /**
     * 
     * This method is used execute a VectorALU Operation.
     * It executes:  VADD, VSUB, VAND, VOR, VXOR and VMUL.
     * @param vecword - wordwidth enum
     * @param rrrr - VectorDestinationRegister: 0 means R0, 1 means R1 etc...
     * @param vvvv - VectorSrcReg1
     * @param wwww - VectorSrcReg2
     * @throws Exception
     */
	private void executeVectorALUOperation(int rrrr, int vvvv, int wwww,int word,char sign) {
		if (sign == '+') {
			try {
				for (int i = 0; i < config.getK(); i++) {
					if (word == 8) {
						/**
						 * UEBER this.getVRegsValue
						 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
						 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!! falls Zugriff auf
						 * VecReg das nicht configuriert ist!!!!!!!!!!!!!!!
						 */
						int v1 = this.getVRegsValue(vvvv, i) & 0x000000ff;
						int v2 = this.getVRegsValue(vvvv, i) & 0x0000ff00;
						int v3 = this.getVRegsValue(vvvv, i) & 0x00ff0000;
						int v4 = this.getVRegsValue(vvvv, i) & 0xff000000;
						int w1 = this.getVRegsValue(wwww, i) & 0x000000ff;
						int w2 = this.getVRegsValue(wwww, i) & 0x0000ff00;
						int w3 = this.getVRegsValue(wwww, i) & 0x00ff0000;
						int w4 = this.getVRegsValue(wwww, i) & 0xff000000;
						int r1 = (v1 + w1) & 0x000000ff;
						int r2 = (v2 + w2) & 0x0000ff00;
						int r3 = (v3 + w3) & 0x00ff0000;
						int r4 = (v4 + w4) & 0xff000000;
						this.setVRegsValue(rrrr, i, r1 | r2 | r3 | r4);
					} else if (word == 16) {
						int v1 = this.getVRegsValue(vvvv, i) & 0x0000ffff;
						int v2 = this.getVRegsValue(vvvv, i) & 0xffff0000;
						int w1 = this.getVRegsValue(wwww, i) & 0x0000ffff;
						int w2 = this.getVRegsValue(wwww, i) & 0xffff0000;
						int r1 = (v1 + w1) & 0x0000ffff;
						int r2 = (v2 + w2) & 0xffff0000;
						this.setVRegsValue(rrrr, i, r1 | r2);
					} else if (word == 64){
						int j=i+1; 
						long v= (this.getVRegsValue(vvvv, i)<<32)|this.getVRegsValue(vvvv, j);
						long w= (this.getVRegsValue(wwww, i)<<32)|this.getVRegsValue(wwww, j);
						long r= v+w;
						this.setVRegsValue(rrrr, i, (int)((r&0xffffffff00000000l)>>>32));
						this.setVRegsValue(rrrr, j, (int)(r&0x00000000ffffffffl));
						i=j;
				    }  else {
						this.setVRegsValue(rrrr, i, (this
								.getVRegsValue(vvvv, i) + this.getVRegsValue(
								vvvv, i)));
					}
				}

			} catch (Exception e) {
				engine.displayConsole("Error in VADD --> Processor.executeVectorALUOperation()", true);
				return;
			}
		}
		else if(sign=='-')
			try {
				for (int i = 0; i < config.getK(); i++) {
					if (word == 8) {
						
						int v1 = this.getVRegsValue(vvvv, i) & 0x000000ff;
						int v2 = this.getVRegsValue(vvvv, i) & 0x0000ff00;
						int v3 = this.getVRegsValue(vvvv, i) & 0x00ff0000;
						int v4 = this.getVRegsValue(vvvv, i) & 0xff000000;
						int w1 = this.getVRegsValue(wwww, i) & 0x000000ff;
						int w2 = this.getVRegsValue(wwww, i) & 0x0000ff00;
						int w3 = this.getVRegsValue(wwww, i) & 0x00ff0000;
						int w4 = this.getVRegsValue(wwww, i) & 0xff000000;
						int r1 = (v1 - w1) & 0x000000ff;
						int r2 = (v2 - w2) & 0x0000ff00;
						int r3 = (v3 - w3) & 0x00ff0000;
						int r4 = (v4 - w4) & 0xff000000;
						this.setVRegsValue(rrrr, i, r1 | r2 | r3 | r4);
					} else if (word == 16) {
						int v1 = this.getVRegsValue(vvvv, i) & 0x0000ffff;
						int v2 = this.getVRegsValue(vvvv, i) & 0xffff0000;
						int w1 = this.getVRegsValue(wwww, i) & 0x0000ffff;
						int w2 = this.getVRegsValue(wwww, i) & 0xffff0000;
						int r1 = (v1 - w1) & 0x0000ffff;
						int r2 = (v2 - w2) & 0xffff0000;
						this.setVRegsValue(rrrr, i, r1 | r2);
					} else if (word == 64){
						int j=i+1;
						long v= (this.getVRegsValue(vvvv, i)<<32)|this.getVRegsValue(vvvv, j);
						long w= (this.getVRegsValue(wwww, i)<<32)|this.getVRegsValue(wwww, j);
						long r= v-w;
						this.setVRegsValue(rrrr, i, (int)((r&0xffffffff00000000l)>>>32));
						this.setVRegsValue(rrrr, j, (int)(r&0x00000000ffffffffl));
						i=j;
				    }  else {
						this.setVRegsValue(rrrr, i, (this
								.getVRegsValue(vvvv, i) - this.getVRegsValue(
								wwww, i)));
					}
				}

			} catch (Exception e) {
				engine.displayConsole("Error in VSUB --> Processor.executeVectorALUOperation()", true);
				return;
			}
		
		else if(sign=='&')
			try {
				for (int i = 0; i < config.getK(); i++) {
					this.setVRegsValue(rrrr, i, this.getVRegsValue(vvvv, i)
							& this.getVRegsValue(wwww, i));
				}
			} catch (Exception e) {
				engine.displayConsole("Error in VAND --> Processor.executeVectorALUOperation()", true);
				return;
			}
		
		else if(sign=='|')
			try {
				for (int i = 0; i < config.getK(); i++) {
					this.setVRegsValue(rrrr, i, this.getVRegsValue(vvvv, i)
							| this.getVRegsValue(wwww, i));
				}
			} catch (Exception e) {
				engine.displayConsole("Error in VOR --> Processor.executeVectorALUOperation()", true);
				return;
			}
		
		else if(sign=='^')
			try {
				for (int i = 0; i < config.getK(); i++) {
					this.setVRegsValue(rrrr, i, this.getVRegsValue(vvvv, i)
							^ this.getVRegsValue(wwww, i));
				}
			} catch (Exception e) {
				engine.displayConsole("Error in VXOR --> Processor.executeVectorALUOperation()", true);
				return;
			}
		
		else if(sign=='*') {
				try {
					for (int i = 0; i < config.getK(); i++) {
						if (word == 8) {
							
							int v1 = this.getVRegsValue(vvvv, i) & 0x000000ff;
							int v2 = this.getVRegsValue(vvvv, i) & 0x0000ff00;
							int v3 = this.getVRegsValue(vvvv, i) & 0x00ff0000;
							int v4 = this.getVRegsValue(vvvv, i) & 0xff000000;
							int w1 = this.getVRegsValue(wwww, i) & 0x000000ff;
							int w2 = this.getVRegsValue(wwww, i) & 0x0000ff00;
							int w3 = this.getVRegsValue(wwww, i) & 0x00ff0000;
							int w4 = this.getVRegsValue(wwww, i) & 0xff000000;
							int r1 = (v1 * w1) & 0x000000ff;
							int r2 = (v2 * w2) & 0x0000ff00;
							int r3 = (v3 * w3) & 0x00ff0000;
							int r4 = (v4 * w4) & 0xff000000;
							this.setVRegsValue(rrrr, i, r1 | r2 | r3 | r4);
						} else if (word == 16) {
							int v1 = this.getVRegsValue(vvvv, i) & 0x0000ffff;
							int v2 = this.getVRegsValue(vvvv, i) & 0xffff0000;
							int w1 = this.getVRegsValue(wwww, i) & 0x0000ffff;
							int w2 = this.getVRegsValue(wwww, i) & 0xffff0000;
							int r1 = (v1 * w1) & 0x0000ffff;
							int r2 = (v2 * w2) & 0xffff0000;
							this.setVRegsValue(rrrr, i, r1 | r2);
						} else if (word == 64){
							int j=i+1;
							long v= (this.getVRegsValue(vvvv, i)<<32)|this.getVRegsValue(vvvv, j);
							long w= (this.getVRegsValue(wwww, i)<<32)|this.getVRegsValue(wwww, j);
							long r= v*w;
							this.setVRegsValue(rrrr, i, (int)((r&0xffffffff00000000l)>>>32));
							this.setVRegsValue(rrrr, j, (int)(r&0x00000000ffffffffl));
							i=j;
					    }  
						else {
							this.setVRegsValue(rrrr, i, (this
									.getVRegsValue(vvvv, i) * this.getVRegsValue(
									wwww, i)));
						}
					}

				}catch (Exception e) {	
					engine.displayConsole("Error in VMUL --> Processor.executeVectorALUOperation()", true);
					return;
				 }
			
		}	
	}
	
	/**
	 * This method is used to execute the V-Shuffle Operation.
	 */
	private void executeVSHUF_Instruction(int div,int word,int rrrr, int vvvv, int wwww,int perm,int regSelect) throws Exception{
		        
		int selectVW[]=new int [4];
		selectVW[0]=regSelect>>>3;
		selectVW[1]=(regSelect<<29)>>>30;
		selectVW[2]=(regSelect<<30)>>>30;
		selectVW[3]=(regSelect<<31)>>>30;
		int perms[]=new int [4];
		perms[0]=perm>>>6;
		perms[1]=(perm<<26)>>>30;
		perms[2]=(perm<<28)>>>30;
		perms[3]=(perm<<30)>>>30;
		int [][] sourceVW=new int[0][0];
		
		int K = config.getK();
		
		if (BitUtils.isXPot(2, K)){

			sourceVW=new int[2][K];
			for (int i = 0; i < K; i++) {    //regs vorbereiten
				
				   sourceVW[0][i]=this.getVRegsValue(vvvv, i);
				   sourceVW[1][i]=this.getVRegsValue(wwww, i);
				
			}
			if (isSimpleShuf(word)) this.doPermutationShuf(K/div, K, selectVW, perms, rrrr, sourceVW);// Vshuf modus erkennen
			else  this.doPermShufOfPartsOfRegs(K, div/K, selectVW, perms, sourceVW, rrrr);
		}
		else {
			//if K is not 2pot z.B K=6;
			int k1=0;
			for (int i = K; !(BitUtils.isXPot(2, i)); i+=2) k1=i+2;
			sourceVW=new int[2][k1];
			
			for (int i = 0; i < k1; i++) {      // nexte K 2pot suchen und regs vorbereiten
				if (i<K){
				   sourceVW[0][i]=this.getVRegsValue(vvvv, i);
				   sourceVW[1][i]=this.getVRegsValue(wwww, i);
				}
				else{
					sourceVW[0][i]=0;
					sourceVW[1][i]=0;
				}
			} 
			
			if (isSimpleShuf(word)){   // Vshuf modus erkennen
			   
				doPermutationShuf(k1/div, k1, selectVW, perms, rrrr, sourceVW);
				  
			}
			else{
				
				doPermShufOfPartsOfRegs(k1, div/k1, selectVW, perms, sourceVW, rrrr);
				
			}		
		}
	}
	
    /**
     * This method recognize the type of shuffle
     * @return true when the minimum parts to shuffle is a Vdisk if the part is less than a Vdisk then return false
     */
	private boolean isSimpleShuf(int word) {
		int K =config.getK();
		if (word==64 && K>2){
			return true;
			
		}
		else if(word==32 && K>4){
			return true;
			
		}
		else if(word==16 && K>8){
			return true;
			
		}
		else if(word==8 && K>16){
			return true;
			
		}
		else 
			return false;
		
	}	
	
	/**
	 *   This method helps doPermOfPartsOfReg
	 */
	private int[] getMask(final int parts){
		int [] mask = new int [parts];
		if (parts==2){
			mask[0]=0xffff0000;
		    mask[1]=0x0000ffff; 
		}
		else if(parts==4){
			mask[0]=0xff000000;
		    mask[1]=0x00ff0000; 
		    mask[2]=0x0000ff00;
		    mask[3]=0x000000ff; 
			
		}
		else if(parts==8){
			mask[0]=0xf0000000;
		    mask[1]=0x0f000000; 
		    mask[2]=0x00f00000;
		    mask[3]=0x000f0000; 
		    mask[4]=0x0000f000;
		    mask[5]=0x00000f00; 
		    mask[6]=0x000000f0;
		    mask[7]=0x0000000f; 
		}
		else if(parts==16){
		    mask[0]=0xc0000000;
		    mask[1]=0x60000000; 
	        mask[2]=0x0c000000;
	        mask[3]=0x06000000; 
		    mask[4]=0x00c00000;
		    mask[5]=0x00600000; 
		    mask[6]=0x000c0000;
		    mask[7]=0x00060000;
		    mask[8]=0x0000c000;
		    mask[9]=0x00006000; 
		    mask[10]=0x00000c00;
		    mask[11]=0x00000600; 
		    mask[12]=0x000000c0;
		    mask[13]=0x00000060; 
		    mask[14]=0x0000000c;
		    mask[15]=0x00000006;
		}
		return mask;
	}

	/**
	 * This method helps doPermShufOfPartsOfRegs to shift the parts of regs
	 */
	private void setXYShuf(final int parts){
		if (parts==2) {
	    	y=16;
	    	x=16;
	    }
	    else if(parts==4){
	    	y=24;
	    	x=8;
	    }
        else if(parts==8){
	    	y=28;
	    	x=4;
	    }
        else if(parts==16){
	    	y=30;
	    	x=2;
	    }
	}
	
	/**
	 *   This method helps Vshuf to execute.
	 */
	private void doPermShufOfPartsOfRegs(int k1,final int parts,int [] selectVW,int [] perms,int [][] sourceVW,int rrrr){
		 int [] source = new int [k1*(parts)];
		 int [] dest = new int [k1*(parts)]; 
		 int [] mask = new int [parts]; 
		 mask=getMask(parts);
		 setXYShuf(parts);
		 int selector=0;
		    for (int i = 0; i < k1; i++) {
		    	for (int j = 0; j < mask.length; j++) {//source registern lesen
		    		source[selector]=((sourceVW[selectVW[selector%4]][i]) & mask[j])>>>y;
		            if (y==0)  setXYShuf(parts);
		            else y-=x;
		    		selector++;
				}
			}
		    
		    for (int i = 0; i < source.length; i++) { //daten mischen
				dest[perms[i%4]+((i/4)*4)]=source[i];
			}
		    
		    int newrrrr[] = new int [k1];
		    setXYShuf(parts);
		    selector=0;
		    for (int i = 0; i < k1; i++) { // neue Vektorscheiben zusammenstellen  
		    	for (int j = 0; j < mask.length; j++) {
		    		newrrrr[i]=newrrrr[i]|(dest[selector]<<y);
		    		if (y==0)  setXYShuf(parts);
		    		else y-=x;
		    		selector++;
		    	}
		    }
		     
		    //in ZielReg schreiben 
		    
		    for (int i = 0; i < config.getK(); i++) {
			    try{
			    	this.setVRegsValue(rrrr,i, newrrrr[i]);
			    }catch(Exception e){}
			}		
	}
	
	/**
	 * This method helps Vshuf to execute.
	 */
	private void doPermutationShuf(final int parts,int k1,int[]selectVW,int[]perms,int rrrr,int[][] sourceVW){
		  int end=0;
		  if (parts==1) end=k1;
		  else end=4;
		  int source[][]=new int [end][parts];
		  int dest[][]=new int [end][parts];
		  for (int i = 0; i < end; i++) {//source registern lesen
			  for (int j = 0; j < (parts); j++) {
				  try{
					  source[i][j]=sourceVW[selectVW[i%4]][j+(i*(parts))];
				  }catch(Exception e){}
				
			  }
	      }
		  for (int i = 0; i < end; i++) {//register daten mischen
				for (int j = 0; j < (parts); j++) {
					dest[perms[i%4]+((i/4)*4)][j]=source[i][j];
				}
		  }
		  int [] newrrrr=new int[k1];
		  for (int i = 0; i < end; i++) {// gemischte datei in ziel register speichern
				for (int j = 0; j < parts; j++) {
					try{
					    newrrrr[j+(i*(parts))]= dest[i][j];
					}catch(Exception e){}
				}
				
		  }
		  for (int i = 0; i < config.getK(); i++) {
			  try{
			    	this.setVRegsValue(rrrr,i, newrrrr[i]);
			    }catch(Exception e){}
		  }	
	}
	
	/**
	 * This method is used to execute a JUMP-Operation. 
	 * This could also be a NOP/VNOP Operation.
	 * @param dd
	 * @param ssh
	 * @param tt
	 * @param instr
	 * @throws Exception
	 */
	private void executeJMP_NOPOperation(int instruction, ScalarRegister dd, ScalarRegister ss, ScalarRegister tt, Instruction instr) throws Exception{
		switch (instr) {
		case JMP:
			if(dd==null) {
				 this.stats.updateStatistics("Jmp");
			}
			else{
				this.stats.updateStatistics("Jal");
				dd.setValue(this.ip+1);

			}
			jump(dd,ss,tt, instruction );
			break;
		case JNC:
			if (this.carryFlag==0){
				jump(dd,ss,tt, instruction);
				this.stats.updateStatistics("Jnc");
			}
			else {
				this.incIP();
			}
			break;
		case JC:			
			if (this.carryFlag==1){
				jump(dd,ss,tt, instruction);
				this.stats.updateStatistics("Jc");
			}
			else {
				this.incIP();
			}
			break;
		case JNZ:
			if (!this.zeroFlag){
				jump(dd,ss,tt, instruction);
				this.stats.updateStatistics("Jnz");
			}
			else {
				this.incIP();
			}
			break;
		case JZ:
			if (this.zeroFlag){
				jump(dd,ss,tt, instruction);
				this.stats.updateStatistics("Jz");
			}
			else {
				this.incIP();
			}
			break;
		}
	}

	/**
	 * This method is used to execute a jmpOperation.
	 * It fetches the bits in the 32bit word "instr".
	 * This number is the programCounter to jump to 
	 * in the next instruction, so it sets the new IP.
	 * @param dd - destination register
	 * @param ss - source register 1
	 * @param tt - source register 2
	 * @param instr - the 32bit word
	 */
	private void jump(ScalarRegister dd, ScalarRegister ss, ScalarRegister tt, int instr){
	    //Jump-add holen: 
		int jump = instr & 0x1FFFF; //ersten 8bit (instr<<15) >>> 15;
		
	     if (ss==null){
			 if (tt==null){
	    		 this.setIP(jump);
	    	 }
	    	 else{	    		
	    		 this.setIP(jump+ tt.getValue());    		 
	    	 }
	     }
	     else {
	    	 if (tt==null){
	    		 this.setIP(ss.getValue()+jump);
	    	 }
	    	 else{
	    		 this.setIP((int)ss.getValue()+ tt.getValue());
	    	 }
	     }
	}
	/**
	 * This method is used to execute a FLAG Operation, 
	 * depending on the given enum instruction.
	 * @param instr - Instruction enum
	 * @throws Exception
	 */
    private void executeFLAGOperation(Instruction instr) throws Exception {

		switch (instr) {
		case CLC:
			setStatusRegs(this.zeroFlag,0);//delete carry
			this.stats.updateStatistics("Clc");
			break;
		case SEC:
			setStatusRegs(this.zeroFlag, 1);//set carry
			this.stats.updateStatistics("Sec");
			break;
		case CLZ:
			setStatusRegs(false, this.carryFlag);
			this.stats.updateStatistics("Clz");
			break;
		case SEZ:
			setStatusRegs(true, this.carryFlag);
			this.stats.updateStatistics("Sez");
			break;
		}
    }



} // end of class processor
