package hicosim.components.frontend;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

/**
 * @author Daniel Obermueller & Minning Sebastian 3rd May 2009
 */
/**
 * This class extends the super class JTextPane and provides a method to append
 * code in a specified color.
 */
public class CustomJTextPane extends JTextPane {
	/**
	 *
	 */
	private static final long serialVersionUID = 1L;
	private DefaultStyledDocument m_defaultStyledDocument = new DefaultStyledDocument();

	public CustomJTextPane() {
		setDocument(m_defaultStyledDocument);
	}

	/**
	 * This method needs to be overwritten in order to be able to disable the
	 * line wrapping. It disables the scrollable tracks viewport width.
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/**
	 * This method needs to be overwritten in order to be able to disable the
	 * line wrapping. It sets the appropriate size of the JTextPane.
	 */
	public void setSize(Dimension d) {
		if (d.width < getParent().getSize().width) {
			d.width = getParent().getSize().width;
		}
		super.setSize(d);
	}

	/**
	 * This method appends a String-Object in a specified color to the existing
	 * text of the JTextPane.
	 */
	public void append(String message, Color color) {
		try {
			SimpleAttributeSet simpleAttributeSet = new SimpleAttributeSet();
			StyleConstants.setForeground(simpleAttributeSet, color);
			m_defaultStyledDocument.insertString(m_defaultStyledDocument
					.getLength(), message, simpleAttributeSet);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method appends a String-Object to the existing text of the
	 * JTextPane.
	 */
	public void append(String message) {
		append(message, Color.BLACK);
	}

	public static void main(String[] args) {
		JFrame f = new JFrame();
		CustomJTextPane pane = new CustomJTextPane();
		pane.append("hello", Color.BLACK);
		pane.append("hello", Color.RED);
		f.getContentPane().add(pane);
		f.setBounds(100, 100, 200, 200);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);
	}
}
