/**
 *
 */
package hicosim.components.frontend;

import java.awt.Color;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Font;
import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.*;
import javax.swing.JTextField;
import javax.swing.JTable;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Vector;

import hicosim.control.Engine;
import hicosim.components.backend.io.CommandProcessor;
import hicosim.components.frontend.dialogues.CustomJDialog;
import hicosim.components.frontend.tablecellrenderers.ColoredTableCellRendererScalarCpuRegisterOverview;
import hicosim.components.frontend.tablecellrenderers.ColoredTableCellRendererVectorCpuRegisterOverview;
import hicosim.components.frontend.tablemodels.CustomAbstractTableModelWithEditiableCells;
import hicosim.components.frontend.tablemodels.CustomDefaultTableModelScalarCpuRegisterOverview;
import hicosim.components.plugins.vga.VGAPlugin;

/*
 * edit Andi: Statistics
 *
 */
import hicosim.control.utils.*;

/**
 * @author Daniel Obermueller & Sebastian Minning
 *
 */
public class MainWindow extends JFrame implements iListenerFrontend,
		IStatisticObserver {

	private static final long serialVersionUID = 1L;

	private Engine engine;
	RunThread thread = null;

	private JMenuBar jMenuBar = null;

	private JMenu jMenuFile = null;
	private JMenuItem jMenuItemOpenSourceCode = null;
	private JMenuItem jMenuItemSaveSourceCode = null;
	private JMenuItem jMenuItemOpenObjectFile = null;
	private JMenuItem jMenuItemSaveStatistics = null;
	private JMenuItem jMenuItemExit = null;

	private JMenu jMenuView = null;
	private JMenuItem jMenuItemHideVectorRegisters = null;
	private JMenuItem jMenuItemShowVectorRegisters = null;
	private JMenuItem jMenuItemCreateToolBar = null;

	private JMenu jMenuSimulate = null;
	private JMenuItem jMenuItemRun = null;
	private JMenuItem jMenuItemStep = null;
	private JMenuItem jMenuItemInterrupt = null;
	private JMenuItem jMenuItemReset = null;
	private JMenuItem jMenuItemResetMemory = null;
	private JMenuItem jMenuItemResetCpu = null;
	private JMenuItem jMenuItemSetBreakpoint = null;
	private JMenuItem jMenuItemDeleteBreakpoint = null;
	private JMenuItem jMenuItemAssemble = null;

	private JMenu jMenuMemory = null;
	private JMenuItem jMenuItemAddMemoryOverview = null;
	private JMenuItem jMenuItemDeleteMemoryOverview = null;
	// private JMenuItem jMenuItemSetMemoryAddress = null;

	private JMenu jMenuPlugins = null;
	private JMenuItem jMenuItemToggleVgaPlugin = null;

	private JMenu jMenuHelp = null;
	private JMenuItem jMenuItemAbout = null;
	private JMenuItem jMenuItemManual = null;

	private JSplitPane jSplitPane = null;
	private JSplitPane leftAndCenterJSplitPane = null;
	private JSplitPane leftJSplitPane = null;
	private JSplitPane centerJSplitPane = null;
	private JSplitPane rightJSplitPane = null;

	private JTabbedPane jTabbedPaneRegisterOverview = null;
	private JSplitPane jSplitPaneCpuRegisters = null;
	private JTable jTableScalarCpuRegisterOverview = null;
	private JTable jTableVectorCpuRegisterOverview = null;
	private JTabbedPane jTabbedPaneMemoryOverview = null;
	private LinkedList<JPanelMemoryOverview> jPanelMemoryOverviewList = null;
	private CustomAbstractTableModelWithEditiableCells customAbstractTableModelVectorCpuRegisterOverview = null;
	private JScrollPane jScrollPaneCpuRegisters = null;

	private JTabbedPane jTabbedPanePlugIn = null;

	private JTabbedPane jTabbedPaneConfiguration = null;
	private JTable jTableCpuConfiguration = null;

	private JTabbedPane jTabbedPaneStatistics = null;
	private JTable jTableCodeStatistics = null;

	private JTabbedPane jTabbedPaneCode = null;
	private JTextAreaWithLineNumbering jTextAreaSourceCode = null;
	private JTextAreaWithLineNumbering jTextAreaDisassembledCode = null;
	private JSplitPane jSplitPaneTopAndCenterBreakpointOverview = null;
	private JSplitPane jSplitPaneCenterAndBottomBreakpointOverview = null;
	private JTextArea jTextAreaBreakpointAtLine = null;
	private JTextArea jTextAreaBreakpointAtLabel = null;
	private JTextArea jTextAreaBreakpointAtAdress = null;

	private JPanel jPanel = null;
	private JTextField jTextFieldCommandLine = null;
	private LinkedList<String> bufferCommandLine = new LinkedList<String>();
	private ListIterator<String> iteratorCommandLine;
	private CustomJTextPane jTextAreaConsole;

	private CommandProcessor cmdProcessor = null;

	private HashMap<String, Integer> statsComands = null;
	private VGAPlugin vgaPlugIn = null;

	private JFrame toolBarFrame = null;

	/**
	 * @throws HeadlessException
	 */
	public MainWindow(Engine engine, CommandProcessor cmdProcessor)
			throws HeadlessException {
		super();
		this.engine = engine;
		initialize();

		this.cmdProcessor = cmdProcessor;
		this.engine.getProcessor().getStatistics().registerObserver(this);

	}

	/**
	 * @param arg0
	 */
	public MainWindow(GraphicsConfiguration arg0) {
		super(arg0);
		initialize();
	}

	/**
	 * @param arg0
	 * @throws HeadlessException
	 */
	public MainWindow(String arg0) throws HeadlessException {
		super(arg0);
		initialize();
	}

	/**
	 * @param arg0
	 * @param arg1
	 */
	public MainWindow(String arg0, GraphicsConfiguration arg1) {
		super(arg0, arg1);
		initialize();
	}

	private class ToolBar extends JPanel
	    implements ActionListener {

		private static final long serialVersionUID = 1L;

		static final private String RUN = "run";
		static final private String STEP = "step";
		static final private String INTERRUPT = "interrupt";
		static final private String RESETCPU = "reset cpu";
		static final private String RESETMEM = "reset memory";
		static final private String BREAKADD = "add breakpoint";
		static final private String BREAKREM = "remove breakpoint";
		static final private String BREAKREMALL = "remove all breakpoints";
		static final private String ASSEMBLE = "assemble";
		static final private String BIN = "bin";
		static final private String DEC = "dec";
		static final private String HEX = "hex";

		/**
		 * This method generates the ToolBar "ToolBar".
		 *
		 * @author Sebastian Minning
		 */
		public ToolBar() {
			super(new BorderLayout());

			JToolBar toolBar = new JToolBar("ToolBar");
			addButtons(toolBar);

			//Lay out the main panel.
			//setPreferredSize(new Dimension(450, 130));
			add(toolBar, BorderLayout.PAGE_START);
		}

		/**
		 * This method adds the buttons to the ToolBar "addButtons".
		 *
		 * @author Sebastian Minning
		 */
		protected void addButtons(JToolBar toolBar) {
			JButton button = null;

			//start
			button = makeNavigationButton("start_3_small", RUN, "Run", "Run");
			toolBar.add(button);

			//step
			button = makeNavigationButton("continue_3_small", STEP, "Step", "Step");
			toolBar.add(button);

			//interrupt
			button = makeNavigationButton("interrupt_2_small", INTERRUPT, "Interrupt", "Interrupt");
			toolBar.add(button);

			//reset cpu
			button = makeNavigationButton("resetcpu_1_small", RESETCPU, "Reset CPU", "Reset CPU");
			toolBar.add(button);

			//reset memory
			button = makeNavigationButton("resetmem_1_small", RESETMEM, "Reset Memory", "Reset Memory");
			toolBar.add(button);

			//add breakpoint
			button = makeNavigationButton("breakpoint_1_small", BREAKADD, "Add Breakpoint", "Breakpoint");
			toolBar.add(button);

			//delete breakpoint
			button = makeNavigationButton("deletebreakpoint_1_small", BREAKREM, "Delete Breakpoint", "Delete Breakpoint");
			toolBar.add(button);

			//delete all breakpoints
			button = makeNavigationButton("deletebreakpointall_1_small", BREAKREMALL, "Delete all Breakpoints", "Delete all Breakpoints");
			toolBar.add(button);

			//assemble
			button = makeNavigationButton("assemble_1_small", ASSEMBLE, "Assemble", "Assemble");
			toolBar.add(button);

			//bin
			button = makeNavigationButton("bin_1_small", BIN, "bin", "bin");
			toolBar.add(button);

			//dec
			button = makeNavigationButton("dec_1_small", DEC, "dec", "dec");
			toolBar.add(button);

			//hex
			button = makeNavigationButton("hex_1_small", HEX, "hex", "hex");
			toolBar.add(button);
		}

		protected JButton makeNavigationButton(String imageName,
		                      String actionCommand,
		                      String toolTipText,
		                      String altText) {

			// #################### TODO ! images in jarFile mit einfuegen !!! #################
			String imgLocation = System.getProperty("user.dir") + "/icons/"
			        + imageName
			        + ".gif";
			
//			URL imageURL = ToolBar.class.getResource(tmp.toString());
			URL imageURL = ToolBar.class.getResource(imgLocation);

			//Image kreieren:
//			ImageIcon tmp =null;
//			try {
//				tmp = new ImageIcon(getClass().getResource("icons/" + imageName +".gif"), altText);
//			}
//			catch (Exception e) {
//				System.err.println("ERROR while creating ImageIcon !!!");
//			}
			
			//Create and initialize the button.
			JButton button = new JButton();
			button.setActionCommand(actionCommand);
			button.setToolTipText(toolTipText);
			button.addActionListener(this);

			if (/*tmp*/ imageURL != null) {                      //image found
			button.setIcon( /*tmp*/   new ImageIcon(imageURL, altText));
			} 
			else {                                     //no image found
			button.setText(altText);

			//Images not found:
			//TODO so klappt es nicht!
//			System.err.println("Toolbar: Resource not found: "+ imgLocation + " - Using default text for buttons.");
			}

			return button;
		}	String description = null;

		/**
		 * This method handles the ToolBar actions "actionPerformed".
		 *
		 * @author Sebastian Minning
		 */
		public void actionPerformed(ActionEvent event) {
			String cmd = event.getActionCommand();

		try {
			// Handle each button.
			if (RUN.equals(cmd)) { //nur wenn oFile geladen:
				if (engine.getProcessor().isObjectFileLoaded()) {
					engine.getProcessor().setIsRunning(true);//run
				
				RunThread t = new RunThread(engine);
				t.start();
				}
				else 
					displayConsoleError("NO OBJECTFILE LOADED");

			}
			else if (STEP.equals(cmd)) { 			//step
				engine.step(false);
			}
			else if (INTERRUPT.equals(cmd)) { 	//interrupt
				engine.interrupt();
			}
			else if (RESETCPU.equals(cmd)) { 		//reset cpu
				engine.reset();
			}
			else if (RESETMEM.equals(cmd)) { 		//reset memory
				engine.resetMemory();
			}
			else if (BREAKADD.equals(cmd)) { 		//add breakpoint
				jMenuItemSetBreakpoint.doClick();
			}
			else if (BREAKREM.equals(cmd)) { 		//remove breakpoint
				jMenuItemDeleteBreakpoint.doClick();
			}
			else if (BREAKREMALL.equals(cmd)) { 	//remove all breakpoints
				try {
					displayConsoleMessage(cmdProcessor.processGUI_Input("delete"));
				}
				catch (Exception ex){
					displayConsoleError(ex.getMessage());
				}
			} else if (ASSEMBLE.equals(cmd)) { 		//assemble
				jMenuItemAssemble.doClick();
			}else if (BIN.equals(cmd)) { 			//BIN
				try {
					engine.setNumerative('b');
				}
				catch (Exception b){

				}
			}else if (DEC.equals(cmd)) { 			//DEC
				try {
						engine.setNumerative('d');
					}
					catch (Exception d){

					}
			}else if (HEX.equals(cmd)) { 			//HEX
				try {
					engine.setNumerative('h');
					}
					catch (Exception h){

					};
			}
		}
		catch (Exception e) {
			displayConsoleError("ERROR in Toolbar --> " + e.getMessage());
		}

		//nach jedem Klick auf Toolbar wieder focus auf CommandLine setzen!
		focusAtCommandline(); 
		}
	}

	/**
	 * This method creates the JToolBar "ToolBar".
	 *
	 * @author Sebastian Minning
	 */
    private void createToolBar() {
    	//Create and set up the window
    	JFrame ToolBarFrame = new JFrame("ToolBar");
    	ToolBarFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    	ToolBarFrame.add(new ToolBar());
    	ToolBarFrame.pack();
    	ToolBarFrame.setVisible(true);
    	ToolBarFrame.setAlwaysOnTop(true);
    }

	/**
	 * This method initializes the JMenuBar "jMenuBar".
	 *
	 * @author Daniel Obermueller
	 */
	private void initializeJMenuBar() {
		if (jMenuBar == null) {
			jMenuBar = new JMenuBar();
			jMenuBar.setPreferredSize(new Dimension(1024, 20));
			jMenuBar
					.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			jMenuBar.add(getJMenuFile());
			jMenuBar.add(getJMenuView());
			jMenuBar.add(getJMenuSimulate());
			jMenuBar.add(getJMenuMemory());
			jMenuBar.add(getJMenuPlugins());
			jMenuBar.add(getJMenuHelp());
			jMenuBar.setVisible(true);
			this.setJMenuBar(jMenuBar);
		}
	}

	/**
	 * This method initializes the JMenu "jMenuFile".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuFile() {
		if (jMenuFile == null) {
			jMenuFile = new JMenu("File");
			jMenuFile.setPreferredSize(new Dimension(35, 20));
			jMenuFile.setMnemonic('F');
			jMenuFile.add(getJMenuItemOpenSourceCode());
			jMenuFile.add(getJMenuItemOpenObjectFile());
			jMenuFile.add(getJMenuItemSaveSourceCode());
			jMenuFile.add(getJMenuItemSaveStatistics());
			jMenuFile.add(getJMenuItemExit());
		}
		return jMenuFile;
	}

	/**
	 * This method displays the passed String-Object on the passed component,
	 * which is identified by its name.
	 *
	 * @author Daniel Obermueller 21st April 2009
	 * @param String
	 *            text
	 * @param String
	 *            componentTitle
	 */
	private void displayText(String text, String componentTitle) {
		if (componentTitle.equals("jTextAreaSourceCode"))
			jTextAreaSourceCode.setText(text);
		else if (componentTitle.equals("jTextAreaDisassembledCode"))
			jTextAreaDisassembledCode.setText(text);
	}

	/**
	 * This method initializes the jMenuItem "jMenuItemOpenSourceCode".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemOpenSourceCode() {
		if (jMenuItemOpenSourceCode == null) {
			jMenuItemOpenSourceCode = new JMenuItem("Open source code");
			jMenuItemOpenSourceCode.setPreferredSize(new Dimension(55, 20));
			jMenuItemOpenSourceCode
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage("Opening source Code.");
							final String FILEEXTENSION = ".asm";
							JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
							chooser.setDialogTitle("Open source Code");
							chooser.setMultiSelectionEnabled(false);
							chooser.setAcceptAllFileFilterUsed(false);
							chooser.addChoosableFileFilter(new FileFilter() {

								/**
								 * This method adds a description on the
								 * JFileChooser.
								 */
								public String getDescription() {
									return "Assembler files (*" + FILEEXTENSION
											+ ")";
								}

								/**
								 * This method guarantees that only directories
								 * or files which end with ".asm" will be
								 * accepted.
								 */
								@Override
								public boolean accept(File f) {
									return f.isDirectory()
											|| f.getName().toLowerCase()
													.endsWith(FILEEXTENSION);
								}
							});
							if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
								File file = chooser.getSelectedFile();
								if (!file.toString().toLowerCase().endsWith(
										FILEEXTENSION))
									displayConsoleError(file.getName()
											+ " can't be opened! Invalid file extension!");
								else {
									try {
										engine.readSourcefile(file
												.getAbsolutePath());
										displayConsoleMessage("File "
												+ file.getName() + " opened.");
									} catch (Exception ex) {
										displayConsoleError(ex.getMessage());
									}
									try {
										displayText(
												engine.getSourceCodeLines(),
												"jTextAreaSourceCode");
									} catch (Exception ex) {
										displayConsoleError(ex.getMessage());
									}
								}
							} else
								displayConsoleMessage("Aborted.");
						}
					});
		}
		return jMenuItemOpenSourceCode;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemSaveSourceCode".
	 *
	 * @author Daniel Obermueller 21th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemSaveSourceCode() {
		if (jMenuItemSaveSourceCode == null) {
			jMenuItemSaveSourceCode = new JMenuItem();
			jMenuItemSaveSourceCode.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_S, java.awt.Event.CTRL_MASK));
			jMenuItemSaveSourceCode.setText("Save source code as ...");
		}
		jMenuItemSaveSourceCode
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						displayConsoleMessage("Saving source code as ...");
						final String FILEEXTENSION = ".asm";
						JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
						chooser.setMultiSelectionEnabled(false);
						chooser.setAcceptAllFileFilterUsed(false);
						chooser.addChoosableFileFilter(new FileFilter() {
							/**
							 * This method adds a description on the
							 * JFileChooser
							 */
							public String getDescription() {
								return "Assembler files (*" + FILEEXTENSION
										+ ")";
							}

							/**
							 * This method guarantees that only directories or
							 * files which end with ".asm" will be accepted
							 */
							@Override
							public boolean accept(File f) {
								return f.isDirectory()
										|| f.getName().toLowerCase().endsWith(
												FILEEXTENSION);
							}
						});
						if (chooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
							File file = chooser.getSelectedFile();
							String path = chooser.getSelectedFile().getPath()
									.toLowerCase();
							String pathAndFileExtensionSplit[] = splitPathFromFileExtension(path);
							boolean selection = false;
							/* If the file got a file extension. */
							if (isFileExtension(pathAndFileExtensionSplit[1])) {
								/* If the file extension is not ".asm". */
								if (!pathAndFileExtensionSplit[1]
										.equals(FILEEXTENSION)) {
									displayConsoleError("Invalid file type "
											+ pathAndFileExtensionSplit[1]
											+ "!");
									return;
								} else if (file.exists())
									selection = showOverwriteDialog(file
											.getName());
								else {
									writeSourceCodeToFile(file);
									displayConsoleMessage("Saved source code to "
											+ file.getName());
									return;
								}
							}
							/* Else if the file got no file extension. */
							else if (!isFileExtension(pathAndFileExtensionSplit[1])) {
								path += FILEEXTENSION;
								file = new File(path);
								if (file.exists())
									selection = showOverwriteDialog(file
											.getName());
								else {
									writeSourceCodeToFile(file);
									displayConsoleMessage("Saved source code to "
											+ file.getName());
									return;
								}
							}
							/*
							 * If the user does not want to overwrite the file.
							 */
							if (!selection) {
								displayConsoleMessage("Aborted.");
								return;
							} else {
								writeSourceCodeToFile(file);
								displayConsoleMessage("Saved source code to "
										+ file.getName());
								return;
							}
						} else
							displayConsoleMessage("Aborted.");
					}
				});
		return jMenuItemSaveSourceCode;
	}

	/**
	 * This method writes the current content of the JTextArea
	 * "jTextAreaSourceCode" in the passed file "file".
	 *
	 * @author Daniel Obermueller 21th April 2009
	 * @param File
	 *            file
	 */
	private void writeSourceCodeToFile(File file) {
		try {
			BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(
					file));
			bufferedWriter.write(jTextAreaSourceCode.getText());
			bufferedWriter.flush();
			bufferedWriter.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemOpenObjectFile".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemOpenObjectFile() {
		if (jMenuItemOpenObjectFile == null) {
			jMenuItemOpenObjectFile = new JMenuItem("Open object file");
			jMenuItemOpenObjectFile.setPreferredSize(new Dimension(55, 20));
			jMenuItemOpenObjectFile.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_O, java.awt.Event.CTRL_MASK));
			jMenuItemOpenObjectFile
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage("Opening object file.");
							final String FILEEXTENSION = ".o";
							JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
							chooser.setDialogTitle("Open object file");
							chooser.setMultiSelectionEnabled(false);
							chooser.setAcceptAllFileFilterUsed(false);
							chooser.addChoosableFileFilter(new FileFilter() {

								/**
								 * This method adds a description on the
								 * JFileChooser.
								 */
								public String getDescription() {
									return "Object files (*" + FILEEXTENSION
											+ ")";
								}

								/**
								 * This method guarantees that only directories
								 * or files which end with ".o" will be
								 * accepted.
								 */
								@Override
								public boolean accept(File f) {
									return f.isDirectory()
											|| f.getName().toLowerCase()
													.endsWith(FILEEXTENSION);
								}
							});
							if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
								File file = chooser.getSelectedFile();
								if (!file.toString().toLowerCase().endsWith(
										FILEEXTENSION))
									displayConsoleError(file.getName()
											+ " can't be opened! Invalid file extension!");
								else {
									String text;
									try {
										text = engine.loadObjectFile(file
												.getAbsolutePath());
										displayConsoleMessage(text);
										displayConsoleMessage("File "
												+ file.getName() + " opened.");
									} catch (Exception ex) {
										displayConsoleError(ex.getMessage());
									}
								}

							} else
								displayConsoleMessage("Aborted.");
						}
					});
		}
		return jMenuItemOpenObjectFile;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemSaveStatistics"
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemSaveStatistics() {
		if (jMenuItemSaveStatistics == null) {
			jMenuItemSaveStatistics = new JMenuItem();
			jMenuItemSaveStatistics.setText("Save statistics as ...");
			jMenuItemSaveStatistics
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage("Saving statistics as ...");
							final String FILEEXTENSION = ".txt";
							JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
							chooser.setMultiSelectionEnabled(false);
							chooser.setAcceptAllFileFilterUsed(false);
							chooser.addChoosableFileFilter(new FileFilter() {
								/**
								 * This method adds a description on the
								 * JFileChooser
								 */
								public String getDescription() {
									return "Text files (*" + FILEEXTENSION
											+ ")";
								}

								/**
								 * This method guarantees that only directories
								 * or files which end with ".txt" will be
								 * accepted
								 */
								@Override
								public boolean accept(File f) {
									return f.isDirectory()
											|| f.getName().toLowerCase()
													.endsWith(FILEEXTENSION);
								}
							});
							if (chooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
								File file = chooser.getSelectedFile();
								String path = chooser.getSelectedFile()
										.getPath().toLowerCase();
								String pathAndFileExtensionSplit[] = splitPathFromFileExtension(path);
								boolean selection = false;
								/* If the file got a file extension. */
								if (isFileExtension(pathAndFileExtensionSplit[1])) {
									/* If the file extension is not ".txt". */
									if (!pathAndFileExtensionSplit[1]
											.equals(FILEEXTENSION)) {
										displayConsoleError("Invalid file type "
												+ pathAndFileExtensionSplit[1]
												+ "!");
										return;
									} else if (file.exists())
										selection = showOverwriteDialog(file
												.getName());
									else {
										writeStatisticsToFile(file);
										displayConsoleMessage("Saved statistics to "
												+ file.getName());
										return;
									}
								}
								/* Else if the file got no file extension. */
								else if (!isFileExtension(pathAndFileExtensionSplit[1])) {
									path += FILEEXTENSION;
									file = new File(path);
									if (file.exists())
										selection = showOverwriteDialog(file
												.getName());
									else {
										writeStatisticsToFile(file);
										displayConsoleMessage("Saved statistics to "
												+ file.getName());
										return;
									}
								}
								/*
								 * If the user does not want to overwrite the
								 * file.
								 */
								if (!selection) {
									displayConsoleMessage("Aborted.");
									return;
								} else {
									writeStatisticsToFile(file);
									displayConsoleMessage("Saved statistics to "
											+ file.getName());
									return;
								}
							} else
								displayConsoleMessage("Aborted.");
						}
					});
		}
		return jMenuItemSaveStatistics;
	}

	/**
	 * This method writes the statistics which are stored in the JTable
	 * "jTableCodeStatistics" to the passed File "file".
	 *
	 * @author Daniel Obermueller 13th May 2009 & Sebastian Minning
	 * @param File
	 *            file
	 */
	private void writeStatisticsToFile(File file) {
		try {
			BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(
					file));
			StringBuffer stringBuffer = new StringBuffer();
			for (int i = 0; i < jTableCodeStatistics.getRowCount(); i++) {
				for (int j = 0; j < jTableCodeStatistics.getColumnCount(); j++)
					stringBuffer.append(jTableCodeStatistics.getValueAt(i, j)
							+ " ");
				stringBuffer.append("\n");
			}
			bufferedWriter.write(stringBuffer.toString());
			bufferedWriter.flush();
			bufferedWriter.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * This method checks out if a String is a file extension.
	 *
	 * @author Daniel Obermueller 5th April 2009
	 * @return boolean
	 * @param String
	 *            expression
	 *
	 */
	private boolean isFileExtension(String expression) {
		final String REGEX = "\\....";
		boolean isFileExtension = expression.matches(REGEX);
		return isFileExtension;
	}

	/**
	 * This method splits a String, which consists of a path concatenated with a
	 * file extension, in two parts, the afore named ones. (e.g. "path/path.txt"
	 * in "path/path" and ".txt"). Afterwards the two Strings are returned in a
	 * String array.
	 *
	 *
	 * @author Daniel Obermueller 5th April 2009
	 * @return String[]
	 * @param String
	 *            pathAndFileExtensionConcatenated
	 */
	private String[] splitPathFromFileExtension(
			String pathAndFileExtensionConcatenated) {
		String path = pathAndFileExtensionConcatenated.substring(0,
				pathAndFileExtensionConcatenated.length() - 4);
		String fileExtension = pathAndFileExtensionConcatenated
				.substring(pathAndFileExtensionConcatenated.length() - 4);
		String pathAndFileExtensionSplit[] = { path, fileExtension };
		return pathAndFileExtensionSplit;
	}

	/**
	 * This method shows a confirm dialog and returns true, if the "YES" button
	 * was pushed and false, if the "NO" button was pushed.
	 *
	 * @author Daniel Obermueller 7th May 2009
	 * @return boolean
	 * @param String
	 *            file
	 */
	private boolean showOverwriteDialog(String file) {
		displayConsoleMessage("Overwriting " + file + " ...");
		if (JOptionPane.showConfirmDialog(null,
				"Do you really want to override " + file + "?", null,
				JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
			displayConsoleMessage("Aborted.");
			return false;
		}
		return true;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemExit".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemExit() {
		if (jMenuItemExit == null) {
			jMenuItemExit = new JMenuItem("Exit");
			jMenuItemExit.setPreferredSize(new Dimension(55, 20));
			jMenuItemExit.setMnemonic('X');
			jMenuItemExit.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_Q, java.awt.Event.CTRL_MASK));
			jMenuItemExit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							System.exit(JFrame.DISPOSE_ON_CLOSE);
						}
					});
		}
		return jMenuItemExit;
	}

	/**
	 * This method initializes the JMenu "jMenuView".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuView() {
		if (jMenuView == null) {
			jMenuView = new JMenu("View");
			jMenuView.setPreferredSize(new Dimension(40, 20));
			jMenuView.setMnemonic('V');
			jMenuView.add(getJMenuItemHideVectorRegisters());
			jMenuView.add(getJMenuItemShowVectorRegisters());
			jMenuView.add(getJMenuItemCreateToolBar());
		}
		return jMenuView;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemHideVectorRegisters".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemHideVectorRegisters() {
		if (jMenuItemHideVectorRegisters == null) {
			jMenuItemHideVectorRegisters = new JMenuItem();
			jMenuItemHideVectorRegisters.setText("Hide vector registers ...");
			final MainWindow mainWindow = this;
			jMenuItemHideVectorRegisters
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							CustomJDialog jDialog = new CustomJDialog(
									mainWindow, "Hide vector registers");
							try {
								String[] input = jDialog.prompt();
								if (input == null) {
									displayConsoleMessage("Aborted.");
									return;
								}
								Integer startIndex = (Integer
										.parseInt(input[1]));
								if (startIndex <= 0) {
									String upperBorder = null;
									if (input[0].equals("rows"))
										upperBorder = ""
												+ (customAbstractTableModelVectorCpuRegisterOverview
														.getAmountOfRows() - 1);
									else
										upperBorder = ""
												+ (customAbstractTableModelVectorCpuRegisterOverview
														.getAmountOfColumns() - 1);
									throw new Exception(
											("Invalid input for "
													+ input[0]
													+ "! Please type in integer values between 1 and "
													+ upperBorder + " only!")
													.toUpperCase());
								}
								Integer endIndex = (Integer.parseInt(input[2])) + 1;
								if (input[0].equals("rows"))
									for (int i = startIndex; i < endIndex; i++)
										customAbstractTableModelVectorCpuRegisterOverview
												.hideRow(i);
								else
									for (int i = startIndex; i < endIndex; i++)
										customAbstractTableModelVectorCpuRegisterOverview
												.hideColumn(i);
							} catch (NumberFormatException ex) {
								displayConsoleError("Invalid input! Please type in integer values only!"
										.toUpperCase());
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemHideVectorRegisters;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemShowVectorRegisters".
	 *
	 * @author Daniel Obermueller 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemShowVectorRegisters() {
		if (jMenuItemShowVectorRegisters == null) {
			jMenuItemShowVectorRegisters = new JMenuItem();
			jMenuItemShowVectorRegisters.setText("Show vector registers ...");
			final MainWindow mainWindow = this;
			jMenuItemShowVectorRegisters
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							CustomJDialog jDialog = new CustomJDialog(
									mainWindow, "Show vector registers");
							try {
								String[] input = jDialog.prompt();
								if (input == null) {
									displayConsoleMessage("Aborted.");
									return;
								}
								Integer startIndex = (Integer
										.parseInt(input[1]));
								if (startIndex <= 0) {
									String upperBorder = null;
									if (input[0].equals("rows"))
										upperBorder = ""
												+ (customAbstractTableModelVectorCpuRegisterOverview
														.getAmountOfRows() - 1);
									else
										upperBorder = ""
												+ (customAbstractTableModelVectorCpuRegisterOverview
														.getAmountOfColumns() - 1);
									throw new Exception(
											("Invalid input for "
													+ input[0]
													+ "! Please type in integer values between 1 and "
													+ upperBorder + " only!")
													.toUpperCase());
								}
								Integer endIndex = (Integer.parseInt(input[2])) + 1;
								if (input[0].equals("rows"))
									for (int i = startIndex; i < endIndex; i++)
										customAbstractTableModelVectorCpuRegisterOverview
												.showRow(i);
								else
									for (int i = startIndex; i < endIndex; i++)
										customAbstractTableModelVectorCpuRegisterOverview
												.showColumn(i);
							} catch (NumberFormatException ex) {
								displayConsoleError("Invalid input! Please type in integer values only!"
										.toUpperCase());
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemShowVectorRegisters;
	}

	/**
	 * This method creates the ToolBar "jMenuCreateToolBar".
	 *
	 * @author Sebastian Minning
	 */
	private JMenuItem getJMenuItemCreateToolBar() {
		if (jMenuItemCreateToolBar == null) {
			jMenuItemCreateToolBar = new JMenuItem("ToolBar");
			jMenuItemCreateToolBar.setPreferredSize(new Dimension(55, 20));

			jMenuItemCreateToolBar
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
					    	if (toolBarFrame != null){
					    		createToolBar();
					    	}
						}
					});
		}
		return jMenuItemCreateToolBar;
	}


	/**
	 * This method initializes the JMenu "jMenuSimulate".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuSimulate() {
		if (jMenuSimulate == null) {
			jMenuSimulate = new JMenu("Simulate");
			jMenuSimulate.setPreferredSize(new Dimension(65, 20));
			jMenuSimulate.setMnemonic('S');
			jMenuSimulate.add(getJMenuItemRun());
			jMenuSimulate.add(getJMenuItemStep());
			jMenuSimulate.add(getJMenuItemInterrupt());
			jMenuSimulate.add(getJMenuItemReset());
			jMenuSimulate.add(getJMenuItemResetCpu());
			jMenuSimulate.add(getJMenuItemSetBreakpoint());
			jMenuSimulate.add(getJMenuItemDeleteBreakpoint());
			jMenuSimulate.add(getJMenuItemAssemble());

		}
		return jMenuSimulate;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemRun".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemRun() {
		if (jMenuItemRun == null) {
			jMenuItemRun = new JMenuItem("Run");
			jMenuItemRun.setPreferredSize(new Dimension(85, 20));
			jMenuItemRun.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_R, java.awt.Event.CTRL_MASK));
			jMenuItemRun.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					try {
						/*
						 * If an object file is loaded, prepare the processor
						 * for the execution of it.
						 */
						if (engine.getProcessor().isObjectFileLoaded())
							engine.getProcessor().setIsRunning(true);
						/*
						 * Else throw an exception.
						 */
						else
							throw new Exception("No object file loaded!");
						/*
						 * Initialize and start a thread, which executes the
						 * loaded object file.
						 */
						if (thread == null) {
							thread = new RunThread(engine);
							thread.setPriority(Thread.MAX_PRIORITY);
							thread.start();
							displayConsoleMessage("Simulation in progress.");
						}
					} catch (Exception ex) {
						displayConsoleError(ex.getMessage());
					}
				}
			});
		}
		return jMenuItemRun;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemStep".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemStep() {
		if (jMenuItemStep == null) {
			jMenuItemStep = new JMenuItem("Step");
			jMenuItemStep.setPreferredSize(new Dimension(85, 20));
			jMenuItemStep.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_N, java.awt.Event.CTRL_MASK));
			jMenuItemStep
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								engine.step(false);
								displayConsoleMessage("Stepped to next instruction.");
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemStep;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemInterrupt".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemInterrupt() {
		if (jMenuItemInterrupt == null) {
			jMenuItemInterrupt = new JMenuItem("Interrupt");
			jMenuItemInterrupt.setPreferredSize(new Dimension(85, 20));
			jMenuItemInterrupt.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_I, java.awt.Event.CTRL_MASK));
			jMenuItemInterrupt
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								engine.interrupt();
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemInterrupt;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemReset".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemReset() {
		if (jMenuItemReset == null) {
			jMenuItemReset = new JMenuItem("Reset");
			jMenuItemReset.setPreferredSize(new Dimension(85, 20));
			jMenuItemReset
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								engine.reset();
								engine.resetMemory();
								displayConsoleMessage("CPU and memory reset.");
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemReset;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemResetCpu".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemResetCpu() {
		if (jMenuItemResetCpu == null) {
			jMenuItemResetCpu = new JMenuItem("Reset CPU");
			jMenuItemResetCpu.setPreferredSize(new Dimension(85, 20));
			jMenuItemResetCpu
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								engine.reset();
								displayConsoleMessage("CPU reset.");
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemResetCpu;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemResetMemory".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemResetMemory() {
		if (jMenuItemResetMemory == null) {
			jMenuItemResetMemory = new JMenuItem("Reset memory");
			jMenuItemResetMemory.setPreferredSize(new Dimension(85, 20));
			jMenuItemResetMemory
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								engine.resetMemory();
								displayConsoleMessage("Memory resetted.");
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
							}
						}
					});
		}
		return jMenuItemResetMemory;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemSetBreakpoint".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemSetBreakpoint() {
		if (jMenuItemSetBreakpoint == null) {
			jMenuItemSetBreakpoint = new JMenuItem("Set breakpoint ...");
			jMenuItemSetBreakpoint.setPreferredSize(new Dimension(200, 20));
			jMenuItemSetBreakpoint
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage("Setting breakpoint ...");
							String input = JOptionPane.showInputDialog(null,
									"Set breakpoint ...", null,
									JOptionPane.PLAIN_MESSAGE);
							if (input != null)
								try {
									String message = engine.setBreakpoint(
											input, false);
									if (message != null)
										displayConsoleMessage(message);
								} catch (Exception ex) {
									displayConsoleError(ex.getMessage());
								}
							else
								displayConsoleMessage("Aborted.");
						}
					});
		}
		return jMenuItemSetBreakpoint;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemDeleteBreakpoint".
	 *
	 * @author Daniel Obermueller 9th April 2009 & Sebastian Minning
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemDeleteBreakpoint() {
		if (jMenuItemDeleteBreakpoint == null) {
			jMenuItemDeleteBreakpoint = new JMenuItem("Delete breakpoint ...");
			jMenuItemDeleteBreakpoint.setPreferredSize(new Dimension(130, 20));
			jMenuItemDeleteBreakpoint
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage("Deleting breakpoint ...");
							String input = JOptionPane.showInputDialog(null,
									"Delete breakpoint ...", null,
									JOptionPane.PLAIN_MESSAGE);
							if (input != null)
								try {
									String message = engine.setBreakpoint(
											input, true);
									if (message != null)
										displayConsoleMessage(message);
								} catch (Exception ex) {
									displayConsoleError(ex.getMessage());
								}
							else
								displayConsoleMessage("Aborted.");
						}
					});
		}
		return jMenuItemDeleteBreakpoint;
	}

	/**
	 * This method displays the passed String-Object "text" in the JTextArea
	 * "jTextAreaBreakpointAtLabel".
	 *
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param String
	 *            text
	 */
	public void displayBreakpointAtLabel(String text) {
		jTextAreaBreakpointAtLabel.setText(text);
	}

	/**
	 * This method displays the passed String-Object "text" in the JTextArea
	 * "jTextAreaBreakpointAtLine".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param String
	 *            text
	 */
	public void displayBreakpointAtLine(String text) {
		jTextAreaBreakpointAtLine.setText(text);
	}

	/**
	 * This method sets a breakpoint mark at the JTextArea "jTextAreaSourceCode"
	 * at the passed index, if it is in range, else it displays an error.
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */

	public void setBreakpointMarkAtSourceCode(int index) {
		try {
			jTextAreaSourceCode.setBreakpointMark(index);
		} catch (Exception ex) {
			displayConsoleError(ex.getMessage());
		}
	}

	/**
	 * This method deletes a breakpoint mark from the JTextArea
	 * "jTextAreaSourceCode" the passed index, if it is in range, else it
	 * displays an error.
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */
	public void deleteBreakpointMarkAtSourceCode(int index) {
		try {
			jTextAreaSourceCode.deleteBreakpointMark(index);
		} catch (Exception ex) {
			displayConsoleError(ex.getMessage());
		}
	}

	/**
	 * This method sets a breakpoint mark at the JTextArea
	 * "jTextAreaDisassembledCode" at the passed index, if it is in range, else
	 * it displays an error.
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */

	public void setBreakpointMarkAtDisassembledCode(int index) {
		try {
			jTextAreaDisassembledCode.setBreakpointMark(index);
		} catch (Exception ex) {
			displayConsoleError(ex.getMessage());
		}
	}

	/**
	 * This method deletes all breakpoints marks.
	 *
	 * @author Sebastian Minning 11th May 2009
	 */
	public void deleteAllBr() {
		jTextAreaDisassembledCode.deleteAllBreakpointMarks();
		jTextAreaSourceCode.deleteAllBreakpointMarks();
	}

	/**
	 * This method deletes a breakpoint mark from the JTextArea
	 * "jTextAreaDisassembledCode" the passed index, if it is in range, else it
	 * displays an error.
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */
	public void deleteBreakpointMarkAtDisassembledCode(int index) {
		try {
			jTextAreaDisassembledCode.deleteBreakpointMark(index);
		} catch (Exception ex) {
			displayConsoleError(ex.getMessage());
		}
	}

	/**
	 * This method highlights a line of the JTextArea "jTextAreaSourceCode",
	 * which is specified by the passed Integer-Object "index".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */
	public void highlightLineOfSourceCode(int index) {
		jTextAreaSourceCode.highlightLine(index);
	}

	/**
	 * This method highlights a line of the JTextArea
	 * "jTextAreaDisassembledCode", which is specified by the passed
	 * Integer-Object "index".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param int
	 *            index
	 */
	public void highlightLineOfDisassembledCode(int index) {
		jTextAreaDisassembledCode.highlightLine(index);
	}

	/**
	 * This method displays the passed String-Object "text" in the JTextArea
	 * "jTextAreaBreakpointAtAdress".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param String
	 *            text
	 */
	public void displayBreakpointAtAdress(String text) {
		jTextAreaBreakpointAtAdress.setText(text);
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemAssemble".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemAssemble() {
		if (jMenuItemAssemble == null) {
			jMenuItemAssemble = new JMenuItem("Assemble");
			jMenuItemAssemble.setPreferredSize(new Dimension(85, 20));
			jMenuItemAssemble.setMnemonic('A');
			jMenuItemAssemble.setAccelerator(KeyStroke.getKeyStroke(
					java.awt.event.KeyEvent.VK_A, java.awt.Event.ALT_MASK));
			jMenuItemAssemble
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							String sourceCode = jTextAreaSourceCode.getText();
							if (!sourceCode.equals("")) {
								try {
									final String SEPARATOR = System
											.getProperty("file.separator");
									final String PATH = System
											.getProperty("user.dir");
									/*
									 * Create a temporary file, which contains
									 * the displayed text of the JTextArea
									 * "jTextAreaSourceCode".
									 */
									File file = new File(PATH + SEPARATOR
											+ "tmp.asm");
									FileWriter fileWriter = new FileWriter(file);
									BufferedWriter bufferedWriter = new BufferedWriter(
											fileWriter);
									bufferedWriter.write(sourceCode);
									bufferedWriter.flush();
									bufferedWriter.close();
									fileWriter.close();
									/*
									 * Execute the external program "scotchas"
									 * in order to assemble the source code.
									 */
									String message = engine
											.execute("scotchas -o " + PATH
													+ SEPARATOR + "tmp.o "
													+ file.getAbsolutePath());
									/*
									 * If there is no String returned, no
									 * exception occured, thus load the object
									 * file.
									 */
									if (message.equals("")) {
										engine.loadObjectFile(PATH + SEPARATOR
												+ "tmp.o");
										displayConsoleMessage("Source code assembled and object file opened.");
									} else
										throw new Exception(message.trim());
									/* Delete the temporary files. */
									file = new File(PATH + SEPARATOR
											+ "tmp.asm");
									file.delete();
									file = new File(PATH + SEPARATOR + "tmp.o");
									file.delete();
								} catch (InterruptedException ex) {
									ex.printStackTrace();
								} catch (IOException ex) {
									ex.printStackTrace();
								} catch (Exception ex) {
									displayConsoleError(ex.getMessage());
								}
							} else
								displayConsoleError("There is no source code available!");
						}
					});
		}
		return jMenuItemAssemble;
	}

	/**
	 * This method initializes the JMenu "jMenuMemory".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuMemory() {
		if (jMenuMemory == null) {
			jMenuMemory = new JMenu("Memory");
			jMenuMemory.setPreferredSize(new Dimension(60, 20));
			jMenuMemory.setMnemonic('M');
			jMenuMemory.add(getJMenuItemAddMemoryOverview());
			jMenuMemory.add(getJMenuItemDeleteMemoryOverview());
			jMenuMemory.add(getJMenuItemResetMemory());
			// jMenuMemory.add(getJMenuItemSetMemoryAddress());
		}
		return jMenuMemory;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemAddMemoryOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemAddMemoryOverview() {
		if (jMenuItemAddMemoryOverview == null) {
			jMenuItemAddMemoryOverview = new JMenuItem("Add memory overview");
			jMenuItemAddMemoryOverview.setPreferredSize(new Dimension(200, 20));
			jMenuItemAddMemoryOverview
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							addMemoryOverviewTab();
						}
					});
			addMemoryOverviewTab();
		}

		return jMenuItemAddMemoryOverview;
	}

	/**
	 * This method initializes the JMenuItem "jMenuItemDeleteMemoryOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemDeleteMemoryOverview() {
		if (jMenuItemDeleteMemoryOverview == null) {
			jMenuItemDeleteMemoryOverview = new JMenuItem(
					"Delete memory overview");
			jMenuItemDeleteMemoryOverview.setPreferredSize(new Dimension(200,
					20));
			jMenuItemDeleteMemoryOverview
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							deleteMemoryOverviewTab();
						}
					});
		}
		return jMenuItemDeleteMemoryOverview;
	}

	// /**
	// * This method initializes the JMenuItem "Set memory address...".
	// *
	// * @return javax.swing.JMenuItem
	// */
	// private JMenuItem getJMenuItemSetMemoryAddress() {
	// if (jMenuItemSetMemoryAddress == null) {
	// jMenuItemSetMemoryAddress = new JMenuItem(
	// "Set value at memory address...");
	// jMenuItemSetMemoryAddress.setPreferredSize(new Dimension(200, 20));
	// jMenuItemSetMemoryAddress
	// .addActionListener(new java.awt.event.ActionListener() {
	// public void actionPerformed(java.awt.event.ActionEvent e) {
	// CustomJComponent customInputDialog = new CustomJComponent(
	// "Memory address:", "Value:");
	// int selection = JOptionPane.showConfirmDialog(null,
	// customInputDialog.getComponent(),
	// "Set value at memory address",
	// JOptionPane.OK_CANCEL_OPTION);
	// if (selection == JOptionPane.OK_OPTION) {
	// String memoryAddressAsString = customInputDialog
	// .getFirstInput();
	// String valueAsString = customInputDialog
	// .getSecondInput();
	// LinkedList<Integer> data = new LinkedList<Integer>();
	// try {
	// int memoryAddress = Integer
	// .parseInt(memoryAddressAsString);
	// data.add(memoryAddress);
	// } catch (NumberFormatException ex) {
	// displayConsoleError(memoryAddressAsString
	// + " is no valid input! Please type in integer values only!");
	// }
	// try {
	// int value = Integer.parseInt(valueAsString);
	// data.add(value);
	// } catch (NumberFormatException ex) {
	// displayConsoleError(valueAsString
	// + " is no valid input! Please type in integer values only!");
	// }
	// if (data.size() == 2)
	// try {
	// engine.setMemory(data.get(0), data
	// .get(1));
	// displayConsoleMessage("Set memory address "
	// + data.get(0)
	// + " to "
	// + data.get(1) + ".");
	// } catch (Exception ex) {
	// displayConsoleError(ex.getMessage());
	// }
	// } else
	// displayConsoleMessage("Aborted.");
	//
	// }
	// });
	// }
	// return jMenuItemSetMemoryAddress;
	// }

	/**
	 * This method initializes the JMenu "jMenuPlugins".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuPlugins() {
		if (jMenuPlugins == null) {
			jMenuPlugins = new JMenu("Plug-ins");
			jMenuPlugins.setPreferredSize(new Dimension(60, 20));
			jMenuPlugins.setMnemonic('P');
			jMenuPlugins.add(getJMenuItemToggleVgaPlugin());
		}
		return jMenuPlugins;
	}

	/**
	 * This method initializes JMenuItem "jMenuItemToggleVgaPlugin".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemToggleVgaPlugin() {
		if (jMenuItemToggleVgaPlugin == null) {
			jMenuItemToggleVgaPlugin = new JMenuItem("Toggle VGA plug-in");
			jMenuItemToggleVgaPlugin.setPreferredSize(new Dimension(140, 20));
			jMenuItemToggleVgaPlugin
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							/*
							 * If there are tabs, which possibly contain the
							 * "VGA plug-in".
							 */
							if (jTabbedPanePlugIn.getTabCount() > 0)
								/* Iterate over all tabs. */
								for (int i = 0; i < jTabbedPanePlugIn
										.getTabCount(); i++)
									/*
									 * If one tab contains the "VGA plug-in",
									 * remove it.
									 */
									if (jTabbedPanePlugIn.getTitleAt(i).equals(
											"VGA plug-in")) {
										vgaPlugIn.stopRendering();
										jTabbedPanePlugIn.removeTabAt(i);
										removePluginKeyListener();
										return;
									}
							vgaPlugIn = new VGAPlugin(320, 240, engine);
							jTabbedPanePlugIn.addTab("VGA plug-in", vgaPlugIn);
							setVgaPluginKeyListener();
						}
					});
		}
		return jMenuItemToggleVgaPlugin;
	}

	private void setVgaPluginKeyListener() {
		jTabbedPanePlugIn.addKeyListener(new java.awt.event.KeyListener() {

			public void keyPressed(KeyEvent e) {
				try {
					/*
					 * If the down arrow is pressed, set the value at memory
					 * adress 65535 to 1024, in order to simulate that the
					 * center button was pressed.
					 */
					if (e.getKeyCode() == KeyEvent.VK_DOWN)
						engine.setMemory(65535, 1024);
					/*
					 * Else if the right arrow is pressed, set the value at
					 * memory adress 65535 to 128, in order to simulate that the
					 * right button was pressed.
					 */
					else if (e.getKeyCode() == KeyEvent.VK_RIGHT)
						engine.setMemory(65535, 128);
					/*
					 * Else if the left arrow iss pressed, set the value at
					 * memory adress 65535 to 16, in order to simulate that the
					 * left button was pressed.
					 */
					else if (e.getKeyCode() == KeyEvent.VK_LEFT)
						engine.setMemory(65535, 16);
				} catch (Exception ex) {
					displayConsoleError(ex.getMessage());
				}
			}

			public void keyReleased(KeyEvent e) {
				try {
					/*
					 * If the down arrow is released, set the value at memory
					 * adress 65535 to 0, in order to simulate that the center
					 * button was released.
					 */
					if (e.getKeyCode() == KeyEvent.VK_DOWN) {
						engine.setMemory(65535, 0);
					}
					/*
					 * Else if the right arrow is released, set the value at
					 * memory adress 65535 to 0, in order to simulate that the
					 * right button was released.
					 */
					else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
						engine.setMemory(65535, 0);
					}
					/*
					 * Else if the left arrow is released, set the value at
					 * memory adress 65535 to 0, in order to simulate that the
					 * left button was released.
					 */
					else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
						engine.setMemory(65535, 0);
					}
				} catch (Exception ex) {
					displayConsoleError(ex.getMessage());
				}
			}

			public void keyTyped(KeyEvent arg0) {
			}
		});
	}

	private void removePluginKeyListener() {
		for (KeyListener keyListener : jTabbedPanePlugIn.getKeyListeners())
			jTabbedPanePlugIn.removeKeyListener(keyListener);
	}

	/**
	 * This method initializes the JMenu "jMenuHelp".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuHelp() {
		if (jMenuHelp == null) {
			jMenuHelp = new JMenu("Help");
			jMenuHelp.setPreferredSize(new Dimension(40, 20));
			jMenuHelp.setMnemonic('H');
//			jMenuHelp.add(getJMenuItemManual());
			jMenuHelp.add(getJMenuItemAbout());
		}
		return jMenuHelp;
	}

	/**
	 * This method initializes JMenuItem "jMenuItemAbout".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemAbout() {
		if (jMenuItemAbout == null) {
			jMenuItemAbout = new JMenuItem("About");
			jMenuItemAbout.setPreferredSize(new Dimension(55, 20));
			jMenuItemAbout
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							displayConsoleMessage(engine.getAboutMessage());
						}
					});
		}
		return jMenuItemAbout;
	}

	/**
	 * This method initializes JMenuItem "jMenuItemManual"
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemManual() {
		if (jMenuItemManual == null) {
			jMenuItemManual = new JMenuItem("Manual");
			jMenuItemManual.setPreferredSize(new Dimension(65, 20));
			jMenuItemManual
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							// TODO
						}
					});
		}
		return jMenuItemManual;
	}

	/**
	 * This method initializes the JSplitPane "jSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setLeftComponent(getLeftAndCenterJSplitPane());
			jSplitPane.setRightComponent(getRightJSplitPane());
			jSplitPane.setDividerLocation(750);
			jSplitPane.setOneTouchExpandable(true);
		}
		return jSplitPane;
	}

	/**
	 * This method initializes the JSplitPane "leftAndCenterJSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getLeftAndCenterJSplitPane() {
		if (leftAndCenterJSplitPane == null) {
			leftAndCenterJSplitPane = new JSplitPane();
			leftAndCenterJSplitPane.setLeftComponent(getLeftJSplitPane());
			leftAndCenterJSplitPane.setRightComponent(getCenterJSplitPane());
			leftAndCenterJSplitPane.setDividerLocation(150);
			leftAndCenterJSplitPane.setOneTouchExpandable(true);
		}
		return leftAndCenterJSplitPane;
	}

	/**
	 * This method initializes the JSplitPane "leftJSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getLeftJSplitPane() {
		if (leftJSplitPane == null) {
			leftJSplitPane = new JSplitPane();
			leftJSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			leftJSplitPane.setTopComponent(getJTabbedPaneConfiguration());
			leftJSplitPane.setBottomComponent(getJTabbedPaneStatistics());
			leftJSplitPane.setDividerLocation(190);
			leftJSplitPane.setOneTouchExpandable(true);
		}
		return leftJSplitPane;
	}

	/**
	 * This method initializes the JSplitPane "centerJSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getCenterJSplitPane() {
		if (centerJSplitPane == null) {
			centerJSplitPane = new JSplitPane();
			centerJSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			centerJSplitPane.setTopComponent(getJTabbedPaneCode());
			centerJSplitPane.setBottomComponent(getJPanel());
			centerJSplitPane.setDividerLocation(500);
			centerJSplitPane.setOneTouchExpandable(true);
		}
		return centerJSplitPane;
	}

	/**
	 * This method initializes the JSplitPane "rightJSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getRightJSplitPane() {
		if (rightJSplitPane == null) {
			rightJSplitPane = new JSplitPane();
			rightJSplitPane.setDividerLocation(400);
			rightJSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			rightJSplitPane.setTopComponent(getJTabbedPaneRegisterOverview());
			rightJSplitPane.setBottomComponent(getJTabbedPanePlugIn());
			rightJSplitPane.setOneTouchExpandable(true);
		}
		return rightJSplitPane;
	}

	/**
	 * This method initializes the JScrollPane "jScrollPaneCpuRegisters".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPaneCpuRegisters() {
		if (jScrollPaneCpuRegisters == null) {
			JViewport viewport = new JViewport();
			jTableVectorCpuRegisterOverview = getJTableVectorCpuRegisterOverview();
			viewport.setView(jTableVectorCpuRegisterOverview);

			jSplitPaneCpuRegisters = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			jSplitPaneCpuRegisters
					.setTopComponent(getJTableScalarCpuRegisterOverview());
			jSplitPaneCpuRegisters
					.setBottomComponent(jTableVectorCpuRegisterOverview);
			jSplitPaneCpuRegisters.setOneTouchExpandable(true);

			jScrollPaneCpuRegisters = new JScrollPane(jSplitPaneCpuRegisters);
			jScrollPaneCpuRegisters.setRowHeaderView(viewport);
			jScrollPaneCpuRegisters.setCorner(JScrollPane.UPPER_LEFT_CORNER,
					jTableVectorCpuRegisterOverview.getTableHeader());
		}
		return jScrollPaneCpuRegisters;
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPaneRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 11th May 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPaneRegisterOverview() {
		if (jTabbedPaneRegisterOverview == null) {
			jTabbedPaneRegisterOverview = new JTabbedPane();
			jTabbedPaneRegisterOverview.addTab("CPU Register Overview", null,
					getJScrollPaneCpuRegisters(), null);
			jTabbedPaneRegisterOverview.addTab("Memory Overview", null,
					getJTabbedPaneMemoryOverview(), null);
		}
		return jTabbedPaneRegisterOverview;
	}

	/**
	 * This method initializes the JTable "jTableScalarCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 14th May 2009
	 * @return javax.swing.JTable
	 */
	private JTable getJTableScalarCpuRegisterOverview() {
		if (jTableScalarCpuRegisterOverview == null) {
			String[][] rowData = { { "Register", "Value" }, { "A", "0" },
					{ "X", "0" }, { "Y", "0" }, { "Zero bit", "0" },
					{ "Carry bit", "0" }, { "Instruction pointer", "0" } };
			String[] columnNames = { "Register", "Value" };
			DefaultTableModel defaultTableModel = new CustomDefaultTableModelScalarCpuRegisterOverview(
					rowData, columnNames);
			jTableScalarCpuRegisterOverview = new JTable(rowData, columnNames);
			jTableScalarCpuRegisterOverview.setDefaultRenderer(Object.class,
					new ColoredTableCellRendererScalarCpuRegisterOverview());
			jTableScalarCpuRegisterOverview.setIntercellSpacing(new Dimension(
					2, 2));
			jTableScalarCpuRegisterOverview.setModel(defaultTableModel);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(0)
					.setPreferredWidth(125);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(1)
					.setPreferredWidth(360);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(0)
					.setMinWidth(125);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(1)
					.setMinWidth(360);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(0)
					.setMaxWidth(125);
			jTableScalarCpuRegisterOverview.getColumnModel().getColumn(1)
					.setMaxWidth(360);
			jTableScalarCpuRegisterOverview
					.addKeyListener(new java.awt.event.KeyListener() {

						public void keyPressed(KeyEvent arg0) {
						}

						public void keyReleased(KeyEvent e) {
							/*
							 * If enter was pressed, parse the String-Object to
							 * an Integer-Object and set the selected vector
							 * register to the parsed value.
							 */
							if (e.getKeyCode() == KeyEvent.VK_ENTER) {
								int row = jTableScalarCpuRegisterOverview
										.getSelectedRow();
								int column = jTableScalarCpuRegisterOverview
										.getSelectedColumn();
								char register = jTableScalarCpuRegisterOverview
										.getValueAt((row), 0).toString()
										.charAt(0);
								String priorValue = null;
								/*
								 * Get the prior value of the selected cell. It
								 * must be retrieved in order to restore the old
								 * value if an exception occured.
								 */
								switch (register) {
								/*
								 * If the selected row belongs to a scalar cpu
								 * register.
								 */
								case 'A':
									priorValue = ""
											+ engine.getProcessor().getRegA();
									break;
								case 'X':
									priorValue = ""
											+ engine.getProcessor().getRegX();
									break;
								case 'Y':
									priorValue = ""
											+ engine.getProcessor().getRegY();
									break;
								/*
								 * If the selected row belongs to a status cpu
								 * register.
								 */
								case 'Z':
									priorValue = ""
											+ engine.getProcessor()
													.getZeroFlag();
									break;
								case 'C':
									priorValue = ""
											+ engine.getProcessor()
													.getCarryFlag();
									break;
								/*
								 * If the selected row belongs to the
								 * instruction pointer.
								 */
								case 'I':
									priorValue = "" + engine.getIP();
									break;
								}
								try {
									String valueAsString = jTableScalarCpuRegisterOverview
											.getValueAt((row), (column))
											.toString();
									/*
									 * Convert number from different
									 * numeratives, into the decimal numerative.
									 */
									valueAsString = convertNumberIntoDecimalNumerative(valueAsString);
									int value = Integer.parseInt(valueAsString);

									boolean scalarCpuRegister = false;
									boolean statusCpuRegister = false;
									boolean instructionPointer = false;
									switch (register) {
									/*
									 * If the selected row belongs to a scalar
									 * cpu register.
									 */
									case 'A':
										engine.getProcessor().setScalarValue(
												register, value);
										scalarCpuRegister = true;
										break;
									case 'X':
										engine.getProcessor().setScalarValue(
												register, value);
										scalarCpuRegister = true;
										break;
									case 'Y':
										engine.getProcessor().setScalarValue(
												register, value);
										scalarCpuRegister = true;
										break;
									/*
									 * If the selected row belongs to a status
									 * cpu register.
									 */
									case 'Z':
										boolean zeroBit = false;
										if (value == 0)
											zeroBit = false;
										else if (value == 1)
											zeroBit = true;
										else
											throw new NumberFormatException(
													value
															+ " is no valid input! Only 0 and 1 are permitted.");
										int carryBit = Integer
												.parseInt(jTableScalarCpuRegisterOverview
														.getValueAt((row + 1),
																column)
														.toString());
										engine.getProcessor().setStatusRegs(
												zeroBit, carryBit);
										statusCpuRegister = true;
										break;
									case 'C':
										zeroBit = false;
										if (jTableScalarCpuRegisterOverview
												.getValueAt((row - 1), column)
												.toString().charAt(0) == 0)
											zeroBit = false;
										else if (jTableScalarCpuRegisterOverview
												.getValueAt((row - 1), column)
												.toString().charAt(0) == 1)
											zeroBit = true;
										engine.getProcessor().setStatusRegs(
												zeroBit, value);
										statusCpuRegister = true;
										break;
									/*
									 * If the selected row belongs to the
									 * instruction pointer.
									 */
									case 'I':
										engine.getProcessor().setIP(value);
										instructionPointer = true;
										break;
									}

									/*
									 * Print the appropriate confirm message to
									 * the console.
									 */
									if (scalarCpuRegister)
										displayConsoleMessage("Set scalar register "
												+ register
												+ " to "
												+ valueAsString + ".");
									else if (statusCpuRegister)
										displayConsoleMessage("Set "
												+ jTableScalarCpuRegisterOverview
														.getValueAt(row, 0)
														.toString()
														.toLowerCase() + " to "
												+ valueAsString + ".");
									else if (instructionPointer)
										displayConsoleMessage("Set instruction pointer to "
												+ valueAsString + ".");
								}
								/*
								 * If an exception occured, catch the exception,
								 * print the error message to the console and
								 * reset the altered value of the selected cell
								 * to the prior value.
								 */catch (Exception ex) {
									displayConsoleError(ex.getMessage());
									jTableScalarCpuRegisterOverview.setValueAt(
											priorValue, row, column);
								}
							}
						}

						public void keyTyped(KeyEvent arg0) {
						}
					});
		}
		return jTableScalarCpuRegisterOverview;
	}

	/**
	 * This method displays the status registers of the CPU in the jTable
	 * "jTableScalarCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 17th May 2009
	 * @param int
	 *            carryBit
	 * @param int
	 *            zeroBit
	 */
	public void displayStatusRegister(int carryBit, int zeroBit) {
		jTableScalarCpuRegisterOverview.setValueAt(zeroBit, 4, 1);
		jTableScalarCpuRegisterOverview.setValueAt(carryBit, 5, 1);
	}

	/**
	 * This method displays the instruction pointer of the CPU in the jTable
	 * "jTableScalarCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 17th May 2009
	 * @param int
	 *            instructionPointer
	 */
	public void displayInstructionPointer(int instructionPointer) {
		jTableScalarCpuRegisterOverview.setValueAt(instructionPointer, 6, 1);
		/* Update the instruction pointer in all memory overviews. */
		for (int i = 0; i < jPanelMemoryOverviewList.size(); i++) {
			jPanelMemoryOverviewList.get(i).updateIP(instructionPointer);
		}

	}

	/**
	 * This method initializes the JTable "jTableVectorCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 14th May 2009
	 * @return javax.swing.JTable
	 */
	private JTable getJTableVectorCpuRegisterOverview() {
		/*
		 * The coordinates of the current selected cell are stored in this
		 * Integer-Array. cell[0] holds the row coordinate of the selected cell
		 * and cell[1] holds the column coordinate of the selected cell.
		 */
		final int[] cell = new int[2];
		/**
		 * This class implements the abstract interface class
		 * ListSelectionListener and provides a method which handles value
		 * changes. It is used to save row coordinates of cell selections.
		 *
		 * @author Daniel Obermuellre 19th June 2009
		 */
		final class RowListener implements ListSelectionListener {

			/**
			 * This method stores the row coordinate of the current selected
			 * cell.
			 *
			 * @author Daniel Obermueller 19th June 2009
			 * @param ListSelectionEvent
			 *            event
			 */
			public void valueChanged(ListSelectionEvent event) {
				/* If the selected cell is edited at the moment, do nothing. */
				if (event.getValueIsAdjusting()) {
					return;
				}
				/* Store the row coordinate of the current cell selection. */
				for (int r : jTableVectorCpuRegisterOverview.getSelectedRows())
					cell[0] = (r - 1);
			}
		}

		/**
		 * This class implements the abstract interface class
		 * ListSelectionListener and provides a method which handles value
		 * changes. It is used to store column coordinates of cell selections.
		 *
		 * @author Daniel Obermueller 19th June 2009
		 */
		final class ColumnListener implements ListSelectionListener {

			/**
			 * This method stores the column coordinate of the current selected
			 * cell.
			 *
			 * @author Daniel Obermueller 19th June 2009
			 * @param ListSelectionEvent
			 *            event
			 */
			public void valueChanged(ListSelectionEvent event) {
				/* If the selected cell is edited at the moment, do nothing. */
				if (event.getValueIsAdjusting()) {
					return;
				}
				/* Store the column coordinate of the current cell selection. */
				for (int c : jTableVectorCpuRegisterOverview
						.getSelectedColumns())
					cell[1] = (c - 1);
			}
		}

		if (jTableVectorCpuRegisterOverview == null) {
			customAbstractTableModelVectorCpuRegisterOverview = new CustomAbstractTableModelWithEditiableCells();
			jTableVectorCpuRegisterOverview = new JTable(
					customAbstractTableModelVectorCpuRegisterOverview);

			/* Add customized ListSelectionListeners. */
			jTableVectorCpuRegisterOverview.getSelectionModel()
					.addListSelectionListener(new RowListener());
			jTableVectorCpuRegisterOverview.getColumnModel()
					.getSelectionModel().addListSelectionListener(
							new ColumnListener());

			jTableVectorCpuRegisterOverview.setDefaultRenderer(Object.class,
					new ColoredTableCellRendererVectorCpuRegisterOverview());
			jTableVectorCpuRegisterOverview.setIntercellSpacing(new Dimension(
					2, 2));
			jTableVectorCpuRegisterOverview
					.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			jTableVectorCpuRegisterOverview
					.addKeyListener(new java.awt.event.KeyListener() {

						public void keyPressed(KeyEvent arg0) {
						}

						public void keyReleased(KeyEvent e) {
							/*
							 * If enter was pressed, parse the String-Object to
							 * an Integer-Object and set the selected vector
							 * register to the parsed value.
							 */
							if (e.getKeyCode() == KeyEvent.VK_ENTER) {
								try {
									String valueAsString = jTableVectorCpuRegisterOverview
											.getValueAt((cell[0] + 1),
													(cell[1] + 1)).toString();
									/*
									 * Convert number from different
									 * numeratives, into the decimal numerative.
									 */
									valueAsString = convertNumberIntoDecimalNumerative(valueAsString);
									int value = Integer.parseInt(valueAsString);
									engine.getProcessor().setVRegsValue(
											cell[0], cell[1], value);
									displayConsoleMessage("Set vector register R"
											+ cell[0]
											+ " K"
											+ cell[1]
											+ " to "
											+ valueAsString + ".");
								}
								/*
								 * If an exception occured, catch the exception,
								 * print the error message to the console and
								 * reset the altered value of the vector
								 * register to the prior value.
								 */catch (Exception ex) {
									displayConsoleError(ex.getMessage());
									displaySingleValueOfVectorRegister(engine
											.getProcessor().getVRegsValue(
													cell[0], cell[1]), cell[0],
											cell[1]);
								}
							}
						}

						public void keyTyped(KeyEvent arg0) {
						}
					});
			jTableVectorCpuRegisterOverview.setVisible(true);
		}
		return jTableVectorCpuRegisterOverview;
	}

	/**
	 * This method sets the size of the JTable
	 * "jTableVectorCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 14th May 2009
	 * @param int
	 *            rows
	 * @param int
	 *            columns
	 */
	public void setJTableVectorCpuRegisterOverviewSize(int rows, int columns) {
		customAbstractTableModelVectorCpuRegisterOverview.setJTableSize(
				rows + 1, columns + 1);
		/* Set the row title for each row. */
		for (int row = 1; row < customAbstractTableModelVectorCpuRegisterOverview
				.getAmountOfRows(); row++)
			customAbstractTableModelVectorCpuRegisterOverview.setValueAt("R"
					+ (row - 1), row, 0);
		/* Set the column title for each column. */
		for (int column = 1; column < customAbstractTableModelVectorCpuRegisterOverview
				.getAmountOfColumns(); column++)
			customAbstractTableModelVectorCpuRegisterOverview.setValueAt("K"
					+ (column - 1), 0, column);
	}

	/**
	 * This method displays a passed value of a specified vector register in the
	 * JTable "jTableVectorCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 14th May 2009
	 * @param Object
	 *            value
	 * @int row
	 */
	public void displaySingleValueOfVectorRegister(Object value, int row,
			int column) {
		try {
			customAbstractTableModelVectorCpuRegisterOverview.setValueAt(value,
					(row + 1), (column + 1));
		} catch (Exception e) {
		}
	}

	/**
	 * This method displays a passed value of a specified vector register in the
	 * JTable "jTableVectorCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 14th May 2009
	 * @param Vector
	 *            <Object> rowData
	 * @param int
	 *            row
	 */
	@SuppressWarnings("unused")
	private void displayValueOfVectorRegister(Vector<Object> rowData, int row) {
		customAbstractTableModelVectorCpuRegisterOverview.setRow(rowData, row);
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPaneMemoryOverview".
	 *
	 * @author Daniel Obermueller 3rd June 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPaneMemoryOverview() {
		if (jTabbedPaneMemoryOverview == null) {
			jTabbedPaneMemoryOverview = new JTabbedPane();
			jPanelMemoryOverviewList = new LinkedList<JPanelMemoryOverview>();
		}
		return jTabbedPaneMemoryOverview;
	}

	/**
	 * This method adds a tab, which is a custom JPanel "JPanelMemoryOverview",
	 * that is used to display the memory, to the JTabbedPane
	 * "jTabbedPaneMemoryOverview".
	 *
	 * @author Daniel Obermueller 3rd June 2009
	 */
	private void addMemoryOverviewTab() {
		try {
			JPanelMemoryOverview tmp = new JPanelMemoryOverview(engine);
			jPanelMemoryOverviewList.add(tmp);
			jTabbedPaneMemoryOverview.addTab("Memory "
					+ jPanelMemoryOverviewList.size(), null, tmp, null);
		} catch (Exception e1) {
		}
	}

	/**
	 * This method deletes a tab, which is a custom JPanel
	 * "JPanelMemoryOverview", that is used to display the memory, from the
	 * JTabbedPane "jTabbedPaneMemoryOverview".
	 *
	 * @author Daniel Obermueller 3rd June 2009
	 */
	private void deleteMemoryOverviewTab() {
		if (!jPanelMemoryOverviewList.isEmpty()) {
			/* Delete plugin in pluginmanager */
			jPanelMemoryOverviewList.get((jPanelMemoryOverviewList.size() - 1))
					.deleteMemViewPlugin();

			jPanelMemoryOverviewList.removeLast();
			jTabbedPaneMemoryOverview.removeTabAt(jPanelMemoryOverviewList
					.size());
		}
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPanePlugIn".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPanePlugIn() {
		if (jTabbedPanePlugIn == null) {
			jTabbedPanePlugIn = new JTabbedPane();
			jTabbedPanePlugIn.setFocusable(true);
		}
		return jTabbedPanePlugIn;
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPaneConfiguration".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPaneConfiguration() {
		if (jTabbedPaneConfiguration == null) {
			jTabbedPaneConfiguration = new JTabbedPane();
			JScrollPane jScrollPane = new JScrollPane(
					getJTableCpuConfiguration());
			jScrollPane
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
			jTabbedPaneConfiguration.addTab("CPU Configuration", null,
					jScrollPane, null);
		}
		return jTabbedPaneConfiguration;
	}

	/**
	 * This method initializes the JTable "jTableCpuConfiguration".
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return javax.swing.JTable
	 */
	private JTable getJTableCpuConfiguration() {
		if (jTableCpuConfiguration == null) {
			String[][] rowData = { { "", "" }, { "", "" }, { "", "" },
					{ "", "" }, { "", "" }, { "", "" }, { "", "" }, { "", "" },
					{ "", "" } };
			String[] columnNames = { "Constant", "Value" };
			DefaultTableModel defaultTableModel = new DefaultTableModel(
					rowData, columnNames);
			jTableCpuConfiguration = new JTable(rowData, columnNames);
			jTableCpuConfiguration.setModel(defaultTableModel);
			jTableCpuConfiguration.setEditingColumn(1);
			jTableCpuConfiguration
					.addKeyListener(new java.awt.event.KeyListener() {
						public void keyPressed(KeyEvent e) {
						}

						public void keyReleased(KeyEvent e) {
							if (jTableCpuConfiguration.getSelectedColumn() == 1
									&& e.getKeyCode() == KeyEvent.VK_ENTER) {
								try {
									int row = jTableCpuConfiguration
											.getSelectedRow();
									String constant = jTableCpuConfiguration
											.getValueAt(row, 0).toString();
									String value = jTableCpuConfiguration
											.getValueAt(row, 1).toString();
									engine.setConfiguration(constant + " "
											+ value);
									engine.reset();
									/*
									 * If the convention is violated, display an
									 * error message on the console and reset
									 * the displayed cpu configuration.
									 */
								} catch (Exception ex) {
									displayConsoleError(ex.getMessage());
									displayCpuConfiguration(engine
											.getCPUConfiguration());
								}
							}
						}

						public void keyTyped(KeyEvent e) {
						}
					});
		}
		return jTableCpuConfiguration;
	}

	/**
	 * This method displays the passed HashMap<String, String> "config", which
	 * represents the CPU configuration, in the jTable "jTableCpuConfiguration".
	 *
	 * @author Daniel Obermueller 17th May 2009
	 * @param HashMap
	 *            <String, String> config
	 */
	public void displayCpuConfiguration(HashMap<String, String> config) {
		try {
			Object[] constants = config.keySet().toArray();
			Object[] values = config.values().toArray();
			for (int i = 0; i < constants.length; i++)
				jTableCpuConfiguration.setValueAt(constants[i], i, 0);
			for (int i = 0; i < values.length; i++)
				jTableCpuConfiguration.setValueAt(values[i], i, 1);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPaneStatistics".
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPaneStatistics() {
		if (jTabbedPaneStatistics == null) {
			jTabbedPaneStatistics = new JTabbedPane();
			jTabbedPaneStatistics.addTab("Code Statistics", null,
					new JScrollPane(getJTableCodeStatistics()), null);
		}
		return jTabbedPaneStatistics;
	}

	/**
	 * This method initializes the JTable "jTableCodeStatistics"
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return javax.swing.JTable
	 */
	private JTable getJTableCodeStatistics() {
		if (jTableCodeStatistics == null) {
			String[][] rowData = { { "", "" } };
			String[] columnNames = { "Command", "Amount" };
			DefaultTableModel statsModel = new DefaultTableModel(rowData,
					columnNames);
			jTableCodeStatistics = new JTable(rowData, columnNames);
			jTableCodeStatistics.setModel(statsModel);
			jTableCodeStatistics.setEnabled(false);
		}
		return jTableCodeStatistics;
	}

	/**
	 * This method initializes the JTabbedPane "jTabbedPaneCode".
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPaneCode() {
		if (jTabbedPaneCode == null) {
			jTabbedPaneCode = new JTabbedPane();
			jTabbedPaneCode.addTab("Source Code", null, new JScrollPane(
					getJTextAreaSourceCode()), null);
			jTabbedPaneCode.addTab("Disassembled Code", null, new JScrollPane(
					getJTextAreaDisassembledCode()), null);
			jTabbedPaneCode.addTab("Breakpoint Overview", null,
					getJSplitPaneTopAndCenterBreakpointOverview(), null);
		}
		return jTabbedPaneCode;
	}

	/**
	 * This method initializes the JPanel "jPanel".
	 *
	 * @author Daniel Obermueller 9th May 2009
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getJTextFieldCommandLine(), BorderLayout.SOUTH);
			jPanel.add(new JScrollPane(getJTextAreaConsole()),
					BorderLayout.CENTER);
		}
		return jPanel;
	}

	/**
	 * This method initializes the JTextField "jTextFieldCommandLine".
	 *
	 * @author Daniel Obermueller 21st April 2009
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextFieldCommandLine() {
		if (jTextFieldCommandLine == null) {
			Font font = new Font(null, Font.PLAIN, 14);
			jTextFieldCommandLine = new JTextField();
			jTextFieldCommandLine.setFont(font);
			final int BUFFERSIZE = 50;
			jTextFieldCommandLine
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							/*
							 * If there was no command typed in, the last one
							 * gets executed again.
							 */
							if (e.getActionCommand().equals("")) {
								if (bufferCommandLine.size() == 0) {
									String message;
									try {
										message = cmdProcessor
												.processGUI_Input(e
														.getActionCommand());
										jTextFieldCommandLine
												.requestFocusInWindow();
										displayConsoleMessage(bufferCommandLine
												.getLast());
										if (message != null)
											displayConsoleMessage(message);
										return;
									} catch (Exception ex) {
										displayConsoleMessage(e
												.getActionCommand());
										displayConsoleError(ex.getMessage());
										return;
									}
								} else {
									try {
										String message = cmdProcessor
												.processGUI_Input(bufferCommandLine
														.getLast());
										jTextFieldCommandLine
												.requestFocusInWindow();
										displayConsoleMessage(bufferCommandLine
												.getLast());
										if (message != null)
											displayConsoleMessage(message);
										return;
									} catch (Exception ex) {
										displayConsoleMessage(bufferCommandLine
												.getLast());
										displayConsoleError(ex.getMessage());
										return;
									}
								}
							}
							String message = null;
							if (bufferCommandLine.size() != BUFFERSIZE) {
								bufferCommandLine.add(e.getActionCommand());
							}
							/*
							 * If the buffer, which is filled with the typed in
							 * commands, is full, remove the first command and
							 * add the current one.
							 */else {
								bufferCommandLine.remove();
								bufferCommandLine.add(e.getActionCommand());
							}
							iteratorCommandLine = bufferCommandLine
									.listIterator();
							try {
								displayConsoleMessage(e.getActionCommand());
								message = cmdProcessor.processGUI_Input(e
										.getActionCommand());
							} catch (Exception ex) {
								displayConsoleError(ex.getMessage());
								jTextFieldCommandLine.setText("");
								jTextFieldCommandLine.requestFocusInWindow();
								return;
							}
							jTextFieldCommandLine.setText("");
							jTextFieldCommandLine.requestFocusInWindow();
							if (message != null)
								displayConsoleMessage(message);
						}
					});
			jTextFieldCommandLine
					.addKeyListener(new java.awt.event.KeyAdapter() {
						public void keyPressed(java.awt.event.KeyEvent e) {
							if (iteratorCommandLine != null) {
								/*
								 * If the DOWNARROW key is pressed iterate
								 * forwards through the list of commands.
								 */
								if (e.getKeyCode() == KeyEvent.VK_DOWN) {
									iterateForwards();
								}
								/*
								 * If the UPARROW key is pressed iterate
								 * backwards through the list of commands.
								 */
								else if (e.getKeyCode() == KeyEvent.VK_UP) {
									iterateBackwards();
								}
							}
						}
					});
		}
		return jTextFieldCommandLine;
	}

	/**
	 * This method is iterating forwards through the list of commands.
	 *
	 * @author Daniel Obermueller 22nd April 2009
	 */
	private void iterateForwards() {
		if (iteratorCommandLine.hasNext())
			jTextFieldCommandLine.setText(iteratorCommandLine.next());
		/*
		 * If the bound of the list is reached, create a new list iterator
		 * "iteratorCommandLine", which is pointing at the first index, in order
		 * to be able to iterate further.
		 */
		else {
			iteratorCommandLine = bufferCommandLine.listIterator();
			if (iteratorCommandLine.hasNext())
				jTextFieldCommandLine.setText(iteratorCommandLine.next());
		}
	}

	/**
	 * This method is iterating backwards through the list of commands.
	 *
	 * @author Daniel Obermueller 22nd April 2009
	 */
	private void iterateBackwards() {
		if (iteratorCommandLine.hasPrevious())
			jTextFieldCommandLine.setText(iteratorCommandLine.previous());
		/*
		 * If the bound of the list is reached, create a new list iterator
		 * "iteratorCommandLine", which is pointing at the last index, in order
		 * to be able to iterate further.
		 */
		else {
			iteratorCommandLine = bufferCommandLine
					.listIterator(bufferCommandLine.size());
			if (iteratorCommandLine.hasPrevious())
				jTextFieldCommandLine.setText(iteratorCommandLine.previous());
		}
	}

	/**
	 * This method initializes the CustomJTextPane "jTextAreaConsole".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 10th April 2009
	 * @return CustomJTextPane
	 */
	private CustomJTextPane getJTextAreaConsole() {
		if (jTextAreaConsole == null) {
			Font font = new Font(null, Font.ROMAN_BASELINE, 10);
			jTextAreaConsole = new CustomJTextPane();
			jTextAreaConsole.setEditable(false);
			jTextAreaConsole.setFont(font);
			jTextAreaConsole.setAutoscrolls(true);
		}
		return jTextAreaConsole;
	}

	/**
	 * This method displays a message on the console, represented as a
	 * String-Object on the console jTextAreaConsole.
	 *
	 * @author Daniel Obermueller 21st April 2009
	 * @param String
	 *            message
	 */
	public void displayConsoleMessage(String message) {
		message += "\n";
		jTextAreaConsole.append(message);
		jTextAreaConsole.setCaretPosition(jTextAreaConsole.getDocument()
				.getLength());
	}

	/**
	 * This method displays an error message on the console, represented as a
	 * String-Object on the console jTextAreaConsole.
	 *
	 * @author Daniel Obermueller 21st April 2009
	 * @param String
	 *            message
	 */
	public void displayConsoleError(String message) {
		message += "\n";
		jTextAreaConsole.append(message, Color.RED);
		jTextAreaConsole.setCaretPosition(jTextAreaConsole.getDocument()
				.getLength());
	}

	/**
	 * This method initializes the JTextArea "jTextAreaSourceCode".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextAreaSourceCode() {
		if (jTextAreaSourceCode == null) {
			jTextAreaSourceCode = new JTextAreaWithLineNumbering(50, 0);
			Font font = new Font(null, Font.PLAIN, 12);
			jTextAreaSourceCode.setFont(font);
		}
		return jTextAreaSourceCode;
	}

	/**
	 * This method initializes the JTextArea "jTextAreaDisassembledCode".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 9th April 2009
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextAreaDisassembledCode() {
		if (jTextAreaDisassembledCode == null) {
			jTextAreaDisassembledCode = new JTextAreaWithLineNumbering(50, 0);
			jTextAreaDisassembledCode.setEditable(false);
			Font font = new Font(null, Font.PLAIN, 12);
			jTextAreaDisassembledCode.setFont(font);
		}
		return jTextAreaDisassembledCode;
	}

	/**
	 * This method initializes the JSplitPane "jSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 21st March 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPaneTopAndCenterBreakpointOverview() {
		if (jSplitPaneTopAndCenterBreakpointOverview == null) {
			jSplitPaneTopAndCenterBreakpointOverview = new JSplitPane(
					JSplitPane.VERTICAL_SPLIT);
			jSplitPaneTopAndCenterBreakpointOverview
					.setTopComponent(new JScrollPane(
							getJTextAreaBreakpointAtLine()));
			jSplitPaneTopAndCenterBreakpointOverview
					.setBottomComponent(getJSplitPaneCenterAndBottomBreakpointOverview());
			jSplitPaneTopAndCenterBreakpointOverview.setDividerLocation(150);
			jSplitPaneTopAndCenterBreakpointOverview
					.setOneTouchExpandable(true);
		}
		return jSplitPaneTopAndCenterBreakpointOverview;
	}

	/**
	 * This method initializes the JSplitPane "jSplitPane".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 21st March 2009
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPaneCenterAndBottomBreakpointOverview() {
		if (jSplitPaneCenterAndBottomBreakpointOverview == null) {
			jSplitPaneCenterAndBottomBreakpointOverview = new JSplitPane(
					JSplitPane.VERTICAL_SPLIT);
			jSplitPaneCenterAndBottomBreakpointOverview
					.setTopComponent(new JScrollPane(
							getJTextAreaBreakpointAtAdress()));
			jSplitPaneCenterAndBottomBreakpointOverview
					.setBottomComponent(new JScrollPane(
							getJTextAreaBreakpointAtLabel()));
			jSplitPaneCenterAndBottomBreakpointOverview.setDividerLocation(150);
			jSplitPaneCenterAndBottomBreakpointOverview
					.setOneTouchExpandable(true);
		}
		return jSplitPaneCenterAndBottomBreakpointOverview;
	}

	/**
	 * This method initializes the JTextArea "jTextAreaBreakpointAtLine".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 27th April 2009
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextAreaBreakpointAtLine() {
		if (jTextAreaBreakpointAtLine == null) {
			jTextAreaBreakpointAtLine = new JTextArea(25, 0);
			Font font = new Font(null, Font.PLAIN, 12);
			jTextAreaBreakpointAtLine.setFont(font);
			jTextAreaBreakpointAtLine.setEditable(false);
		}
		return jTextAreaBreakpointAtLine;
	}

	/**
	 * This method initializes JTextArea "jTextAreaBreakpointAtLabel".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 27th April 2009
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextAreaBreakpointAtLabel() {
		if (jTextAreaBreakpointAtLabel == null) {
			jTextAreaBreakpointAtLabel = new JTextArea(25, 0);
			Font font = new Font(null, Font.PLAIN, 12);
			jTextAreaBreakpointAtLabel.setFont(font);
			jTextAreaBreakpointAtLabel.setEditable(false);
		}
		return jTextAreaBreakpointAtLabel;
	}

	/**
	 * This method initializes the JTextArea "jTextAreaBreakpointAtAdress".
	 *
	 * @author Daniel Obermueller & Sebastian Minning 27th April 2009
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextAreaBreakpointAtAdress() {
		if (jTextAreaBreakpointAtAdress == null) {
			jTextAreaBreakpointAtAdress = new JTextArea(25, 0);
			Font font = new Font(null, Font.PLAIN, 12);
			jTextAreaBreakpointAtAdress.setFont(font);
			jTextAreaBreakpointAtAdress.setEditable(false);
		}
		return jTextAreaBreakpointAtAdress;
	}

	/**
	 * This method initializes this.
	 *
	 * @author Daniel Obermueller 10th April 2009
	 */
	private void initialize() {
		this.setSize(1280, 720);
//		this.setContentPane(getJSplitPane());
		this.setTitle("HiCoSim");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setVisible(true);
		this.setLayout(new BorderLayout());
		this.add(getJSplitPane(), BorderLayout.CENTER);
		initializeJMenuBar();
		focusAtCommandline();
		this.add(new ToolBar(), BorderLayout.NORTH);

	}

	/**
	 * This method converts numbers, passed as String-Object from several
	 * numeratives into the decimal numerative and returns them as
	 * String-Object. If an unknown numerative or an invalid input was verified
	 * the method throws a NumberFormatException.
	 *
	 * @author Daniel Obermueller 17th June 2009
	 *
	 * @param String
	 *            value
	 * @return String
	 * @throws NumberFormatException
	 */
	public String convertNumberIntoDecimalNumerative(String value)
			throws NumberFormatException {
		/*
		 * If the passed value is an octal number, convert it into an Integer
		 * value.
		 */
		String number = null;
		if (value.matches("0[0-7]*"))
			number = "" + BitUtils.octalToDecimal(value);
		/*
		 * If the passed value is a decimal number, do nothing.
		 */
		else if (value.matches("[0-9][0-9]*"))
			number = value;
		/*
		 * If the passed value is a hexadecimal number, convert it into an
		 * integer value.
		 */
		else if (value.matches("0x[a-zA-Z0-9]*")) {
			number = value.substring(2);
			number = "" + BitUtils.hex2Decimal(number);
		}
		/*
		 * If the passed value is a binary number, convert it into an integer
		 * value.
		 */
		else if (value.matches("b[0-1]*")) {
			number = value.substring(1);
			number = "" + BitUtils.binary2Decimal(number);
		}
		if (number == null)
			throw new NumberFormatException("Unknown numerative.");
		return number;
	}

	/**
	 * This method requests and focuses the JTextField "JTextFieldCommandLine".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 */
	public void focusAtCommandline() {
		jTextFieldCommandLine.setFocusable(true);
		jTextFieldCommandLine.requestFocus();

	}

	/**
	 * This method displays the String-Object "text" in the JTextArea
	 * "jTextAreaSourceCode".
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param String
	 *            text
	 */
	public void displaySourceCode(String text) {
		displayText(text, "jTextAreaSourceCode");
	}

	/**
	 * This method displays the String-Object "text" in the JTextArea
	 * "jTextAreaDisassembledCode". *
	 *
	 * @author Daniel Obermueller 11th May 2009
	 * @param String
	 *            text
	 */
	public void displayDisassembledCode(String text) {
		displayText(text, "jTextAreaDisassembledCode");
	}

	/**
	 * This method displays the passed Integer-Objects in the JTable
	 * "jTableScalarCpuRegisterOverview".
	 *
	 * @author Daniel Obermueller 13th May 2009
	 * @param int
	 *            valOfA
	 * @param int
	 *            valOfX
	 * @param int
	 *            valOfY
	 */
	public void displayScalarRegChanges(Object valOfA, Object valOfX, Object valOfY) {
		jTableScalarCpuRegisterOverview.setValueAt(valOfA, 1, 1);
		jTableScalarCpuRegisterOverview.setValueAt(valOfX, 2, 1);
		jTableScalarCpuRegisterOverview.setValueAt(valOfY, 3, 1);
	}

	/**
	 * This method displays the passed Integer-Array, which contains the values
	 * of the vector registers, in the JTable "jTableVectorCpuRegisterOverview".
	 *
	 *  @author Daniel Obermueller 13th May 2009
	 *  @param int[][] regs
	 */
	public void displayVectorRegChanges(int[][] regs) {
		for (int i = 0; i < regs.length; i++) {
			for (int j = 0; j < regs[i].length; j++)
				displaySingleValueOfVectorRegister(regs[i][j], i, j);
		}
	}

	/*
	 * ____________________________________________________________________________
	 * Past this line is the code of ANDI
	 */

	/**
	 * @author ANDI
	 */

	/*
	 * Callback-Funktion, wird aufgerufen sobald ein Comand erhoeht wird
	 *
	 * Keine Aenderungen in der Klasse Statistik noetig, wenn Befehlssatz
	 * erweitert wird. Auch unbekannte Befehle werden der GUI gesendet. Die
	 * weitere Implementierung, also z.B. Gruppierung in ALU-Befehle usw. kann
	 * dann die GUI uebernehmen
	 */
	public void updateStatsObserver(String comand, String type) {
		DefaultTableModel model = (DefaultTableModel) getJTableCodeStatistics()
				.getModel();
		Vector<String> newV = new Vector<String>();
		newV.add(comand);
		newV.add("1");

		boolean newComand = true;

		int modelRows = model.getRowCount();

		for (int i = 0; i < modelRows; i++) {
			/* Springe zu Zeile mit richtigem Typ */
			if (model.getValueAt(i, 0) == type) {
				/* setze Anzahl des Comand-Types um eins hoch */
				int oldValueType = Integer.parseInt(model.getValueAt(i, 1)
						.toString());
				model.setValueAt((oldValueType + 1), i, 1);
				/* Richtige Zeile gefunden, nun Befehle vergleichen */
				for (int y = 1; y < (statsComands.get(type) + 1); y++) {
					if (model.getValueAt(y + i, 0) == comand) {
						newComand = false;
						int oldValue = Integer.parseInt(model.getValueAt(y + i,
								1).toString());
						model.setValueAt((oldValue + 1), y + i, 1);
						/* Sortiervorgang starten! */
						for (int z = y + i; z > i; z--) {
							int newValue = (oldValue + 1);
							int prevValue = Integer.parseInt(model.getValueAt(
									z - 1, 1).toString());
							if (newValue > prevValue) {
								String prevComand = model.getValueAt(z - 1, 0)
										.toString();
								model.setValueAt(prevComand, z, 0);
								model.setValueAt(prevValue, z, 1);

								model.setValueAt(comand, z - 1, 0);
								model.setValueAt(newValue, z - 1, 1);
								continue;
							}
							break;
						}
					}
				}
				if (newComand) {
					model.insertRow((i + 1), newV);
					int anzCmd = statsComands.get(type);
					statsComands.put(type, ++anzCmd);

				}
			}

		}
		int anzRows = model.getRowCount();

		int oldGesamt = Integer.parseInt(model.getValueAt((anzRows - 1), 1)
				.toString());

		model.setValueAt((oldGesamt + 1), (anzRows - 1), 1);

	}

	/* Statistik reset - Methode wird per Callback von Statistiken aufgerufen */
	public void resetStats() {
		String[] types = this.engine.getProcessor().getStatistics().getTypes();
		statsComands = new HashMap<String, Integer>();

		DefaultTableModel model = (DefaultTableModel) jTableCodeStatistics
				.getModel();
		int anzRows = model.getRowCount();

		for (int i = 0; i < anzRows; i++) {
			model.removeRow(0);
		}

		for (int i = 0; i < types.length; i++) {
			Vector<String> v = new Vector<String>();
			v.add(types[i]);
			v.add("0");
			model.insertRow(i, v);
			statsComands.put(types[i], 0);

		}
		Vector<String> v = new Vector<String>();
		v.add("Total:");
		v.add("0");
		model.addRow(v);
		jTableCodeStatistics.setModel(model);

		/*
		 * CellRenderer zuweisen und Methode ueberlagern fuer Darstellung in
		 * Fettschrift/kursiv
		 */
		jTableCodeStatistics.setDefaultRenderer(Object.class,
				new DefaultTableCellRenderer() {
					/**
					 *
					 */
					private static final long serialVersionUID = 1L;

					public Component getTableCellRendererComponent(
							JTable table, Object value, boolean isSelected,
							boolean hasFocus, int row, int column) {
						Component component = super
								.getTableCellRendererComponent(table, value,
										isSelected, hasFocus, row, column);
						if (statsComands.containsKey(value.toString())) {
							component.setFont(component.getFont().deriveFont(
									Font.BOLD));
						}

						if (value.toString() == "Total:") {
							component.setFont(component.getFont().deriveFont(
									Font.BOLD));
						}

						if (row == (table.getModel().getRowCount() - 1)
								&& column == 1) {
							component.setFont(component.getFont().deriveFont(
									Font.PLAIN));
						}
						return component;
					}
				});

	}

}

// @jve:decl-index=0:visual-constraint="10,10"

