package hicosim.components.frontend;

import hicosim.control.Engine;

/**
 * @author Daniel Obermueller 30th May 2009
 *
 */
/**
 * This class extends the super class Thread and provides a method run, which is
 * needed to start a program with the "HiCoSim" frontend.
 */
public class RunThread extends Thread {

	private Engine engine;

	public RunThread(Engine engine) {
		super();
		this.engine = engine;
	}

	private synchronized void step() {
		try {
			engine.step(true);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private synchronized boolean isProcessorRunning() {
		return engine.getProcessor().isRunning();
	}

	@SuppressWarnings("deprecation")
	public void run() {
		if (engine.getProcessor().isHALT()) {
			engine.displayConsole("Already reached halt. Try 'reset cpu'.",
					true);
			super.stop();
			super.destroy();
		}
		engine.setProcessorRunning(true);
		engine.getProcessor().setStepping(false);
		while (isProcessorRunning()) {
			step();
		}
		
		//und update der register:
//		engine.displayInstructionPointer(engine.getIP());
//		engine.displayStatusRegister(engine.getProcessor().getCarryFlag(), engine.getProcessor().getZeroFlag()==0 ? false:true);
//		engine.checkNumerativeFlagAndDisplayAll();
		
		super.stop();
		super.destroy();
	}
}
