package hicosim.components.plugins;

import hicosim.components.backend.processor.*;
import hicosim.control.Engine;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/* Test fuer ein einfaches Plugin - 
 * ueberwacht Speicherbereich zwischen
 * StartAt und EndAt, zeichnet diesen in einer Tabelle
 */
public class TestPlugin implements IPlugin {

	private IPluginManager pluginMem;
	int startAt;
	int endAt;
	boolean observer;

	public JTable tabelle;
	protected JFrame frame;
	
	private Engine engine=null;

	
	public TestPlugin(int startAt, int endAt, Engine eng) throws Exception {
		this.startAt = startAt;
		this.endAt = endAt;
		
		this.engine = eng;

		renderFrame();
		
	

	}

	public void notifyPlugin(int adr, int value) {
//		getTable().setCellSelectionEnabled(true);
//		
//		getTable().setSelectionBackground(Color.red);
//		getTable().changeSelection((adr-startAt), 4, false	, false);
					
		getTable().getModel().setValueAt(Integer.toString(value),
				(adr - startAt), 1);
		getTable().getModel().setValueAt("0x" + Integer.toHexString(value),
				(adr - startAt), 2);
		getTable().getModel().setValueAt(Integer.toBinaryString(value),
				(adr - startAt), 3);
		
		//################### EDIT FUE DISAS ANZEIGE
		try {
			getTable().getModel().setValueAt(engine.getMemory().getDisassembledInstructionAt(adr)==null?
				"NONE" :engine.getMemory().getDisassembledInstructionAt(adr),(adr - startAt), 4);
				
		}
		catch (Exception e) {
			getTable().getModel().setValueAt("NONE",(adr - startAt), 4);		
		}

	}

	public String toString() {
		return "TestPlugin";
	}

	private void renderFrame() throws Exception {

		String[][] rowData = new String[endAt - startAt][2];

		for (int i = 0; i < (this.endAt - this.startAt); i++) {

			rowData[i][0] = Integer.toString(startAt + i);
			rowData[i][1] = "";

		}
																				
		String[] columnNames = { "Adresse", "Wert Dezimal", "Wert Hex","Wert Binaer", "DISAS" };
				
		DefaultTableModel model = new DefaultTableModel(rowData, columnNames);
		this.frame = new JFrame("Anzeige Speicherbereich von " + startAt+ " bis " + endAt);
		frame.setLayout(new BorderLayout());		
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		final JTextField input = new JTextField(30); 
				
		this.tabelle = new JTable(model);
		tabelle.setModel(model);
		frame.add(new JScrollPane(tabelle), BorderLayout.NORTH);

		JButton buttonUp = new JButton("Rauf");
		JButton buttonDown = new JButton("Runter");

		frame.add(buttonUp, BorderLayout.WEST);
		
		frame.add(buttonDown, BorderLayout.EAST);
		frame.add(input, BorderLayout.CENTER);

		
		frame.pack();

		frame.setVisible(true);
		
		

		ActionListener a1 = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
				startAt = startAt + 10;
				endAt = endAt + 10;

				updateRange();

				reFresh();

			}
		};

		ActionListener a2 = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			
				if (startAt > 10) {
					startAt = startAt - 10;
					endAt = endAt - 10;

					updateRange();
					
					reFresh();

				}

			}
		};
		
		ActionListener inputAction = new ActionListener(){
			public void actionPerformed(ActionEvent e){
//				String inputText = e.get
				String inputText = input.getText();
				startAt = Integer.parseInt(inputText);
				endAt = (startAt + 9);
				
				updateRange();
				
				reFresh();
			}
		};

		buttonUp.addActionListener(a1);
		buttonDown.addActionListener(a2);
		input.addActionListener(inputAction);
	}

	/* Manuelles refrsehen er Anzeige - noetig z.B. wenn der Speicher ueber
	 * die loadObjectFile-Methode beschrieben wurde
	 */
	public void reFresh(){

		for (int i = 0; i < (endAt - startAt); i++) {
			getTable().getModel().setValueAt(
					Integer.toString(startAt + i), i, 0);
			getTable().getModel().setValueAt(
					Integer.toString(getMemory(startAt + i)), i, 1);
			getTable().getModel().setValueAt(
					"0x" + Integer.toHexString(getMemory(startAt + i)), i, 2);
			getTable().getModel().setValueAt(
					Integer.toBinaryString(getMemory(startAt + i)), i, 3);
			
			//################### EDIT FUER DISAS ANZEIGE
			try {
				getTable().getModel().setValueAt(engine.getMemory().getDisassembledInstructionAt(startAt + i)==null?
					"NONE" :engine.getMemory().getDisassembledInstructionAt(startAt+i),(i), 4);
					
			}
			catch (Exception exc) {
				getTable().getModel().setValueAt("NONE",(i), 4);		
			}
		}
		getFrame().setTitle(
				"Speicherbereich von " + startAt + " bis " + endAt);

		
	}
	
	protected JTable getTable() {
		return this.tabelle;
	}

	protected JFrame getFrame() {
		return this.frame;
	}

	public void setStartAt(int newStartAt) {
		this.startAt = newStartAt;
	}

	public void setEndAt(int newEndAt) {
		this.endAt = newEndAt;
	}

	public int getStartAt() {
		return this.startAt;
	}

	public int getEndAt() {
		return this.endAt;
	}

	private void updateRange() {
		pluginMem.setNewRange(this, startAt, endAt);
	}

	private int getMemory(int adr) {
		try {
		return pluginMem.pluginGetMemory(this, adr); }
		catch (Exception  e1){
			System.out.println(e1.toString());
			System.out.println("Fehler!");
		
			return 0;
			
		}
	}

	public void setPluginManager(IPluginManager newPluginManager) {
		this.pluginMem = newPluginManager;
	}

	public void setObserver(boolean observe) {
		this.observer = observe;
	}

	public boolean isObserver() {
		return this.observer;
	}

}
