package hicosim.components.plugins.vga;

import hicosim.components.backend.processor.IPluginManager;
import hicosim.components.plugins.IPlugin;
import hicosim.control.Engine;
import hicosim.control.utils.*;


/**
 * ProcessASCII-plugin observes the memory range 0x4000 - 0x43e7,
 * which is the valid range for interpreting ascii-letters in the
 * hicovec-scotchrace.
 * The plugin computes the new address and writes the new values into it.
 * 
 * @author Weber, Wager
 *
 */
public class ProcessASCII implements IPlugin {

	private int startAt;
	private int endAt;
	
	int letterOrSign;
	int readFrontColor;
	int readBackColor;
	
	int value;
	int currentAddress;
	int startASCII = 16384;
	int startVGA = 8192;
	
	int anzZeilen = 0;
	int[] targetAdress = new int[8];
	
	int offset320 = 0;
	int offset40 = 0;
	
	BitUtils bitUtils;
	
	private Engine e;
	
	public boolean observer;
	
	public ProcessASCII(Engine e){
		this.startAt = 16384;
		this.endAt = 17383;
		
		this.e = e;
		
		this.observer = true;
		this.bitUtils = new BitUtils();
		
		//##################   SET UP DER LookUpTable einmalig!
		LookUpTable.setUp();
	}
	
	
	public int getEndAt() {
		return this.endAt;
	}

	public int getStartAt() {
		return this.startAt;
	}

	public boolean isObserver() {
		return this.observer;
	}

	/*
	 * In diesem Plugin: ASCII-Zeichen umwandeln / in neuen Memory schreiben!
	 */
	public void notifyPlugin(int address, int value) {

		if(value == 0){
			return;
		}

		/* Calculate Row and TargetAddress of given address */
		int tmpRow = calculateRow(address);
		calculateTarget(address, tmpRow);
		
		//die letzten 8 bit sind immer der Buchstabe oder ein Zeichen..(ASCII)
		letterOrSign = value & 0xff;
		//die Vordergrundfarbe immer bit 20 21 22 23; von rechts, bei 0 beginnend,32 bit
		readFrontColor = (value>>>8) & 0xF; //..... 1111 00000000						
		//die Hintergrundfarbe immer bit 16 17 18 19; von rechts, bei 0 beginnend,32 bit
		readBackColor = (value>>>12) & 0xF; //....11110000 00000000
		
		checkSigns(value, letterOrSign, readFrontColor, readBackColor);

	}
	
	
	private void checkSigns(int readValASCII, int letterOrSign, int readFrontColor, int readBackColor) {
		boolean alreadyUsed=false;		
		
		if(letterOrSign >= 65 && letterOrSign<=90 ) {//GROSSBUCHSTABEN:
			alreadyUsed=true;
			char let = (char)letterOrSign;			
			this.writeToVGA_RAM(LookUpTable.ALUT(let), readFrontColor, readBackColor);							
		}
		
		else if(letterOrSign >= 48 && letterOrSign<=57 ) {//ZAHLEN:
			alreadyUsed=true;
			char zahl = (char)letterOrSign;						
			this.writeToVGA_RAM(LookUpTable.ALUT(zahl), readFrontColor, readBackColor);
		}
		
			switch (letterOrSign) {														
			//-------------------------------------------SONDERZEICHEN:
			case 33: // '!'
				alreadyUsed=true;					
				this.writeToVGA_RAM(LookUpTable.ALUT('!'), readFrontColor, readBackColor);
				break;							
			case 63: // '?'
				alreadyUsed=true;					
				this.writeToVGA_RAM(LookUpTable.ALUT('?'), readFrontColor, readBackColor);
				break;							
			case 46: // '.'
				alreadyUsed=true;				
				this.writeToVGA_RAM(LookUpTable.ALUT('.'), readFrontColor, readBackColor);
				break;	
			case 58: // ':'
				alreadyUsed=true;								
				this.writeToVGA_RAM(LookUpTable.ALUT(':'), readFrontColor, readBackColor);
				break;	
			case 45: // '-'
				alreadyUsed=true;				 					
				this.writeToVGA_RAM(LookUpTable.ALUT('-'), readFrontColor, readBackColor);
				break;	
			case 95: // '_'
				alreadyUsed=true;	
				this.writeToVGA_RAM(LookUpTable.ALUT('_'), readFrontColor, readBackColor);
				break;															
												
			case 32: // ' ' --> SPACE, ein LEERZEICHEN MACHEN	
				alreadyUsed=true;				
				this.writeToVGA_RAM(LookUpTable.ALUT(' '), readFrontColor, readBackColor);
				break;
				
			case 116: //'t' SCOTCH RACE TREE
				alreadyUsed=true;			
				this.writeToVGA_RAM(LookUpTable.ALUT('t'), readFrontColor, readBackColor);
				break;
			case 107: //'k' SCOTCH RACE kleeblatt
				alreadyUsed=true;				
				this.writeToVGA_RAM(LookUpTable.ALUT('k'), readFrontColor, readBackColor);
				break;
			case 98: //'b' SCOTCH RACE bomb
				alreadyUsed=true;							
				this.writeToVGA_RAM(LookUpTable.ALUT('b'), readFrontColor, readBackColor);
				break;
				
			case 252: //NICHTS TUN HIER; KOMMT IMMER NACH NEM STRING
				return;								
				
		//FUER ALLES WAS NICHT ERKANNT WURDE EIN 8x8 VIERECK IN DER VORDERGRUNDFARBE ZEICHNEN ...
			default:
				if(readValASCII!=0 && (!(alreadyUsed)) ) {											
					this.writeToVGA_RAM(LookUpTable.ALUT('#'), readFrontColor, readBackColor);
				}

			} 		
	}
	
	
	/**
	 * This method writes a letter or sign into the graphical memory area.
	 * @param signOrLetter   - int
	 * @param readFrontColor - int
	 * @param readBackColor  - int
	 */
	private void writeToVGA_RAM(String signOrLetter, int readFrontColor, int readBackColor) {
		char arr[] = signOrLetter.toCharArray();
		
		if(readFrontColor==readBackColor && readBackColor==0) {
			readFrontColor=15;
		}
		String hexWord = "";
		
		int writeAdd = getTargetAdr()[0];
		
		for(int i = 0; i<64; i++) {
			if(arr[i]=='1'){
				hexWord += Integer.toHexString(readFrontColor);
			}
				
			else {
				hexWord += Integer.toHexString(readBackColor);
			}
				
			
			if((i+1)%8==0) {
				
				try {

					
					int value=0;
					try {
						value = BitUtils.hex2Decimal(hexWord);								
					}
					catch (Exception e) {
						System.err.println("ERROR! (in hex2Decimal) STOPPING VGA_PLUGIN.");

					}
				

						this.e.getMemory().writeTo(writeAdd, value);

						writeAdd = writeAdd + 40;
						
					
				}
				catch (Exception e) {

					System.out.println(e.getMessage());
					e.printStackTrace() ;

				}
				
				hexWord="";																			
			}
			
		}//end for
		

	}	
	
	private void calculateTarget(int adr, int row){
		int[] retInt = new int[8];
		
		/* Adressberechnung:
		 * 
		 */
		retInt[0] = startVGA + ((adr - startASCII) + (row * 280));
		retInt[1] = startVGA + (((adr - startASCII) + 40) + (row * 280));
		retInt[2] = startVGA + (((adr - startASCII + 80)) + (row * 280));
		retInt[3] = startVGA + (((adr - startASCII + 120)) + (row * 280));
		retInt[4] = startVGA + (((adr - startASCII + 160)) + (row * 280));
		retInt[5] = startVGA + (((adr - startASCII + 200)) + (row * 280));
		retInt[6] = startVGA + (((adr - startASCII + 240)) + (row * 280));
//		retInt[7] = startVGA + (((adr - startASCII + 280)) + (row * 280));
				
		setTargetAdr(retInt);

	}
	
	/* Calculate row of given address */
	private int calculateRow(int adr){
		int retInt = 0;
		
		//TODO rechnet zu viel...
		while((adr - 40) >= startASCII){
			retInt = retInt + 1;
			adr = adr - 40;
		}	
		return retInt;
	}
	
	private void setTargetAdr(int[] newAdr){
		this.targetAdress = newAdr;
	}
	
	private int[] getTargetAdr(){
		return this.targetAdress;
	}
	
		
	public void reFresh() {
		
	}

	
	public void setObserver(boolean observe) {
		this.observer = observe;
	}

	
	public void setPluginManager(IPluginManager newManager) {
		
	}
	
	public String toString(){
		return "ProcessASCII-Plugin";
	}

}
