package hicosim.components.plugins.vga;

import hicosim.components.backend.processor.IPluginManager;
import hicosim.components.plugins.IPlugin;
import hicosim.control.Engine;

public class ProcessVGA implements IPlugin {
	
	
	private Engine e;
	private int startVGA = 8192;
	private int endVGA = 16152;
	
	boolean observer;
	
	private final int startVideoRAM = 2147403647;
	
	private int[] readPixel;

	public ProcessVGA(Engine e){
		this.e = e;
		this.observer = true;
		
		readPixel = new int[8];
	}
	
	
	public int getEndAt() {
		return this.endVGA;
	}

	
	public int getStartAt() {
		return this.startVGA;
	}

	
	public boolean isObserver() {
		return this.observer;
	}

	
	public void notifyPlugin(int address, int value) {
//		System.out.println("notify on vgaplugin called");
		
		if(value == 0){
			return;
		}
		
		//die pixel aus dem 32 bit wort holen:
//		readPixel[0] = value >>> 28;
//		readPixel[1] = (value << 4) >>> 28;
//		readPixel[2] = (value << 8) >>> 28;
//		readPixel[3] = (value << 12) >>> 28;
//		readPixel[4] = (value << 16) >>> 28;
//		readPixel[5] = (value << 20) >>> 28;
//		readPixel[6] = (value << 24) >>> 28;
//		readPixel[7] = (value << 28) >>> 28;
		readPixel[0] = value >>> 28;
		readPixel[1] = (value>>>24) & 0xF ;
		readPixel[2] = (value>>>20) & 0xF ;
		readPixel[3] = (value>>>16) & 0xF ;
		readPixel[4] = (value>>>12) & 0xF ;
		readPixel[5] = (value>>>8)  & 0xF ;
		readPixel[6] = (value>>>4)  & 0xF ;
		readPixel[7] = value & 0xF ;
	
		int rowTmp = calculateRow(address);
		int target = calculateTargetAddress(address, rowTmp);
//		System.out.println("target: " + target);
//		System.out.println("row: " + rowTmp);
		
		/* writing pixels in vga-ram */
		this.e.getMemory().writeTo(target++, readPixel[0]);
		this.e.getMemory().writeTo(target++, readPixel[1]);
		this.e.getMemory().writeTo(target++, readPixel[2]);
		this.e.getMemory().writeTo(target++, readPixel[3]);
		this.e.getMemory().writeTo(target++, readPixel[4]);
		this.e.getMemory().writeTo(target++, readPixel[5]);
		this.e.getMemory().writeTo(target++, readPixel[6]);
		this.e.getMemory().writeTo(target++, readPixel[7]);
		
	}

	private int calculateRow(int adr){
		int row = 0;
		int tmpAdr = adr;
		
		while(((tmpAdr) - 40) >= startVGA){
			tmpAdr = tmpAdr - 40;
			row = row + 1;
		}
		
		return row;
	}
	
	public int calculateTargetAddress(int adr, int row){
		int tmpInt;

		tmpInt = adr + row * 320;

		/* Spalte berechnen */
		int tmpCol = adr - (startVGA + row * 40);
//		System.out.println("col: " + tmpCol);
		tmpInt = (tmpInt - adr) + startVideoRAM + (tmpCol * 8);

		return tmpInt;
	}
	
	
	public void reFresh() {

	}

	
	public void setObserver(boolean observe) {
		
	}

	
	public void setPluginManager(IPluginManager newManager) {
		
	}

}
