package hicosim.components.plugins.vga;

import hicosim.components.backend.processor.IMemory;
import hicosim.control.Engine;
import hicosim.control.utils.BitUtils;

import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.awt.image.WritableRaster;

/**
 *
 * @author Daniel Obermueller
 *
 */
public class VGAPlugin extends JPanel implements KeyListener {
	/**
	 *
	 */
	private static final long serialVersionUID = 1L;
	private int height;
	private int width;
	protected int pixels[];
	VGAListener vga_listener;
	private Engine e;

	private BufferedImage bufferedImage;
	private WritableRaster writableRaster;

	private JFrame frame = null;

	public VGAPlugin(int width, int height, Engine e) {
		super();
		this.e = e;
		this.height = height;
		this.width = width;
		this.pixels = new int[width * height];
		setIgnoreRepaint(true);
		bufferedImage = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_ARGB);
		writableRaster = bufferedImage.getRaster();
		DataBufferInt dataBufferInt = (DataBufferInt) writableRaster
				.getDataBuffer();
		pixels = dataBufferInt.getData(0);

		/* Hier Plugin erzeugen, das ASCII umwandelnt und in 0x2000 schreibt */
		ProcessASCII plugin_ascii = new ProcessASCII(e);
		plugin_ascii.setObserver(true);
		e.addPlugin(plugin_ascii);

		/*
		 * NOTE: Die anderen zwei IPlugins werden ignoriert und ein thread
		 * ueberwacht diese bereiche
		 */

//		ProcessVGA plugin_vga = new ProcessVGA(e);
//		plugin_vga.setObserver(true);
//		e.addPlugin(plugin_vga);

//		ProcessMAIN_RAM plugin_main = new ProcessMAIN_RAM(e, this);
//		plugin_main.setObserver(true);
//		e.addPlugin(plugin_main);

		this.vga_listener = new VGAListener(this, e);
		// starte Thread um VGA Adressen durchzugehn
		this.vga_listener.start(); // muss thread sein sonst haengt er hier ...
		// ... und es ist nicht mehr moeglich weitere Eingaben zu machen
	}

	/**
	 * This method should be called from GUI to stop the rendering Process.
	 */
	public void stopRendering() {
		if (this != null && this.vga_listener != null) {
			this.vga_listener.stop();
		}
	}

	/**
	 * This method should only be used from consoleApp to stop the rendering
	 * Process.
	 */
	public void stopRenderFromConsole() {
		if (frame != null && this != null && this.vga_listener != null) {
			this.frame.setVisible(false);
			this.setVisible(false);
			this.vga_listener.stop();
		}
	}

	// GETTER:
	public int width() { // == x
		return width;
	}

	public int height() {// == y
		return height;
	}

	/**
	 * This method is only used from consoleApp to show the VGA Screen to stop
	 * the rendering Process.
	 */
	public void setUpFrame() {
		// HIER AENDERN WENN IN GUI EINBAUEN..
		frame = new JFrame();
		frame.getContentPane().add(this);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				vga_listener.stop();
			}
		});
		// frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(width, height);
		frame.setVisible(true);
		frame.setResizable(false);

		frame.setAlwaysOnTop(true);
		frame.setLocation(800, 520);

		// fuer KeyListener auf ein JPanel:
		this.setFocusable(true);
		this.addKeyListener(this);
	}

//	public synchronized void setPixelColor(int x, int y, int color) {
//		long start = System.currentTimeMillis();
//		pixels[y * width + x] = color; //opaque << 24 | red << 16 | green << 8 | blue;
//		System.out.println(System.currentTimeMillis() - start);
//	}

	
	public synchronized void paintComponent(Graphics g) {
		setDataElementsAndPaint(g);
	}

	private synchronized void setDataElementsAndPaint(Graphics g) {
		// long start = System.currentTimeMillis();
		// for (int y = 0; y < height; y++) {
		// writableRaster.setDataElements(0, y, height, 0, pixels);
		// }
		writableRaster.setDataElements(0, 0, width, height, pixels);
		g.drawImage(bufferedImage, 0, 0, null);
		// long end = System.currentTimeMillis();
		// System.out.println(end - start);
	}

	// public void paint(Graphics g) {
	// memoryImageSource = new MemoryImageSource(width, height, pixels, 0,
	// width);
	// memoryImageSource.setAnimated(true);
	// Image img = createImage(memoryImageSource);
	// g.drawImage(img, 0, 0, this);
	// }

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == 40) {// DOWN     ENTER: 10
			this.e.getMemory().writeTo(65535, 1024);
		}
		if (e.getKeyCode() == 37)// LINKS / LEFT
			this.e.getMemory().writeTo(65535, 16);
		if (e.getKeyCode() == 39)// RECHTS / RIGHT
			this.e.getMemory().writeTo(65535, 128);
		// ignore everything else
	}

	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == 40)// DOWN     ENTER: 10
			this.e.getMemory().writeTo(65535, 0);
		if (e.getKeyCode() == 37)// LINKS / LEFT
			this.e.getMemory().writeTo(65535, 0);
		if (e.getKeyCode() == 39)// RECHTS / RIGHT
			this.e.getMemory().writeTo(65535, 0);
	}

	public void keyTyped(KeyEvent e) {
	}

	// ########################################################## MAIN
//	 public static void main(String[] args) {
//	 }
}

/**
 * This class is used to check VGA-RAM and render to screen.
 *
 * @author Michael Wager , 18th May 2009
 */
class VGAListener extends Thread {
	private VGAPlugin vgaPlugin = null;
	private IMemory mem = null;

	private final int VGA_GRAPHIC_BEGIN = 0x1fff; // Adressen der VGA-Zeilen
	// 0x2000
	private final int VGA_GRAPHIC_END = 0x3F18; // von 0x2000 bis 0x3F18 ????

	private final int VGA_RAM_START = 2147403647; // == Integer-MAX - 80000

	private int vgaGraphicAddress = 0;
	private int vgaRAM_READAddress = 0;
	private int vgaRAM_WriteAddress = 0;

	public VGAListener(VGAPlugin p, Engine e) {
		this.vgaPlugin = p;
		this.mem = e.getMemory();

		// starte timer, welcher immer wieder repaint() aufruft
		new javax.swing.Timer(60, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				vgaPlugin.repaint();
			}
		}).start();

	}

	/**
	 * This method checks the VGA Memory and renders the screen depending on the
	 * values in the RAM.
	 */
	public void run() {
		/*
		 * Es gibt 3 Speicherbereiche. Einen fuer reinen Grafikmodus(ab 0x2000)
		 * und einen fuer ASCII Modus(ab 0x4000) und einen um die 'Pixel' bzw.
		 * deren Farben hineinzuschreiben. Wenn in dem ASCII Bereich zB eine 65
		 * steht(ASCII fuer 'A') wird direkt ein 8x8 Zeichen in den
		 * Schreibbereich geschrieben. In den reinen Grafikmodus-bereich kann
		 * man dann die Pixel einzeln zeichnen. Dort wird dann ein 32bit wort
		 * als 8 pixel a 4bit interpretiert und in den Schreibbereich
		 * geschrieben. Der Schreibbereich(Size: 320*240) wird dann immer
		 * ausgelesen und entsprechend des dort gelesenen Werte die Farbe fuer
		 * einen Pixel gesetzt.
		 */

		// System.out.println(" VGA_Listener: STARTING READ/RENDER -
		// THREAD...");
		// SIMULATION EINIGER BUCHSTABEN:
		// this.mem.writeTo(0x4000, 65);//A
		// this.mem.writeTo(0x4001, 66);//B
		// this.mem.writeTo(0x4002, 67);//C
		int color = 0;

		int tmp = 0;

		// ein 32bit wort besteht aus 8 pixel a 4bit!!!
		int[] readPixel = new int[8];

		int readValGraphic = 0;// der aktuell aus GRAFIK RAM gelesene wert
		int readVGA_RAM_val = 0;// der aktuell aus Farbenbereich gelesene wert

		int limit_graphic = VGA_GRAPHIC_BEGIN
				+ (VGA_GRAPHIC_END - VGA_GRAPHIC_BEGIN);
		while (true) {

			// ----------- RESET! -------------
			// dies sind die gelesenen Adressen der ASCII Zeichen:
			// vgaASCIIAddress = VGA_ASCII_BEGIN ;
			// dies sind die gelesenen Adressen der VGA Zeilen:
			vgaGraphicAddress = VGA_GRAPHIC_BEGIN;

			// dies sind die Adressen um den VGA RAM zu lesen (just increment,
			// read and setPixelCOLOR)
			vgaRAM_READAddress = VGA_RAM_START;
			// dies sind die Adressen zum Schreiben:
			vgaRAM_WriteAddress = VGA_RAM_START;

			int offset = 0, graphic_offset = 0; // reset

			for (int y = 0; y < vgaPlugin.height(); y++) {
				for (int x = 0; x < vgaPlugin.width(); x++) {

					readValGraphic = 0;// reset
					if (vgaGraphicAddress <= VGA_GRAPHIC_END) {
						try {
							readValGraphic = this.mem.readValueAt(vgaGraphicAddress);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
					if (readValGraphic != 0) {
						// CHECK GRAPHIC MEMORY:
						// die pixel aus dem 32 bit wort holen:
//						readPixel[0] = readValGraphic >>> 28;
//						readPixel[1] = (readValGraphic << 4) >>> 28;
//						readPixel[2] = (readValGraphic << 8) >>> 28;
//						readPixel[3] = (readValGraphic << 12) >>> 28;
//						readPixel[4] = (readValGraphic << 16) >>> 28;
//						readPixel[5] = (readValGraphic << 20) >>> 28;
//						readPixel[6] = (readValGraphic << 24) >>> 28;
//						readPixel[7] = (readValGraphic << 28) >>> 28;
						readPixel[0] = readValGraphic >>> 28;
						readPixel[1] = (readValGraphic>>>24) & 0xF ;
						readPixel[2] = (readValGraphic>>>20) & 0xF ;
						readPixel[3] = (readValGraphic>>>16) & 0xF ;
						readPixel[4] = (readValGraphic>>>12) & 0xF ;
						readPixel[5] = (readValGraphic>>>8)  & 0xF ;
						readPixel[6] = (readValGraphic>>>4)  & 0xF ;
						readPixel[7] = readValGraphic & 0xF ;

						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[0]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[1]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[2]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[3]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[4]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[5]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[6]);
						this.mem.writeTo(vgaRAM_WriteAddress++, readPixel[7]);
//						readPixel = new int[8];
					} else { // wenn an LineAdd nix steht, ein Zeichen bzw 8
						// pixel weiter:
						vgaRAM_WriteAddress += 8;
					}

					try {
						readVGA_RAM_val = this.mem.readValueAt(vgaRAM_READAddress);
					} catch (Exception e) {
						e.printStackTrace();
					}
					// ------------------ DIE FARBEN SETZEN:
					color = LookUpTable.CLUT(readVGA_RAM_val);
					// FARBE FUER EINEN PIXEL SETZEN:
					vgaPlugin.pixels[y * vgaPlugin.width() + x] = color; //setPixelColor(x, y, color);

					// OVERWRITE:
					try {
						this.mem.writeTo(vgaRAM_READAddress, 0);
					} catch (Exception e) {
						// ignore
					}

					// NEUE ZEILE:
					if (vgaGraphicAddress == (VGA_GRAPHIC_BEGIN + offset)) {
						vgaRAM_WriteAddress = (VGA_RAM_START + graphic_offset);
						offset += 40;// immer wenn im 0x2000er.. Bereich eine neue Zeile erreicht...
						graphic_offset += 320;// ...muss im VGA_RAM auch eine
						// zeile weitergeschoben werden
					}
//					 ######### Adressen erhoehen

					if (vgaGraphicAddress <= (limit_graphic)) {
						vgaGraphicAddress++;
					}

					vgaRAM_READAddress++;

				}// end first for

			}// ->end all for

		}// end while true
	}// end run()

}