package hicosim.control;


import hicosim.components.backend.processor.Labels;
import hicosim.components.backend.processor.Memory;
import hicosim.components.backend.processor.Processor;
import hicosim.components.backend.io.Sourcefile;
import hicosim.components.frontend.iListenerFrontend;
import hicosim.control.utils.BitUtils;
import hicosim.control.utils.ExecuteUtils;
import hicosim.control.utils.Messages;
import hicosim.control.utils.Numeratives;
import hicosim.control.ProcessorConfig;
import hicosim.components.plugins.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Vector;

/**
 * This class is the interface between the front- and backend
 * and is used by clients(such as commandProcessor) to call
 * methods in the backend.
 * @author Michael Wager, Andreas Weber, Luca Calchera 29th May 2009
 *
 */
public class Engine {

	//interface references to memory and pocessor
	protected hicosim.components.backend.processor.IProcessor processorI;
	protected hicosim.components.backend.processor.IMemory memoryI;

	//Engine verwaltet sourcecode
	private Sourcefile srcFile=null;

	/* Variable fuer die Konfigurationseinstellung des HiCoVec */
	private ProcessorConfig config;
	private ExecuteUtils exeUtil=null;
	private iListenerFrontend frontendListener=null;

	private StringBuffer breaksAtAdds=new StringBuffer();
	private StringBuffer breaksAtLabels=new StringBuffer();

	private int currentLineNumber=0;

	private boolean processorRunning=false;

	private char numerativeFlag='d'; //decimal default

	/**
	 * Setup processorConfiguration, processor and memory and the utils.
	 */

	public Engine() {

		this.memoryI=new Memory(this);

		//lese einmalig die config ein:
		this.config = new ProcessorConfig(this);

		this.processorI = new Processor(config, this); //Processor bekommt auch die Engine !!!
		this.exeUtil=new ExecuteUtils();

		this.srcFile = new Sourcefile(this);

//		IPlugin TestPlugin = new TestPlugin(memoryI.getPluginManager());
//		memoryI.getPluginManager().addPlugin(TestPlugin, 13000, 100000);
//
	}
	public Processor getProcessor(){
		return (Processor)this.processorI;
	}
	public Memory getMemory(){
		return (Memory)this.memoryI;
	}
	public iListenerFrontend getListener() {
		if(frontendListener!=null)
			return this.frontendListener;
		else return null;
	}

	public void focusAtCommandline(){
		this.frontendListener.focusAtCommandline();
	}

	public void addListener(iListenerFrontend listener) throws Exception {
		if(listener!=null) {
			this.frontendListener = listener;
			//HIER DIE INIT MESSAGE AUCH AN GUI SCHICKEN:
			displayConsole(Messages.initialMessage + "\n" + Messages.getHelp(), false);

			//===> REIHENFOLGE SO BELASSEN !!! !!! !!! !!! !!!

			//UND DIE LAENGE DER CONFIG FUER VEC-REGS HOCHSCHICKEN:
			setJTableVectorCpuRegisterOverviewSize(this.config.getN(), this.config.getK());
			//und auch die config selbst:
			this.config.displayCPUConfiguration(false);

			//und die reg-infos auf GUI anzeigen
			displayStatusRegister(this.processorI.getCarryFlag(), this.processorI.getZeroFlag()==1 ? true : false);
			displayScalarRegChanges(this.processorI.getRegA(), this.processorI.getRegX(), this.processorI.getRegY());
			displayVectorRegChanges(this.processorI.getVectorRegisters());

//			checkNumerativeFlagAndDisplay('A',this.processorI.getRegA());
//			checkNumerativeFlagAndDisplay('X',this.processorI.getRegX());
//			checkNumerativeFlagAndDisplay('Y',this.processorI.getRegY());
//			checkNumerativeFlagAndDisplay('v', 0);

		}
		else
			throw new Exception("Listener is null!");
	}
	/**
	 * This method is used by backend to show messages(like next instruction to execute)
	 * 	in the GUI Konsole or in the commandline. ---> CALLBACK
	 * @param message - String - the message to display.
	 * @param error - boolean -  if the message is an errorMessage or not.
	 */
	public void displayConsole(String message, boolean error) {
		if(this.frontendListener!=null) {
			if(error)
					this.frontendListener.displayConsoleError(message);
				else
					this.frontendListener.displayConsoleMessage(message);
		}
		else  {//SONST EINFACH AUSGEBEN !!!
			if(error)
				System.err.println(message);
			else
				System.out.println(message);
		}
	}

	public void setJTableVectorCpuRegisterOverviewSize(int rows, int columns) {
		if(this.frontendListener!=null) {
			this.frontendListener.setJTableVectorCpuRegisterOverviewSize(rows, columns);
			for(int N=0; N<config.getN(); N++) {
				for(int K=0; K<config.getK() ; K++) {
					this.processorI.setVRegsValue(N, K, this.processorI.getVRegsValue(N, K));
				}
			}
		}
	}
	
	public void displayInstructionPointer(int ip) {
		if(this.frontendListener!=null)
			this.frontendListener.displayInstructionPointer(ip);
	}
	
	public void displayScalarRegChanges(Object valOfA, Object valOfX, Object valOfY) {
		if(this.frontendListener!=null)
			this.frontendListener.displayScalarRegChanges(valOfA, valOfX, valOfY);
	}
	public void displayVectorRegChanges(int [][]vRegs) {
		if(this.frontendListener!=null)
			this.frontendListener.displayVectorRegChanges(vRegs);
	}
	public void displaySingleVecValue(int i, int j, Object value) {
		if(this.frontendListener!=null) {
			switch(numerativeFlag) {
			case 'd':
				this.frontendListener.displaySingleValueOfVectorRegister(value, i, j);
				break;
			case 'h':
				this.frontendListener.displaySingleValueOfVectorRegister(value, i, j);
				break;
			case 'b':
				this.frontendListener.displaySingleValueOfVectorRegister(value, i, j);
				break;
			}
		}
			
	}
	/**
	 * display status register content
	 */
	public void displayStatusRegister(int carry, boolean zero) {
		if(this.frontendListener!=null)    		 //if zero==true: show a '1' else a '0'
			this.frontendListener.displayStatusRegister(carry, zero ? 1:0);
	}
	/**
	 * This method is used to display the changed configuration in the GUI.
	 */
	public void displayCpuConfiguration(HashMap<String, String> config) {
		if(this.frontendListener!=null)
			this.frontendListener.displayCpuConfiguration(config);
	}
	public HashMap<String, String> getCPUConfiguration() {
		if(this.frontendListener!=null)
			return this.config.displayCPUConfiguration(true);
		else
			return null;
	}
	//-------------------------------------------------------------------------------------- BREAK_MARKS:
	public void displayBreakpointAtSourcecodeline(int index) {
		if(this.frontendListener!=null)
			this.frontendListener.setBreakpointMarkAtSourceCode(index);
	}
	public void displayBreakpointAtDisascodeline(int index) {
		if(this.frontendListener!=null)
			this.frontendListener.setBreakpointMarkAtDisassembledCode(index);
	}
	public void deleteBreakpointAtSourcecodeline(int index) {
		if(this.frontendListener!=null)
			this.frontendListener.deleteBreakpointMarkAtSourceCode(index);
	}
	public void deleteBreakpointAtDisascodeline(int index) {
		if(this.frontendListener!=null)
			this.frontendListener.deleteBreakpointMarkAtDisassembledCode(index);
	}
	public void highlightSourcecodeline(int index) {
		if(this.frontendListener!=null) {
			try {
				this.frontendListener.highlightLineOfSourceCode(index);
			}
			catch (Exception e) {
				// ignore
			}
		}
	}
	public void highlightDisascodeline(int index) {
		if(this.frontendListener!=null) {
			try {
				this.frontendListener.highlightLineOfDisassembledCode(index);
			}
			catch (Exception e) {
				// ignore
			}
		}
	}
	//--------------------------------------------------------------------------------------
	

	public void addPlugin(IPlugin newPlugin){
		newPlugin.setPluginManager(this.memoryI.getPluginManager());
		this.memoryI.getPluginManager().addPlugin(newPlugin, newPlugin.getStartAt(), newPlugin.getEndAt());
		newPlugin.reFresh();
//				System.out.println("Testen in der Engine bei addPlugin:");
//		System.out.println("newPlugin: ");
//		System.out.println(newPlugin.toString());
//		System.out.println("StartAt: " + newPlugin.getStartAt() + " und EndAt: " + newPlugin.getEndAt());
//		System.out.println("PluginManager: ");
//		System.out.println(this.memoryI.getPluginManager().toString());
	}

	/**
	 * This method uses the memoryReference
	 * to load the objectfile.
	 * It also tries to read in the sourcefile.
	 *
	 * @param PATH - PATH to the objectFile
	 * @return - InfoString == "" if everything is OK.
	 * @throws Exception
	 */
	public String loadObjectFile(String PATH)throws Exception {
		if(!(PATH.contains(".o")))
			throw new Exception("Objectfile must end with '.o' --> " + PATH);
		try {
			File f = new File(PATH);
		}
		catch (Exception e) {
			throw new Exception("FILE "+PATH+" NOT FOUND.");
		}
		//try reading the sourcefile too:
		String PATH2=PATH.replace(".o", "");
		PATH2 += ".asm";


		//VERSUCHEN OB ES SRCFILE MIT GLEICHEM NAMEN GIBT:
		try {
			this.readSourcefile(PATH2);
			if(this.frontendListener!=null)
				this.frontendListener.displaySourceCode(this.getSourceCodeLines());
		}
		catch (Exception e) {
//			System.err.println("SOURCEFILE: " + PATH2 + " - COULD NOT BE READ IN. " +
//			"IF YOU WANT TO SHOW IT YOU HAVE TO SPECIFY THE PATH.");
		}
		//--> SONST KANN MAN SIE MIT "list" jetzt ausgeben!!!

		try {
			//standard reset vorher:
			String re = this.memoryI.loadObjectfile(PATH);
			return re;
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	/**
	 * This method is used to know if the processor is running or not.
	 */
	public void setProcessorRunning(boolean running) {
		processorRunning = running;
	}
	public boolean isProcessorRunning() {
		return this.processorRunning ;
	}

	public void reset() throws Exception {
//		if(!this.isObjectFileLoaded())   --> muss auch vorher schon gehn, wenn man zB setr genutzt hat
//			throw new Exception("NOTHING TO RESET. OBJECTFILE NOT LOADED.");

		try {
			this.processorI.reset();
			this.setProcessorRunning(false);
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO RESET CPU, OBJECTFILE ALREADY LOADED?");
		}
	}
	public void resetMemory() throws Exception {
//		if(!this.isObjectFileLoaded())  --> muss auch vorher schon gehn, wenn man zB set genutzt hat
//			throw new Exception("NO MEMORY CONTENT TO RESET. OBJECTFILE NOT LOADED.");

		try {
			this.memoryI.resetMemory();
			this.processorI.setOFileNotLoaded();
			setProcessorRunning(false);
			this.deleteAllBr(); //RESET MEMORY LOESCHT AUCH ALLE BREAKPOINTS
			//SONST IST ES SEHR LAESTIG WENN MAN EIN ANDERES OBJECTFILE
			//LADET UND DIE BREAKS VOM ALTEN NOCH GESETZT SIND!!!

		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO RESET MEMORY, OBJECTFILE ALREADY LOADED?");
		}
	}
	/*
	 * This method starts the execution of the processor,
	 * while using the private method engine.process();
	 * This method is only used by the HICOSIM consoleApp,
	 * gui uses class components/frontend/RunThread
	 * to start the execution.
	 */
	public void run() throws Exception {
		if(this.isObjectFileLoaded()) {
			if(processorI.isHALT()) {
				throw new Exception("ALREADY REACHED HALT INSTRUCTION");
			}
			this.processorI.setIsRunning(true);
			this.processorI.setStepping(false);
			setProcessorRunning(true);
			try {
				if(this.frontendListener==null) {
					InterruptThread t = new InterruptThread(this);
					t.start();
					if(this.frontendListener==null)
						System.out.println("running...Type 'i' + <ENTER> to interrupt.");
					this.process();
					t.stopThread();
					t=null;

				}
			}
			catch (Exception e) {
				throw new Exception("ERROR WHILE TRYING TO RUN, OBJECTFILE ALREADY LOADED?");
			}
			try {
				this.highlightSourcecodeline(this.srcFile.getLineNumberAtAddress(this.getIP())-1 );
				this.highlightDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(this.getIP()));
			}
			catch (Exception e) {
				//ignore..
			}
		}
		else
			throw new Exception("OBJECTFILE NOT LOADED.");
	}
	/**
	 * This method makes executes one single instruction.
	 * @param run - if client wants to execute just one instruction, run has to be false.
	 * 						This also shows the disassembly output of executed instruction.
	 * 						Else it can be used to run in a loop.
	 * @throws Exception
	 */
	public void step(boolean run) throws Exception {
		if(!this.isObjectFileLoaded())
			throw new Exception("ERROR WHILE TRYING TO STEP. OBJECTFILE NOT LOADED.");
//		if(!this.isProcessorRunning())
//			throw new Exception("PROCESSOR IS NOT RUNNING");
		if(processorI.isHALT())
			throw new Exception("ALREADY REACHED HALT INSTRUCTION");

		try {
			if(!run)
				this.processorI.setStepping(true);
			this.processorI.step();
			if(this.frontendListener!=null) {
				currentLineNumber++;

				if(!run) {
					try {
						this.highlightSourcecodeline(this.srcFile.getLineNumberAtAddress(this.getIP()) - 1);
					}catch (Exception e) {} //ignore if srcFile not read in yet
					try {
						int line = this.memoryI.getDisassembler().getLineOfAddress(this.getIP());
						this.highlightDisascodeline(line);
					}catch (Exception e) {/* ignore this error: displayConsole("ERROR WHILE HIGHLIGHTING DISASCODELINE", true);*/}
				}
			}
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO STEP, OBJECTFILE ALREADY LOADED?, BREAKPOINT/s SET?");
		}
			}

	public void interrupt() throws Exception {
		if(!this.isObjectFileLoaded())
			throw new Exception("CANNOT INTERRUPT. OBJECTFILE NOT LOADED.");
		if(!(this.processorI.isRunning()))
			throw new Exception("CANNOT INTERRUPT. PROCESSOR NOT RUNNING.");

		try {
			this.processorI.interrupt();
			
			try {
				displayConsole("Simulation interrupted.", false);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			//nach interrupt zeile an der gehalten wurde highlighten:
			try {
				highlightSourcecodeline(this.getSourcefile().getLineNumberAtAddress(this.processorI.getIP())-1);
			
				highlightDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(this.processorI.getIP()));
			}
			catch (Exception e) {
				// ignore ...
			}
			
			//aktuellen Stand der Registeranzeige updaten:
			checkNumerativeFlagAndDisplayAll();
			this.displayInstructionPointer(getIP());
			this.displayStatusRegister(this.processorI.getCarryFlag(), this.processorI.getZeroFlag()==0 ? false:true);
			
		}
		catch (Exception e) {
			throw new Exception(e.getMessage());
		}
	}

	/**
	 * This method is used to run the processor from consoleApp.
	 */
	private void process() {
		while(this.processorI.isRunning()) {
			this.processorI.step();
		}
	}

	public boolean isObjectFileLoaded(){
	    return this.processorI.isObjectFileLoaded();
	}
	
	public boolean setWatchPoint(int adr){
		if(memoryI.isWatchPoint(adr)){
			displayConsole("Watchpoint already set at given address", true);
			return false;
		}
		memoryI.setWatchPoint(adr);
		return true;
	}
	
	public boolean isWatchPoint(int adr){
		return memoryI.isWatchPoint(adr);
	}
	
	public boolean delWatchPoint(int adr){
		if(isWatchPoint(adr)){
			memoryI.delWatchPoint(adr);
			return true;
		} else {
			displayConsole("No watchpoint set at given address " + adr, true);
			return false;
		}
	}
	public HashSet<Integer> getAllWatchPoints () {
		return this.memoryI.getAllWatchPoints();
	}

	/**
	 * This method delete all breakpoints
	 */
	public void deleteAllBr(){
		breaksAtAdds=new StringBuffer();
		breaksAtLabels=new StringBuffer();
		this.processorI.resetBr();
		if(this.frontendListener!=null) {   
			this.frontendListener.displayBreakpointAtLabel("");
			this.frontendListener.displayBreakpointAtAdress("");
			this.frontendListener.displayBreakpointAtLine("");
			this.frontendListener.deleteAllBr();
		}
	}

	public Vector<Integer> getBreakpoints() {
		return this.processorI.getBreakpoints();
	}
	
	/**
	 * This method interprets the parameter to the setBreakpoint-command and returns an InfoString
	 * or throws an Exception if this fails.
	 * @param paramBreak - parameter to set breakpoint command (decimal, 0xhex, label/+offset)
	 * @return String - InfoString
	 * @throws Exception
	 */
	public String setBreakpoint(String paramBreak,boolean deletebr) throws Exception{
		if(!this.isObjectFileLoaded())
			throw new Exception("OBJECTFILE NOT LOADED.");

		while(paramBreak.contains(" "))
			paramBreak=paramBreak.replace(" ", "");


		try {
			//kann man als dezimalzahl, hexzahl oder direkt als label+offset angeben:
			int breakAddress=0;
			boolean found=false;
			//AT LINE:
			if(paramBreak.contains("#")) {
				paramBreak=paramBreak.replace("#" , "");
				currentLineNumber = Integer.parseInt(paramBreak);
				try {
					breakAddress = this.srcFile.getAddressAtLine(currentLineNumber);
				}
				catch (Exception e) {
					displayConsole("CANNOT SET BREAKPOINT AT A LINENUMBER BECAUSE SOURCEFILE IS NOT READ IN YET", true);
					return "";
				}
				found=true;
			}

			//CASE: break <decimal>
			try {
				if(!found) {
					breakAddress = Integer.parseInt(paramBreak);
					found=true;
				}
			}catch (Exception e) {} //ignore, no decimal

			//CASE: break <0xHEX>
			if(paramBreak.contains("0x")) {
				paramBreak=paramBreak.replace("0x" , "");
				try {
					breakAddress = Integer.parseInt(paramBreak, 16);
					found=true;
				}catch (Exception e) {} //ignore, no hex
			}

			if(!found) {
				//CASE: break <label + offset>
				if(paramBreak.contains("+")) {
					paramBreak = paramBreak.replace(" ", "");
					paramBreak = paramBreak.replace("+", " ");
					String [] tmp = paramBreak.split(" ");
					String label = tmp[0];
					int offset = Integer.parseInt(tmp[1]);
					int addOfLabel = this.getAddressOfLabel(label);
					this.setBreakpointAtAddress(addOfLabel+offset,deletebr);
					if (this.frontendListener != null) {
						if (!deletebr) {
							if (breaksAtLabels.indexOf("Breakpoint set  at label: '"+ label + "' + " + offset + ", address: "+ (addOfLabel + offset) + "\n")==-1)
									breaksAtLabels.append("Breakpoint set  at label: '"+ label + "' + " + offset + ", address: "+ (addOfLabel + offset) + "\n");

							if(this.srcFile.getLines()!=null)
								this.displayBreakpointAtSourcecodeline(this.srcFile.getLineNumberAtAddress(addOfLabel + offset) - 1);
							this.displayBreakpointAtDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(addOfLabel + offset));
						}
						else{
                        	int i=breaksAtLabels.indexOf("Breakpoint set  at label: '"+ label + "' + " + offset + ", address: "+ (addOfLabel + offset) + "\n");
                        	int j=0;
                        	for (j=i;breaksAtLabels.charAt(j)!='\n';j++){}
                        	j++;
                        	if (i!=-1)breaksAtLabels.replace(i, j, "");
                        	if(this.srcFile.getLines()!=null)
                        	this.frontendListener.deleteBreakpointMarkAtSourceCode(this.srcFile.getLineNumberAtAddress(addOfLabel + offset) - 1);

						}
						this.frontendListener.displayBreakpointAtLabel(breaksAtLabels.toString());
				        this.frontendListener.displayBreakpointAtAdress(breaksAtAdds.toString());

						return null;

					}
					if (!deletebr) return ("Breakpoint set  at label: '" + label + "' + " + offset + ", address: "+ (addOfLabel+offset));
					return ("Breakpoint deleted  at label: '" + label + "' + " + offset + ", address: "+ (addOfLabel+offset));
				}
				//CASE: break <label>
				else {
					int addOfLabel = this.getAddressOfLabel(paramBreak);

					this.setBreakpointAtAddress(addOfLabel,deletebr);
					if(this.frontendListener!=null) {
						if (!deletebr) {
							if (breaksAtLabels.indexOf("Breakpoint set  at label: '"+ paramBreak + "' , address: " + (addOfLabel) + "\n")==-1)
								breaksAtLabels.append("Breakpoint set  at label: '" + paramBreak + "' , address: " + (addOfLabel) + "\n");

							if(this.srcFile.getLines()!=null)
								this.displayBreakpointAtSourcecodeline(this.srcFile.getLineNumberAtAddress(addOfLabel) - 1);
							this.displayBreakpointAtDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(addOfLabel));

						}
						else{
                        	int i=breaksAtLabels.indexOf("Breakpoint set  at label: '" + paramBreak + "' , address: " + (addOfLabel) + "\n");
                        	int j=0;
                        	for (j=i;breaksAtLabels.charAt(j)!='\n';j++){}
                        	j++;
                        	if (i!=-1)breaksAtLabels.replace(i, j, "");

                        	this.deleteBreakpointAtSourcecodeline(this.srcFile.getLineNumberAtAddress(addOfLabel) - 1);
                        	this.deleteBreakpointAtDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(addOfLabel));
						}
						if(frontendListener!=null) {
							if(this.srcFile.getLines()!=null)
								this.frontendListener.displayBreakpointAtLabel(breaksAtLabels.toString());
							this.frontendListener.displayBreakpointAtAdress(breaksAtAdds.toString());
						}
						return null;
					}
					if (!deletebr) return ("Breakpoint set at label: '" + paramBreak + "'" + ", address of that label is "+ addOfLabel);
					return ("Breakpoint deleted at label: '" + paramBreak + "'" + ", address of that label is "+ addOfLabel);
				}
			}
			else if(found) {
				try {
					this.setBreakpointAtAddress(breakAddress,deletebr);
					if(this.frontendListener!=null) {
                        if (!deletebr) {
                        	if (breaksAtAdds.indexOf("Breakpoint set at address: "+ breakAddress + "\n")==-1)
							      breaksAtAdds.append("Breakpoint set at address: "+ breakAddress + "\n");

                        	if(this.srcFile.getLines()!=null)
                        		this.displayBreakpointAtSourcecodeline(this.srcFile.getLineNumberAtAddress(breakAddress) - 1);
							this.displayBreakpointAtDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(breakAddress));
						}
                        else{
                        	int i=breaksAtAdds.indexOf("Breakpoint set at address: "+ breakAddress + "\n");
                        	int j=0;
                        	for (j=i;breaksAtAdds.charAt(j)!='\n';j++){}
                        	j++;
                        	if (i!=-1)breaksAtAdds.replace(i, j, "");
                        	if(this.srcFile.getLines()!=null)
                        		this.deleteBreakpointAtSourcecodeline(this.srcFile.getLineNumberAtAddress(breakAddress) - 1);
							this.deleteBreakpointAtDisascodeline(this.memoryI.getDisassembler().getLineOfAddress(breakAddress));
						}
                        if(frontendListener!=null) {
                        	this.frontendListener.displayBreakpointAtLabel(breaksAtLabels.toString());
				        	this.frontendListener.displayBreakpointAtAdress(breaksAtAdds.toString());
                        }
						return null;
					}
					if (!deletebr) return ("Breakpoint set at address: " + breakAddress);
					return ("Breakpoint deleted at address: " + breakAddress);
				}
				catch (Exception e) {
					throw new Exception(e.getMessage());
				}
			}
			return null;
		}

		catch (Exception e) {
			if (!deletebr)
				throw new Exception("ERROR WHILE TRYING TO SET BREAKPOINT: " + paramBreak +", " + e.getMessage());
			else
				throw new Exception("ERROR WHILE TRYING TO DELETE BREAKPOINT: " + paramBreak +", " + e.getMessage());
		}
	}

	private void setBreakpointAtAddress(int address,boolean deletebr)throws Exception{
		try {
			if (deletebr) this.processorI.deleteBreakpoint(address);
			else this.processorI.setBreakpoint(address);
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO SET BREAKPOINT AT: " + address+ ", " + e.getMessage());
		}
	}


	public int getAddressOfLabel(String label) throws Exception{
		if(!this.isObjectFileLoaded())
			throw new Exception("ERROR WHILE TRYING TO GET ADRESS OF LABEL '"+label+"'. OBJECTFILE ALREADY LOADED?");
		try {
			return this.memoryI.getAddressOfLabel(label);
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO GET ADDRESS OF LABEL: " + label);
		}
	}

	/**
	 * String representation of all registers.
	 */
	public String getRegs() throws Exception{
		String ret="";
		try {
			ret += this.processorI.getRegs() + "\n";
			ret += "ZEROFLAG: " + this.processorI.getZeroFlag() + "\n" +
					"CARRYFLAG: "+ this.processorI.getCarryFlag()+"\n";
			return ret;
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO GET CONTENT OF REGISTERS, OBJECTFILE LOADED?");
		}

	}
	/**
	 * Get current program counter.
	 */
	public int getIP() {
		return this.processorI.getIP();
	}
	/**
	 * String representation of all labels.
	 */
	public Labels getLabels() throws Exception{
		if(this.isObjectFileLoaded())
			return this.memoryI.getLabels();
		else
			throw new Exception("ERROR WHILE TRYING TO GET LABELS. OBJECTFILE NOT LOADED.");
	}
	/**
	 * String representation of all statistics.
	 */
//	public String getStats() throws Exception {
//		try {
//			return this.processorI.s;
//		}
//		catch (Exception e) {
//			throw new Exception("ERROR WHILE TRYING TO GET STATISTICS.");
//		}
//	}
	/**
	 * String representation of all processor configuration.
	 */
	public String getConfig() throws Exception {
		try {
			return this.processorI.getConfiguration();
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO GET CONFIGURATION.");
		}
	}

	public String getAboutMessage() {
		return Messages.getAboutMessage();
	}


	//############################################### SOURCEFILE
	public String readSourcefile(String PATH) throws Exception {
		try {
			this.srcFile.read(PATH);
			return this.srcFile.getLines();
		}
		catch(FileNotFoundException e ) {
			throw new FileNotFoundException("FILE "+ PATH + " NOT FOUND");
		}
		catch(Exception e1) {
			throw new Exception("ERROR WHILE TRYING TO READ IN FILE " + PATH + ", " + e1.getMessage());
		}
	}

	public String getSourceCodeLines() throws Exception{
		try {
			if(this.srcFile.getLines()==null) {
				throw new Exception("SOURCEFILE NOT READ IN YET.");
			}
			return this.srcFile.getLines();
		}
		catch(Exception e) {
			throw new Exception("SOURCEFILE NOT READ IN YET.");
		}
	}
	public String getSourceCodePATH()throws Exception {
		try {
			if(this.srcFile.getPathOfSourcefile()==null ) {
				throw new Exception("SOURCEFILE NOT READ IN YET.");
			}
			return this.srcFile.getPathOfSourcefile();
		}
		catch(Exception e) {
			throw new Exception("SOURCEFILE NOT READ IN YET.");
		}
	}
	public Sourcefile getSourcefile() {
		if(this.srcFile!=null)
			return this.srcFile;
		else
			return null;
	}

	public void writeToSourcefile(String PATH, String text)throws Exception {
		try {
			this.srcFile.write(PATH, text);
		}
		catch(FileNotFoundException e ) {
			throw new FileNotFoundException("FILE "+ PATH + " NOT FOUND");
		}
		catch(Exception e1) {
			throw new Exception("ERROR WHILE TRYING TO WRITE TO FILE " + PATH);
		}
	}

	public int getMemoryContentAt(int address)throws Exception {
		try {
			return this.memoryI.readValueAt(address);
		}
		catch(Exception e) {
			return 0; //dann ist value dort=0
		}
	}

	public void setMemory(int address, int value) throws Exception{
		try {
			this.memoryI.writeTo(address, value);
		}
		catch(Exception e) {
			throw new Exception("ERROR WHILE TRYING TO WRITE IN MEMORY AT: " + address);
		}
	}


	public ProcessorConfig getProcessorConfiguration()throws Exception {
		return this.config; //to use the config_object from GUI or CmdProcessor
	}

	/**
	 * This method sets the given Configuration Parameter.
	 * 		the parameter has to be given like: e.g. "N 4" or "use_scalar_mult 1"
	 */
	public String setConfiguration(String paramsConfig) throws Exception {
		String [] tmp = paramsConfig.split(" ");
		String var   = tmp[0]; //zB N
		String val =   tmp[1]; //zB 5
		int value=0;
		String retMessage=null;
		try {
			if(var.equals("N") || var.equals("n")) {
				value = Integer.parseInt(val);
				this.setConfig(value, config.getK(), config.isDebug_mode(),
						config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
						config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
			}
			else if(var.equals("K") || var.equals("k")) {
				value = Integer.parseInt(val);
				//hier returnString holen, wenn K mod 2 != 0, kommt einer zurueck!
				retMessage = this.setConfig(config.getN(), value, config.isDebug_mode(),
						config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
						config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
			}

			else if(var.equals("use_scalar_mult")) {
				value = Integer.parseInt(val);
				if(value == 0) { //scalar_mult = false
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							false, config.isVector_mul(), config.isShuffle_unit(),
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
				else if(value == 1) { //scalar_mult = true
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							true, config.isVector_mul(), config.isShuffle_unit(),
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
			}
			else if(var.equals("use_vector_mult")) {
				value = Integer.parseInt(val);
				if(value == 0) { //vector_mult = false
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), false, config.isShuffle_unit(),
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
				else if(value == 1) { //vector_mult = true
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), true, config.isShuffle_unit(),
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
			}
			else if(var.equals("use_shuffle")) {
				value = Integer.parseInt(val);
				if(value == 0) { //vector_shuffle = false
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), config.isVector_mul(), false,
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
				else if(value == 1) { //vector_shuffle = true
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), config.isVector_mul(), true,
							config.getShuffle_width(), config.isVectorshift_unit(), config.getVectorshift_width());
				}
			}
			else if(var.equals("shuffle_width")) {
				value = Integer.parseInt(val); //muss durch 4 teilbar sein!
				retMessage = this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
						config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
						value, config.isVectorshift_unit(), config.getVectorshift_width());
			}
			else if(var.equals("use_vectorshift")) {
				value = Integer.parseInt(val);
				if(value == 0) { //vector_shift = false
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
							config.getShuffle_width(), false, config.getVectorshift_width());
				}
				else if(value == 1) { //vector_shift = true
					this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
							config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
							config.getShuffle_width(), true, config.getVectorshift_width());
				}
			}
			else if(var.equals("shift_width")) {
				value = Integer.parseInt(val);
				this.setConfig(config.getN(), config.getK(), config.isDebug_mode(),
						config.isScalar_mul(), config.isVector_mul(), config.isShuffle_unit(),
						config.getShuffle_width(), config.isVectorshift_unit(), value);
			}
			else {
				throw new Exception("ERROR WHILE TRYING TO SETUP NEW CONFIGURATION: " + var + " " + value + ". TRY 'help config' ");
			}

			if(retMessage!=null) {
				if(this.frontendListener!=null) {
					this.frontendListener.displayConsoleError(retMessage);
					return null;
				}
				return retMessage;
			}

			else {
				if(this.frontendListener!=null) {
					this.frontendListener.displayConsoleMessage("NEW CONFIGURATION SET.");
					return null;
				}
				return "NEW CONFIGURATION SET.";
			}
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO SETUP NEW CONFIGURATION: " + var + " " + value + ". TRY 'help config' ");
		}

	}
	private String setConfig(int N, int K, boolean use_debugger, boolean use_scalar_mult,
			boolean use_vector_mult, boolean use_shuffle, int max_shuffle_width,
			boolean use_vectorshift, int vectorshift_width) throws Exception {

		String ret=null;

		this.config.setN(N);
		ret = this.config.setK(K);
		this.config.setDebug_mode(use_debugger);
		this.config.setScalar_mul(use_scalar_mult);
		this.config.setVector_mul(use_vector_mult);
		this.config.setShuffle_unit(use_shuffle);
		if(ret==null) //wenn schon Nachricht kam dass K falsch war muss shufflewidth nicht gesetzt werden
			ret = this.config.setShuffle_width(max_shuffle_width);
		this.config.setVectorshift_unit(use_vectorshift);
		this.config.setVectorshift_width(vectorshift_width);

		this.processorI.setConfig(this.config);
		setJTableVectorCpuRegisterOverviewSize(config.getN(), config.getK());

		if(ret != null)
			return ret;
		else
			return null;
	}

	public String disas() throws Exception {
		if(!this.isObjectFileLoaded())
			throw new Exception("NOTHING TO DISASSEMBLE. OBJECTFILE NOT LOADED.");
		try {
			return this.memoryI.getDisassembler().disas();
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO DISASSEMBLE, OBJECTFILE LOADED?");
		}
	}

	/**
	 * execute an external process. This method is used by HICOSIM
	 * to execute "clear" and "scotchas"
	 */
	public String execute(String cmd) throws Exception{
		try {
			String text = this.exeUtil.execute(cmd);
			return text;
		}
		catch(Exception e) {
			throw new Exception("ERROR WHILE TRYING TO EXECUTE COMMAND: "  + cmd );
		}
	}

	/**
	 * This method is used by GUI to set a flag to know
	 * in which Numerative the value of registers should be displayed.
	 *
	 * @param numerative - 'd' = decimal; 'b' = binary; 'h' = hexadecimal else throw Exception
	 */
	public void setNumerative(char numerative) throws Exception{
		int a=9; a+=9;
		switch(numerative) {
		case 'd': //DECIMAL
			this.numerativeFlag='d';
			//wenn ein neuer flag gesetzt wird ALLE updaten:
			this.checkNumerativeFlagAndDisplayAll();
			break;

		case 'h': //HEX
			this.numerativeFlag='h';
			//wenn ein neuer flag gesetzt wird ALLE updaten:
			this.checkNumerativeFlagAndDisplayAll();
			break;

		case 'b': //BINARY
			this.numerativeFlag='b';
			//wenn ein neuer flag gesetzt wird ALLE updaten:
			this.checkNumerativeFlagAndDisplayAll();
			break;
		default:
			throw new Exception("ERROR WHILE SETTING NUMERATIVE FLAG. ONLY 'd' = decimal; 'b' = binary; 'h' = hexadecimal IS POSSIBLE.");
		}
	}
	/**
	 * This method is used to display All registers in the new setted Numerative
	 */
	public void checkNumerativeFlagAndDisplayAll() {
		if(this.frontendListener!=null) {
			checkNumerativeFlagAndDisplay('A', this.processorI.getRegA() );
			checkNumerativeFlagAndDisplay('X', this.processorI.getRegX() );
			checkNumerativeFlagAndDisplay('Y', this.processorI.getRegY() );
//			checkNumerativeFlagAndDisplay('i', getIP());
			checkNumerativeFlagAndDisplay('v', 0); //0 because this method updates ALL vecRegs if flag == 'v'
		}
	}
	/**
	 * This method checks the Numerative flags and displays the registervalues on the gui
	 * depending on the flags. Registervalues are for A,X,Y, /*IP is ignored// and VectorRegs.
	 * @param regFlag - to know which reg should be updated
	 * @param value
	 */
	public void checkNumerativeFlagAndDisplay(char regFlag, int value) {
		if(this.frontendListener!=null) {
			switch(regFlag) {
			case 'A':
				if(this.numerativeFlag=='d') {
					this.displayScalarRegChanges(value, this.processorI.getRegX(), this.processorI.getRegY());
				}
				else if(this.numerativeFlag=='h') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(value), Numeratives.HEXADECIMAL, false),
					BitUtils.convert(new Integer(this.processorI.getRegX()), Numeratives.HEXADECIMAL, false), 
					BitUtils.convert(new Integer(this.processorI.getRegY()), Numeratives.HEXADECIMAL, false)); 
				}
				else if(this.numerativeFlag=='b') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(value), Numeratives.BINARY, false),
							BitUtils.convert(new Integer(this.processorI.getRegX()), Numeratives.BINARY, false), 
							BitUtils.convert(new Integer(this.processorI.getRegY()), Numeratives.BINARY, false)); 
				}
				break;
			case 'X':
				if(this.numerativeFlag=='d') {
					this.displayScalarRegChanges(this.processorI.getRegA(), value,  this.processorI.getRegY());
				}
				else if(this.numerativeFlag=='h') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(this.processorI.getRegA()), Numeratives.HEXADECIMAL, false),
					BitUtils.convert(new Integer(value), Numeratives.HEXADECIMAL, false), 
					BitUtils.convert(new Integer(this.processorI.getRegY()), Numeratives.HEXADECIMAL, false)); 
				}
				else if(this.numerativeFlag=='b') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(this.processorI.getRegA()), Numeratives.BINARY, false),
							BitUtils.convert(new Integer(value), Numeratives.BINARY, false), 
							BitUtils.convert(new Integer(this.processorI.getRegY()), Numeratives.BINARY, false)); 
				}
				break;
			case 'Y':
				if(this.numerativeFlag=='d') {
					this.displayScalarRegChanges(this.processorI.getRegA(), this.processorI.getRegX(), value);
				}
				else if(this.numerativeFlag=='h') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(this.processorI.getRegA()), Numeratives.HEXADECIMAL, false),
					BitUtils.convert(new Integer(this.processorI.getRegX()), Numeratives.HEXADECIMAL, false), 
					BitUtils.convert(new Integer(value), Numeratives.HEXADECIMAL, false)); 
				}
				else if(this.numerativeFlag=='b') {
					this.displayScalarRegChanges(BitUtils.convert(new Integer(this.processorI.getRegA()), Numeratives.BINARY, false),
							BitUtils.convert(new Integer(this.processorI.getRegX()), Numeratives.BINARY, false), 
							BitUtils.convert(new Integer(value), Numeratives.BINARY, false)); 
				}
				break;
//			case 'i':
//				if(this.numerativeFlag=='d') {
//					displayInstructionPointer(getIP());
//				}
//				else if(this.numerativeFlag=='h') {
//					displayInstructionPointer("0x"+BitUtils.convert(new Integer(getIP()), Numeratives.HEXADECIMAL, false));
//				}
//				else if(this.numerativeFlag=='b') {
//					displayInstructionPointer(BitUtils.convert(new Integer(getIP()), Numeratives.BINARY, false));
//				}
//				break;
			case 'v':
				switch(numerativeFlag) {
				case 'd':
					for(int i=0; i<this.config.getN(); i++) {
						for (int k = 0; k < config.getK(); k++) {
							if(this.processorI.getVRegsValue(i, k) != 0)
								this.displaySingleVecValue(i, k, this.processorI.getVRegsValue(i, k));		
						}
					}
					break;
				case 'h':
					for(int i=0; i<this.config.getN(); i++) {
						for (int k = 0; k < config.getK(); k++) {
							if(this.processorI.getVRegsValue(i, k) != 0)
								this.displaySingleVecValue(i, k, BitUtils.convert(
									new Integer(this.processorI.getVRegsValue(i, k)), Numeratives.HEXADECIMAL, false));		
						}
					}
					break;
				case 'b':
					for(int i=0; i<this.config.getN(); i++) {
						for (int k = 0; k < config.getK(); k++) {
							if(this.processorI.getVRegsValue(i, k) != 0)
								this.displaySingleVecValue(i, k, BitUtils.convert(
									new Integer(this.processorI.getVRegsValue(i, k)), Numeratives.BINARY, false));		
						}
					}
				}
				break;
			}
		}

	}


	/**
	 * This method is used to set the value of a register.
	 * It is possible to set statusregister, scalarRegister
	 * and vectorRegister. for example: paramSetR=="carry 1" set carry=1
	 * or paramSetR=="R12(5) 1234" set word 5 in R12 = 1234,
	 * or "A 7" : set content of reg A = 7.
	 * or "ip 12" : set ip = 12.
	 * @param paramSetR
	 * @throws Exception
	 * @return String
	 */
	public String setRegister(String paramSetR) throws Exception {
//		BitUtils bitUtil = new BitUtils();
		try {
			if(paramSetR.contains("ip")) {
				paramSetR=paramSetR.replace("ip ", "");
				if(paramSetR.contains("0x")) {
					paramSetR=paramSetR.replace("0x", "");
					this.processorI.setIP(BitUtils.hex2Decimal(paramSetR));
					return "Instruction Pointer now: " + BitUtils.hex2Decimal(paramSetR);
				}
				else {
					this.processorI.setIP(Integer.parseInt(paramSetR));
					return "Instruction Pointer now: " + paramSetR;
				}

			}
			//################################### zERO CARRY
			else if(paramSetR.contains("zero")) {
				paramSetR=paramSetR.replace("zero ", "");
				this.processorI.setStatusRegs(paramSetR.equals("1")?true:false, this.processorI.getCarryFlag());
				return paramSetR.equals("1") ? "Zeroflag set." : "Zeroflag deleted.";
			}
			else if(paramSetR.contains("carry")) {
				paramSetR=paramSetR.replace("carry ", "");
				this.processorI.setStatusRegs(this.processorI.getZeroFlag()==1?true:false, paramSetR.equals("1")?1:0 );
				return paramSetR.equals("1") ? "Carryflag set." : "Carryflag deleted.";
			}
			//################################### ScalarRegs
			else if(paramSetR.contains("A")) {
				paramSetR=paramSetR.replace("A ", "");
				if(paramSetR.contains("0x")) {
					paramSetR=paramSetR.replace("0x", "");
					this.processorI.setScalarValue('A', BitUtils.hex2Decimal(paramSetR));
					return "SET CONTENT OF REGISTER A = " + BitUtils.hex2Decimal(paramSetR);
				}
				else {
					this.processorI.setScalarValue('A', Integer.parseInt(paramSetR));
					return "SET CONTENT OF REGISTER A = " + paramSetR;
				}
			}
			else if(paramSetR.contains("X")) {
				paramSetR=paramSetR.replace("X ", "");
				if(paramSetR.contains("0x")) {
					paramSetR=paramSetR.replace("0x", "");
					this.processorI.setScalarValue('X', BitUtils.hex2Decimal(paramSetR));
					return "SET CONTENT OF REGISTER X = " + BitUtils.hex2Decimal(paramSetR);
				}
				else {
					this.processorI.setScalarValue('X', Integer.parseInt(paramSetR));
					return "SET CONTENT OF REGISTER X = " + paramSetR;
				}
			}
			else if(paramSetR.contains("Y")) {
				paramSetR=paramSetR.replace("Y ", "");
				if(paramSetR.contains("0x")) {
					paramSetR=paramSetR.replace("0x", "");
					this.processorI.setScalarValue('Y', BitUtils.hex2Decimal(paramSetR));
					return "SET CONTENT OF REGISTER Y = " + BitUtils.hex2Decimal(paramSetR);
				}
				else {
					this.processorI.setScalarValue('Y', Integer.parseInt(paramSetR));
					return "SET CONTENT OF REGISTER Y = " + paramSetR;
				}
			}
			//################################### VecRegs
			else if(paramSetR.contains("R")) { //setr "R0(i) value"  //setr R12(34) 123
				int rrrr=0, i=0, value=0;
				String Srrrr="", Si="", Svalue="";
				boolean rr=false, bi=false;

				for (int j = 0; j < paramSetR.length(); j++) {
					if(j==0) {

					}

					else if(j>0 && !rr) {
						for (int k = 0; k < 5; k++) {
							if(paramSetR.charAt(j) == '(') {
								rr=true;
								break;
							}
							else
								Srrrr += paramSetR.charAt(j++);
						}
					}
					else if(!bi){
						for (int k = 0; k < 5; k++) {
							if(paramSetR.charAt(j) == ')') {
								bi=true;
								break;
							}
							else
								Si += paramSetR.charAt(j++);
						}
					}
					else {
						try {
						while(paramSetR.charAt(j) != ' ' || paramSetR.charAt(j) != '\n') {
							Svalue += paramSetR.charAt(j++);
						}
						}catch (Exception e) {
							while(Svalue.contains(" "))
								Svalue = Svalue.replace(" ", "");
						}

					}
				}

				rrrr = Integer.parseInt(Srrrr);
				i=Integer.parseInt(Si);

				if(Svalue.contains("0x")) {
					Svalue=Svalue.replace("0x", "");
//					value = bitUtil.hex2Decimal(Svalue);
					value = BitUtils.hex2Decimal(Svalue);
					this.processorI.setVRegsValue(rrrr, i, value);
					return "SET CONTENT OF VECTORREGISTER R"+rrrr + "("+i+") = "+ value;
				}
				else {
					value = Integer.parseInt(Svalue);
					this.processorI.setVRegsValue(rrrr, i, value);
					return "SET CONTENT OF VECTORREGISTER R"+rrrr + "("+i+") = "+ value;
				}
			}
		}
		catch (Exception e) {
			throw new Exception("ERROR WHILE TRYING TO SET CONTENT OF REGISTER: " + paramSetR);
		}
		return "ERROR WHILE TRYING TO SET CONTENT OF REGISTER: " + paramSetR;
	}

	/**
	 * This class reads in a line from stdin while running and
	 * if this line equals "i" call engine.interrupt to stop cpu.
	 */
private class InterruptThread extends Thread {

	private Engine engine=null;
	private Scanner sc = null;
	private BufferedReader r;

	public InterruptThread(Engine engine) {
		this.engine = engine;
//		this.sc = new Scanner(System.in);
		this.r=new BufferedReader(new InputStreamReader(System.in));
	}
	public void stopThread() {
		try {
			super.interrupt();
			super.stop();
			sc=null;
			return;
		}
		catch (Exception e) {}
	}
	public void run() {
		while(true) {
			try {
				PrintWriter w = new PrintWriter(System.out);
				w.print('\n');
				w.flush();
				Thread.sleep(10);
				String line = this.r.readLine();
				if(line.equals("i"))  {
					this.engine.interrupt();
					super.interrupt();
					super.stop();
					super.destroy();
					return;
				}

			}
			catch (Exception e) {
				//ignore..
			}
		}
	}
}
}
