package hicosim.control;

import java.util.HashMap;
import hicosim.components.backend.io.Configurationfile;

/**
 * @author  Weber,Calchera, Wager
 *  Klasse stellt die moegliche Konfiguration des HiCoVec dar
 *
 */
public class ProcessorConfig {

	/* K fuer Woerter pro Vektorregister */
	private int K;
	/* N fuer Anzahl an Vektorregistern */
	private int N;

	/* Shuffleunit und Breite */
	private boolean shuffle_unit;
	private int shuffle_width;

	/* Vektorshift und Breite */
	private boolean vectorshift_unit;
	private int vectorshift_width;

	/* Debug-Mode einstellbar */
	private boolean debug_mode;

	/* Skalar- und Vektormultiplikationsunit */
	private boolean skalar_mul;
	private boolean vector_mul;

	/* Getter und Setter-Methoden */
	public boolean isDebug_mode() {
		return debug_mode;
	}
	public void setDebug_mode(boolean debug_mode) {
		this.debug_mode = debug_mode;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}

	}
	public int getK() {
		return K;
	}
	public String setK(int k) {
		if((k%2)==0){
			K = k;
			//AUCH SCHREIBEN:
			try {
				this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
				this.displayCPUConfiguration(false);
			}
			catch (Exception e) {
				this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
			}
			return null; //alles OK
		}
		else
			return "K not divisible by 2. Still using old value: " + getK(); //....
	}
	public int getN() {
		return N;
	}
	public void setN(int n) {
		N = n;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}
	public boolean isShuffle_unit() {
		return shuffle_unit;
	}
	public void setShuffle_unit(boolean shuffle_unit) {
		this.shuffle_unit = shuffle_unit;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}
	public int getShuffle_width() {
		return shuffle_width;
	}
	public String setShuffle_width(int shuffle_width) {
		if((shuffle_width%4)==0) {
			this.shuffle_width = shuffle_width;
			//AUCH SCHREIBEN:
			try {
				this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
				this.displayCPUConfiguration(false);
			}
			catch (Exception e) {
				this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
			}
			return null; //alles OK
		}
		else
			return "MAX_SHUFFLE_WIDTH not divisible by 4. Still using old value: " + getShuffle_width(); //....
	}
	public boolean isScalar_mul() {
		return skalar_mul;
	}
	public void setScalar_mul(boolean skalar_mul) {
		this.skalar_mul = skalar_mul;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}
	public boolean isVector_mul() {
		return vector_mul;
	}
	public void setVector_mul(boolean vector_mul) {
		this.vector_mul = vector_mul;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}
	public boolean isVectorshift_unit() {
		return vectorshift_unit;
	}
	public void setVectorshift_unit(boolean vectorshift_unit) {
		this.vectorshift_unit = vectorshift_unit;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}
	public int getVectorshift_width() {
		return vectorshift_width;
	}
	public void setVectorshift_width(int vectorshift_width) {
		this.vectorshift_width = vectorshift_width;
		//AUCH SCHREIBEN:
		try {
			this.configFile.write(N, K, debug_mode, skalar_mul, vector_mul, shuffle_unit, shuffle_width, vectorshift_unit, vectorshift_width);
			this.displayCPUConfiguration(false);
		}
		catch (Exception e) {
			this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIG.", true);
		}
	}

	public String toString() {
		return "Processor-Configuration:\n" +
		"N: " + this.N +"\t" + "K: "+this.K +"\n" +
		"use_debugger: " + this.isDebug_mode() + "\n"+
		"use_scalar_mult: " + this.isScalar_mul() + "\n" +
		"use_vector_mult: "+this.isVector_mul() + "\n" +
		"use_shuffle: " + this.isShuffle_unit() + ", max_shuffle_width: "+this.shuffle_width +"\n"+
		"use_vectorshift: " +this.vectorshift_unit +", vectorshift_width: "+this.vectorshift_width +"\n";
	}
	/**
	 * This method is either used as callback or as getter for the GUI.
	 * 
	 * @param returnConfig - if the configHashMap should be returnt or not
	 * @return HashMap<String, String> config 
	 */
	public HashMap<String, String> displayCPUConfiguration(boolean returnConfig) {
		HashMap<String, String> config = new HashMap<String, String>();

		config.put("N", ""+this.getN());
		config.put("K", ""+this.getK());

		config.put("use_debug", ""+this.isDebug_mode());

		config.put("use_scalar_mult", ""+ this.isScalar_mul());
		config.put("use_vector_mult", ""+ this.isVector_mul());

		config.put("use_shuffle", ""+this.isShuffle_unit());
		config.put("max_shuffle_width", ""+this.getShuffle_width());
		config.put("use_vectorshift", ""+this.isVectorshift_unit());
		config.put("max_vectorshift_width", ""+this.getVectorshift_width());

		if(returnConfig)
			return config;
		else {
			this.engine.displayCpuConfiguration(config);
			return null;
		}
		
	}

	public void setUp(int N, int K, boolean use_debugger,
			boolean use_scalar_mult,boolean use_vector_mult, boolean use_shuffle, int max_shuffle_width,
			boolean use_vectorshift, int vectorshift_width) { //um nach lesen am Anfang zu setzen...

		this.N = N;
		this.K = K;
		this.debug_mode = use_debugger; //IGNORE...
		this.shuffle_unit = use_shuffle;
		this.shuffle_width = max_shuffle_width;
		this.skalar_mul = use_scalar_mult;
		this.vector_mul = use_vector_mult;
		this.vectorshift_unit = use_vectorshift;
		this.vectorshift_width = vectorshift_width;

	}

	private Configurationfile configFile=null;
	private Engine engine=null;


	public ProcessorConfig(Engine engine){

		this.engine=engine;

		//kennen sich beide.....naja...
		this.configFile = new Configurationfile(this);
		/**
		 * einmalig am Anfang auslesen:
		 */
		try {
			//liest ein und macht dann hier setUp()
			this.configFile.read();
		}
		catch (Exception e) {
			//WENN READ SCHIEF GEHT STANDARD CONFIG SETZEN!
			String path = System.getProperty("user.dir") + "/config.vhd"; 
			this.configFile.setPATH(path);
			this.setUp(20, 16,  false,   true, true,   true, 32,  true, 32);
			try {
				this.configFile.write(20, 16,  false,   true, true,   true, 32,  true, 32);
				
			}
			catch (Exception ex) {
				this.engine.displayConsole("ERROR WHILE TRYING TO WRITE NEW CONFIGURATIONFILE. SETTING DEFAULT ATTRIBUTES.", true);
				this.setUp(20, 16,  false,   true, true,   true, 32,  true, 32);
				this.engine.setJTableVectorCpuRegisterOverviewSize(20, 16);
			}			
			
		}

	}

}
