package hicosim.control.utils;

/**
 * Hilfsklasse, welche Methoden fuer Bitoperationen bereitstellt.
 *
 * Class which provides methods for bitmanipulation.
 *
 * @author Calchera, Wager, Obermueller
 */

public class BitUtils {

	public static int setBit(int n, int pos) {
		return n | (1 << pos);
	}

	public static int clearBit(int n, int pos) {
		return n & ~(1 << pos);
	}

	public static int flipBit(int n, int pos) {
		return n ^ (1 << pos);
	}

	public static boolean testBit(int n, int pos) {
		int mask = 1 << pos;

		return (n & mask) == mask;
		// alternativ: return (n & 1<<pos) != 0;
	}

	/**
	 *
	 * @param instr
	 *            int wert zum testen
	 * @param start
	 *            start position
	 * @param end
	 *            letzte position
	 * @param check
	 *            art des bit zum testen 0,1
	 * @return -1==fehler,0=false,1=true beispiel:
	 *         00000000000000000000000000000001 byte==1 int int i=1;
	 *         checkBitsRange(i,31,1,0) returns 1; checkBitsRange(i,31,0,0)
	 *         returns 0; checkBitsRange(i,1,2,0) returns -1;
	 */
	public static int checkBitsRange(int instr, int start, int end, int check) {
		if (start < end)
			return -1;
		if (check == 0) {
			for (int i = start; i >= end; i--) {
				if (testBit(instr, i)) {
					return 0;
				}
			}

		} else if (check == 1) {
			for (int i = start; i >= end; i--) {
				if (!testBit(instr, i)) {
					return 0;
				}
			}
		} else {
			return -1;
		}
		return 1;
	}

	public int twosComplement(int x) {

		return (~x + 1);
	}

	/**
	 * This method converts an Integer-Object from the decimal numerative into
	 * other numeratives and returns the result as a String-Object. The
	 * enumeration "Numeratives" must be extended in order to be able to extend
	 * this method.
	 *
	 * @author Daniel Obermueller 10th April 2009
	 * @param -
	 *            Integer number
	 * @param -
	 *            Numeratives numeratives
	 * @param -
	 *            boolean grouped
	 * @return - String
	 */

	public static String convert(Integer number, Numeratives numeratives,
			boolean grouped) {
		String result = null;
		switch (numeratives) {
		case OCTAL:
			result = String.format("%o", number);
			result = "0" + result;
			break;
		case HEXADECIMAL:
			result = String.format("%X", number);
			result = "0x" + result;
			break;
		case BINARY:
			int digits = 32;
			result = Integer.toBinaryString(number);
			/* Add zeros to meet the required amount of digits */
			result = addLeadingZeros(result, digits);
			if (grouped)
				result = toGroupedString(result, digits);
			break;
		case DECIMAL:
			result = number.toString();
			break;
		}
		return result;
	}
	//for long words:
	public static String convert(Long number, Numeratives numeratives,
			boolean grouped) {
		String result = null;
		switch (numeratives) {
		case OCTAL:
			result = String.format("%o", number);
			result = "0" + result;
			break;
		case HEXADECIMAL:
			result = String.format("%X", number);
			result = "0x" + result;
			break;
		case BINARY:
			int digits = 64;
			result = Long.toBinaryString(number);
			/* Add zeros to meet the required amount of digits */
			result = addLeadingZeros(result, digits);
			if (grouped)
				result = toGroupedString(result, digits);
			break;
		case DECIMAL:
			result = number.toString();
			break;
		}
		return result;
	}

	/**
	 * This methos checks if the number N is a pot of param pot
	 *
	 * @param pot
	 * @param N
	 * @return true if N is a pot of param pot and if not then false
	 */
	public static boolean isXPot(int pot, final int N) {
		if (N % pot != 0)
			return false;
		for (int i = pot; i <= N; i *= pot) {
			if (i == N)
				return true;
		}
		return false;
	}

	/**
	 * This method adds leading zeros to the passed String-Object, which
	 * represents a number and returns the resulting number as a String-Object.
	 *
	 * @author Daniel Obermueller 10th April 2009
	 * @param String
	 *            number
	 * @return String
	 *
	 */
	private static String addLeadingZeros(String number, int digits) {
		int amountOfLeadingZeros = digits - number.length();
		for (int i = 0; i < amountOfLeadingZeros; i++)
			number = 0 + number;
		return number;
	}

	/**
	 * This method applies an eight character grouping pattern to a
	 * String-Object and returns it (e.g. "00000000111111110000000011111111" to
	 * "00000000 11111111 00000000 11111111").
	 *
	 * @author Daniel Obermueller 10th April 2009
	 * @param String
	 *            string
	 * @param int
	 *            length
	 * @return String
	 */
	private static String toGroupedString(String string, int length) {
		StringBuffer stringBuffer = new StringBuffer(string);
		int amountOfBlanks = (length / 8) - 1;
		for (int i = 1; i < amountOfBlanks + 1; i++)
			/* Insert a blank after every eighth character */
			stringBuffer.insert(i * 8 + i - 1, ' ');
		return stringBuffer.toString();
	}

	// /**
	// * @param instr
	// * @return the binary string-representation of the int value
	// */
	// public String getBinaryString(int instr) {
	// int len = Integer.toBinaryString(instr).toCharArray().length;
	// String binaryInstructionAsString="";
	// //Nullen anhaengen: (fuer Ausgbae und Vertaendnis der Befehlskodierung!)
	// for (int k=0; k<(32-len); k++) {
	// binaryInstructionAsString += "0";
	// }
	// return binaryInstructionAsString += Integer.toBinaryString(instr);
	//
	// }

	/**
	 * convert a hexadecimal string to a decimal number
	 *
	 * @param hexWord
	 *            ==> for instance: ff -> returns 255
	 *
	 * @return int: decimal value of hexadecimal string
	 */
	public static int hex2Decimal(String hexWord) {
		char[] arr = hexWord.toCharArray();
		int value = 0, exp = 0;
		for (int i = (arr.length - 1); i >= 0; i--) {
			if (arr[i] == 'f' || arr[i] == 'F') {
				value += (15 * pot(16, exp));
			} else if (arr[i] == 'e' || arr[i] == 'E') {
				value += (14 * pot(16, exp));
			} else if (arr[i] == 'd' || arr[i] == 'D') {
				value += (13 * pot(16, exp));
			} else if (arr[i] == 'c' || arr[i] == 'C') {
				value += (12 * pot(16, exp));
			} else if (arr[i] == 'b' || arr[i] == 'B') {
				value += (11 * pot(16, exp));
			} else if (arr[i] == 'a' || arr[i] == 'A') {
				value += (10 * pot(16, exp));
			} else {
				value += (Integer.parseInt("" + arr[i]) * pot(16, exp));
			}
			exp++;
		}
		return value;
	}

	/**
	 * This method converts an octal number, which is passed as a String-Object
	 * "octalNumberAsString", into a decimal number and returns it as an
	 * Integer-Object. The method works with inputs, that use the leading zero
	 * and inputs that don't use it.
	 *
	 * (E.g. octalToDecimal("18") will return 16).
	 *
	 * @author Daniel Obermueller 18th June 2009
	 * @param String
	 *            octalNumberAsString
	 * @return int
	 */
	public static int octalToDecimal(String octalNumberAsString) {
		return Integer.parseInt(octalNumberAsString, 8);
	}

	/**
	 * convert a binaryString to a decimal number
	 *
	 * @param binaryString
	 *            ==> for instance: 00000000100000001000000010000001 --> return:
	 *            8421505 ==> 10000010000000000000000000000000 --> return:
	 *            -2113929216
	 *
	 * @return int: decimal value of binaryString
	 */
	public static long binary2Decimal(String binaryString) {
		long value = 0, exp = 0;
		char charArray[] = binaryString.toCharArray();

		// ==================================UMRECHNUNG:
		for (int i = charArray.length - 1; i >= 0; i--) {
			if (charArray[i] == '1') {
				value += pot(2, exp);
				exp++;
			} else if (charArray[i] == '0') {
				exp++;
			}
		}

		return value;
	}

	public static long pot(long x, long n) {
		if (n == 0)
			return 1;
		if (n == 1)
			return x;
		else
			return (x * pot(x, n - 1));
	}
	/**
	 * this method converts four bytes to one integer.
	 * 		(see method readInt() from java.io.DataInput)
	 * @param a
	 * @param b
	 * @param c
	 * @param d
	 * @return - the integer value of the given four bytes
	 */
	public static int fourBytesToOneInteger(int a, int b, int c, int d) {
		int wert =  (((a & 0xff) << 24) | ((b & 0xff) << 16) |  ((c & 0xff) << 8) | (d & 0xff));		
		return wert;
	}
}