------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:	ASCII
--
-- PURPOSE:     Modul ASCII - VGA
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-12
-- LAST CHANGE: 
-- VERSION:     0.1
-------------------------------------------------------------------------------
-- Design of the Characters are used from the following Source:
-- Permanent Link:
-- http://de.wikipedia.org/w/index.php?title=Datei:Fonts-C64.png&oldid=59855804
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.cfg.all;

entity ascii is
  port(
	-- BUS Verbindung
    clk: std_logic;
    adr: in std_logic_vector (31 downto 0);
    data: inout std_logic_vector (31 downto 0);
    adressed: out std_logic;      
    rd, wr: in std_logic;
    ready: out std_logic;
	-- Verbindung zum VGA RAM
	do_ascii: out std_logic;
	vga_data: out std_logic_vector (31 downto 0);
	vga_adr: out std_logic_vector (31 downto 0);
	vga_wr: out std_logic;
	vga_ready: in std_logic
	);
end ascii;

architecture rtl of ascii is
-- Adressed??

-- Datentyp Zeichen (8x8 Pixel)   
	type ascii is array (0 to 7) of STD_LOGIC_VECTOR(7 downto 0);
-- Signale
	signal zeichen: ascii;
	signal ascii_code: std_logic_vector(7 downto 0);
--	signal i: integer range 0 to 8;
	signal vga_data_out: std_logic_vector(31 downto 0);
	signal data_in: std_logic_vector(31 downto 0);
	signal data_out: std_logic_vector(31 downto 0) := "11011101110111011101110111011101";	
-- States
type t_state is (		--Zustaende festlegen
		idle,
		firstline,
		firstdone,
		secondline,
		seconddone,
		thirdline,
		thirddone,
		fourthline,
		fourthdone,
		fifthline,
		fifthdone,
		sixthline,
		sixthdone,
		seventhline,
		seventhdone,
		eigthline,
		eigthdone		
		);

signal state: t_state;
	
-- ASCII-LookUpTable Funktion
-- Gibt bei eingabe von ASCII Werten das entsprechende Zeichen zurueck
-- bis jetzt nur A-Z, 0-9, punkt und strich
-- Die 1 steht fuer Vordergrund und 0 fuer Hintergrund
-- Die Zeichen sind von rechts nach links zun lesen (also Spiegelverkehrt)
	function alut(input : in std_logic_vector(7 downto 0)) return ascii is
	begin
	case (conv_integer(input)) is
		--------- - ----------
		when 45 => return(	("00000000"),
					("00000000"),
					("00000000"),
					("01111110"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"));
		--------- . ----------
		when 46 => return(	("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00011000"),
					("00011000"),
					("00000000"));	
		--------- 0 ----------
		when 48 => return(	("00111100"),
					("01100110"),
					("01110110"),
					("01101110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- 1 ----------
		when 49 => return(	("00011000"),
					("00011000"),
					("00011100"),
					("00011000"),
					("00011000"),
					("00011000"),
					("01111110"),
					("00000000"));
		--------- 2 ----------
		when 50 => return(	("00111100"),
					("01100110"),
					("01100000"),
					("00110000"),
					("00001100"),
					("00000110"),
					("01111110"),
					("00000000"));
		--------- 3 ----------
		when 51 => return(	("00111100"),
					("01100110"),
					("01100000"),
					("00111000"),
					("01100000"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- 4 ----------
		when 52 => return(	("01100000"),
					("01110000"),
					("01111000"),
					("01100110"),
					("11111110"),
					("01100000"),
					("01100000"),
					("00000000"));	
		--------- 5 ----------
		when 53 => return(	("01111110"),
					("00000110"),
					("00111110"),
					("01100000"),
					("01100000"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- 6 ----------
		when 54 => return(	("00111100"),
					("01100110"),
					("00000110"),
					("00111110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));	
		--------- 7 ----------
		when 55 => return(	("01111110"),
					("01100110"),
					("00110000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00000000"));
		--------- 8 ----------
		when 56 => return(	("00111100"),
					("01100110"),
					("01100110"),
					("00111100"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- 9 ----------
		when 57 => return(	("00111100"),
					("01100110"),
					("01100110"),
					("01111100"),
					("01100000"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- : ----------
		when 58 => return(	("00000000"),
					("00011000"),
					("00011000"),
					("00000000"),
					("00000000"),
					("00011000"),
					("00011000"),
					("00000000"));
		--------- A ----------
		when 65 => return(	("00011000"),
					("00111100"),
					("01100110"),
					("01111110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00000000"));
		--------- B ----------
		when 66 => return(	("00111110"),
					("01100110"),
					("01100110"),
					("00111110"),
					("01100110"),
					("01100110"),
					("00111110"),
					("00000000"));
		--------- C ----------
		when 67 => return(	("00111100"),
					("01100110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- D ----------
		when 68 => return(	("00011110"),
					("00110110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00110110"),
					("00011110"),
					("00000000"));
		--------- E ----------
		when 69 => return(	("01111110"),
					("00000110"),
					("00000110"),
					("00011110"),
					("00000110"),
					("00000110"),
					("01111110"),
					("00000000"));
		--------- F ----------
		when 70 => return(	("01111110"),
					("00000110"),
					("00000110"),
					("00011110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("00000000"));
		--------- G ----------
		when 71 => return(	("00111100"),
					("01100110"),
					("00000110"),
					("01110110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- H ----------
		when 72 => return(	("01100110"),
					("01100110"),
					("01100110"),
					("01111110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00000000"));
		--------- I ----------
		when 73 => return(	("00111100"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00111100"),
					("00000000"));
		--------- J ----------
		when 74 => return(	("01111000"),
					("00110000"),
					("00110000"),
					("00110000"),
					("00110000"),
					("00110110"),
					("00011100"),
					("00000000"));
		--------- K ----------
		when 75 => return(	("01100110"),
					("00110110"),
					("00011110"),
					("00001110"),
					("00011110"),
					("00110110"),
					("01100110"),
					("00000000"));
		--------- L ----------
		when 76 => return(	("00000110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("01111110"),
					("00000000"));
		--------- M ----------
		when 77 => return(	("11000110"),
					("11101110"),
					("11111110"),
					("11010110"),
					("11000110"),
					("11000110"),
					("11000110"),
					("00000000"));
		--------- N ----------
		when 78 => return(	("01100110"),
					("01101110"),
					("01111110"),
					("01111110"),
					("01110110"),
					("01100110"),
					("01100110"),
					("00000000"));
		--------- O ----------
		when 79 => return(	("00111100"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- P ----------
		when 80 => return(	("00111110"),
					("01100110"),
					("01100110"),
					("00111110"),
					("00000110"),
					("00000110"),
					("00000110"),
					("00000000"));
		--------- Q ----------
		when 81 => return(	("00111100"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("01110000"),
					("00000000"));
		--------- R ----------
		when 82 => return(	("00111110"),
					("01100110"),
					("01100110"),
					("00111110"),
					("00011110"),
					("00110110"),
					("01100110"),
					("00000000"));
		--------- S ----------
		when 83 => return(	("00111100"),
					("01100110"),
					("00000110"),
					("00111100"),
					("01100000"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- T ----------
		when 84 => return(	("01111110"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00000000"));
		--------- U ----------
		when 85 => return(	("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00000000"));
		--------- V ----------
		when 86 => return(	("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00011000"),
					("00000000"));
		--------- W ----------
		when 87 => return(	("11000110"),
					("11000110"),
					("11000110"),
					("11010110"),
					("11111110"),
					("11101110"),
					("11000110"),
					("00000000"));
		--------- X ----------
		when 88 => return(	("01100110"),
					("01100110"),
					("00111100"),
					("00011000"),
					("00111100"),
					("01100110"),
					("01100110"),
					("00000000"));
		--------- Y ----------
		when 89 => return(	("01100110"),
					("01100110"),
					("01100110"),
					("00111100"),
					("00011000"),
					("00011000"),
					("00011000"),
					("00000000"));
		--------- Z ----------
		when 90 => return(	("01111110"),
					("01100000"),
					("00110000"),
					("00011000"),
					("00001100"),
					("00000110"),
					("01111110"),
					("00000000"));
		------ Bombe ("b") --------
		when 98 => return (	("00000000"),
					("01100000"),
					("00010000"),
					("00101000"),
					("01011100"),
					("01111100"),
					("01111100"),
					("00111000"));	
		------ Kleeblatt ("k") --------
		when 107 => return (	("00111100"),
					("00111100"),
					("11011011"),
					("11111111"),
					("11111111"),
					("11001011"),
					("00001000"),
					("00000100"));	
		------ Tanne ("t") --------
		when 116 => return (	("00000000"),
					("00010000"),
					("00010000"),
					("00111000"),
					("00111000"),
					("01111100"),
					("00010000"),
					("00010000"));	
		------ others --------
		when others => return (	("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"),
					("00000000"));	
	end case;
	end function alut;
-- Umwandeln von Bits in Farbwerte (Vordergrund: Bits 11 downto 8, Hintergrund: Bits 15 downto 12)
-- Die Farbtabelle befindet sich in der VGA-Core Datei
	function vga_ram_conv(input: in std_logic_vector(7 downto 0)) return std_logic_vector is
	variable puffer: std_logic_vector(31 downto 0);
	begin
		for i in 0 to 7 loop
			if input(i) = '1' then
				puffer((i*4)+3 downto (i*4)) := data_in(11 downto 8);
			else 
				puffer((i*4)+3 downto (i*4)) := data_in(15 downto 12);
			end if;
		end loop;
	return puffer;
	end function vga_ram_conv;
-- Adressberechnung (Fuer jeden neue Zeichenzeile muessen 8 VGA Zeilen uebersprungen werden)
	function adr_calc(input: integer range -1 to 1000) return integer is
	begin
		if input > -1 and  input < 40 then
			return 0;
		elsif input > 39 and input < 80 then
			return 280;
		elsif input > 79 and input< 120 then
			return 560;
		elsif input > 119 and input < 160 then
			return 840;
		elsif input > 159 and input < 200 then
			return 1120;
		elsif input > 199 and input < 240 then
			return 1400;
		elsif input > 239 and input < 280 then
			return 1680;
		elsif input > 279 and input < 320 then
			return 1960;
		elsif input > 319 and input < 360 then
			return 2240;
		elsif input > 359 and input < 400 then
			return 2520;
		elsif input > 399 and input < 440 then
			return 2800;
		elsif input > 439 and input < 480 then
			return 3080;
		elsif input > 479 and input < 520 then
			return 3360;
		elsif input > 519 and input < 560 then
			return 3640;
		elsif input > 559 and input < 600 then
			return 3920;
		elsif input > 599 and input < 640 then
			return 4200;
		elsif input > 639 and input < 680 then
			return 4480;
		elsif input > 679 and input < 720 then
			return 4760;
		elsif input > 719 and input < 760 then
			return 5040;
		elsif input > 759 and input < 800 then
			return 5320;
		elsif input > 799 and input < 840 then
			return 5600;
		elsif input > 839 and input < 880 then
			return 5880;
		elsif input > 879 and input < 920 then
			return 6160;
		elsif input > 919 and input < 960 then
			return 6440;
		elsif input > 959 and input < 1000 then
			return 6720;
		else
			return 0;
		end if;
	end function adr_calc;
	
begin
-- MOORE Automat welcher die Daten in den VGA-RAM schreibt
	state_trans: process (clk)						-- Eingang NUR clk
	begin
		if clk'event and clk = '1' then					-- nur bei ansteigender Taktflanke
			case state is
				when idle => 
					if vga_ready = '0' and wr = '1' then
						state <= firstline;
					else
						state <= idle;
					end if;
				when firstline =>				--schreibe erste Zeile
					if vga_ready = '1' then
						state <= firstdone;
					else
						state <= firstline;
					end if;
				when firstdone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= secondline;
					else
						state <= firstdone;
					end if;
				when secondline =>				--schreibe zweite Zeile
					if vga_ready = '1' then
						state <= seconddone;
					else
						state <= secondline;
					end if;
				when seconddone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= thirdline;
					else
						state <= seconddone;
					end if;
				when thirdline =>				--schreibe dritte Zeile
					if vga_ready = '1' then
						state <= thirddone;
					else
						state <= thirdline;
					end if;
				when thirddone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <=fourthline;
					else
						state <= thirddone;
					end if;
				when fourthline =>				--schreibe vierte Zeile
					if vga_ready = '1' then
						state <= fourthdone;
					else
						state <= fourthline;
					end if;
				when fourthdone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= fifthline;
					else
						state <= fourthdone;
					end if;
				when fifthline =>				--schreibe f�nfte Zeile
					if vga_ready = '1' then
						state <= fifthdone;
					else
						state <= fifthline;
					end if;
				when fifthdone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= sixthline;
					else
						state <= fifthdone;
					end if;
				when sixthline =>				--schreibe sechste Zeile
					if vga_ready = '1' then
						state <= sixthdone;
					else
						state <= sixthline;
					end if;
				when sixthdone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= seventhline;
					else
						state <= sixthdone;
					end if;
				when seventhline =>				--schreibe siebte Zeile
					if vga_ready = '1' then
						state <= seventhdone;
					else
						state <= seventhline;
					end if;
				when seventhdone =>				--Warte auf Speicher
					if vga_ready = '0' then
						state <= eigthline;
					else
						state <= seventhdone;
					end if;
				when eigthline =>				--schreibe achte Zeile
					if vga_ready = '1' then
						state <= eigthdone;
					else
						state <= eigthline;
					end if;
				when eigthdone =>				--Warte auf Speicher
					if vga_ready = '0' and wr = '0' then
						state <= idle;
					else
						state <= eigthdone;
					end if;
				when others => null;
			end case;
		end if;
	end process;
-- Ausgangsschaltnetz
	output: process (state)							-- Eingang NUR State
	begin
		-- Standardwerte
		vga_adr <= "00000000000000000000000000000000";
		vga_data_out <= "00000000000000000000000000000000";
		vga_wr <= '0';
		do_ascii <= '0';
		ready <= '0';
		case state is
			when idle => 
				vga_wr <= '0';
			when firstline =>				--schreibe erste Zeile
				vga_data_out <= vga_ram_conv(zeichen(0));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (0 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when firstdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(0));
			   vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (0 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when secondline =>				--schreibe zweite Zeile
				vga_data_out <= vga_ram_conv(zeichen(1));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (1 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';				
			when seconddone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(1));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (1 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when thirdline =>				--schreibe dritte Zeile
				vga_data_out <= vga_ram_conv(zeichen(2));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (2 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';				
			when thirddone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(2));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (2 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when fourthline =>				--schreibe vierte Zeile
				vga_data_out <= vga_ram_conv(zeichen(3));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (3 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when fourthdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(3));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (3 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when fifthline =>				--schreibe f�nfte Zeile
				vga_data_out <= vga_ram_conv(zeichen(4));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (4 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when fifthdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(4));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (4 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when sixthline =>				--schreibe sechste Zeile
				vga_data_out <= vga_ram_conv(zeichen(5));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (5 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when sixthdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(5));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (5 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when seventhline =>				--schreibe siebte Zeile
				vga_data_out <= vga_ram_conv(zeichen(6));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (6 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when seventhdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(6));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (6 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				do_ascii <= '1';
			when eigthline =>				--schreibe achte Zeile
				vga_data_out <= vga_ram_conv(zeichen(7));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (7 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '1';
				do_ascii <= '1';
			when eigthdone =>				--Warte auf Speicher
				vga_data_out <= vga_ram_conv(zeichen(7));
				vga_adr <= conv_std_logic_vector(((((conv_integer(adr) - conv_integer(ascii_base_adr))) + (adr_calc(conv_integer(adr) - conv_integer(ascii_base_adr))) + (7 * 40)) + conv_integer(vga_base_adr)), 13);
				vga_wr <= '0';
				ready <= '1';
				do_ascii <= '1';
			when others => null;
		end case;
	end process;
	
-- GLue logic
	adressed <= '1'
	when ascii_base_adr(31 downto 10) = adr (31 downto 10)
	else '0';
	
	vga_data <= vga_data_out;
	
	zeichen <= alut(data_in(7 downto 0));
  
	data_in <= data;
	data <= data_out when rd = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
end;

