------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      BLOCKRAM
--
-- PURPOSE:     Block RAM Module
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-28
-- LAST CHANGE: 2009-01-02 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.cfg.all;

entity blockram is
	generic (width: integer := 11); --ergibt 2048*32Bit
	port (
		clk: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;      
		rd, wr: in std_logic;
		ready: out std_logic
	);
end blockram;

architecture RTL of blockram is
	constant words: integer := (2 ** width);

	type t_memory is array (0 to words-1) of STD_LOGIC_VECTOR(31 downto 0);
	signal ram: t_memory;
	attribute ram_style: string;
	attribute ram_style of ram: signal is "block";
	signal br_adr: std_logic_vector (width-1 downto 0);
	signal br_di, br_do: std_logic_vector (31 downto 0);
	signal br_en, br_we: std_logic;
begin
  -- Glue logic...
	adressed <= '1'
	when blockram_base_adr(31 downto width) = adr (31 downto width)
	else '0';

	br_adr <= adr(width-1 downto 0);
	br_di <= data;
	data <= br_do when rd = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
	br_en <= rd or wr;
	br_we <= wr;  

  -- RAM process according to Xilinx example...
	process (clk)
	begin
		if clk'event and clk = '1' then
			ready <= '0';
			if br_en = '1' then
				if br_we = '1' then
					ram(conv_integer(br_adr)) <= br_di;
				end if;
				br_do <= ram(conv_integer(br_adr));
				ready <= '1';
			end if;
		end if;
	end process;

end RTL;

