------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      DEBUG_IO
--
-- PURPOSE:     IO um Debug leitungen des HiCoVec auf Datenbus zu legen
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-11-24
-- LAST CHANGE: 
-- VERSION:     0.1
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use work.cfg.all;

entity debug_io is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		dbg_a: in std_logic_vector (31 downto 0); 
		dbg_x: in std_logic_vector (31 downto 0); 
		dbg_y: in std_logic_vector (31 downto 0); 
		dbg_ir: in std_logic_vector (31 downto 0); 
		dbg_ic: in std_logic_vector (31 downto 0);
		-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh: in std_logic_vector (3 downto 0)  
	);
end debug_io;

architecture RTL of debug_io is
begin
	adressed <= '1' 
	when debug_io_base_adr (31 downto 3) = adr (31 downto 3)
	else '0';
	data <= dbg_a
		when debug_io_base_adr (31 downto 3) & "000" = adr (31 downto 0)
	else dbg_x
		when debug_io_base_adr (31 downto 3) & "001" = adr (31 downto 0)
	else dbg_y
		when debug_io_base_adr (31 downto 3) & "010" = adr (31 downto 0)
	else dbg_ir
		when debug_io_base_adr (31 downto 3) & "011" = adr (31 downto 0)
	else dbg_ic
		when debug_io_base_adr (31 downto 3) & "100" = adr (31 downto 0)
	else "0000000000000000000000000000" & dbg_czrh (3 downto 0)
		when debug_io_base_adr (31 downto 3) & "101" = adr (31 downto 0)
	else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
-- Das heist:
-- Inhalt des Registers A liegt auf der Basiadresse
-- Inhalt des Registers X liegt auf der Basiadresse + 1
-- Inhalt des Registers Y liegt auf der Basiadresse + 2
-- Inhalt des InstrucionRegisters liegt auf der Basiadresse + 3
-- Inhalt des InstructionCounters liegt auf der Basiadresse + 4
-- Carry Signal ist das Niederwertigste Bit in Basiadresse + 5
-- Zero Signal ist das zweite Bit in Basiadresse + 5
-- IR Ready Signal ist das dritte Bit in Basiadresse + 5
-- CPU Halted Signal ist das vierte Bit in Basiadresse + 5

-- Ausgabe des Ready signals
	process (clk)
	begin
		if clk'event and clk = '1' then
			ready <= '0';
			if rd = '1' then
				ready <= '1';
			end if;
			if wr = '1' then
				ready <= '1';
			end if;
		end if;
	end process;
end RTL;

