------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      IOMISC
--
-- PURPOSE:     Standad IO des Spartan3a
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE: 2008-12-04
-- VERSION:     0.1
-----------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.all;
use work.cfg.all;

entity iomisc is
	port (
		clk: std_logic;
		-- bus interface...
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- I/O ports...
		LED : out std_logic_vector(7 downto 0);
		BTN_EAST, BTN_NORTH, BTN_SOUTH, BTN_WEST: in std_logic;
		ROT_CENTER, ROT_A, ROT_B: in std_logic;
		SW: in std_logic_vector(0 to 3);
		J18_IO: in std_logic_vector(0 to 3);
		J20_IO: in std_logic_vector(0 to 3)
	);
end iomisc;

architecture RTL of iomisc is
	signal led_state: std_logic_vector (7 downto 0) := "10011001";
	signal rot_read, ROT_L, ROT_R: std_logic;
	signal ROT_A_FF, ROT_B_FF, Z1, Z2, Z1_ENTP, ROT_KLICK, ROT_LEFT: std_logic;
	signal ROT: std_logic_vector(1 downto 0);

begin
	adressed <= '1' 
	when iomisc_base_adr (31 downto 0) = adr (31 downto 0)
	else '0';
	led <= led_state;
	data <= "000000000000" & J20_IO & J18_IO & '0' & ROT_CENTER & ROT_L & ROT_R & BTN_EAST & BTN_NORTH & BTN_SOUTH & BTN_WEST & SW 
	--data <= "000000000000" & J20_IO & J18_IO & '0' & ROT_CENTER & ROT_A & ROT_B & BTN_EAST & BTN_NORTH & BTN_SOUTH & BTN_WEST & SW 
	when rd = '1'
	else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";


	-- State Machine für Event und Richtung
	filter: process(clk)
  	begin
		if clk'event and clk='1' then

		-- Eingänge in FlipFlop laden (synchronisieren)
		      ROT_A_FF <= ROT_A;
		     	ROT_B_FF <= ROT_B;

		-- Signale Verbinden
		      ROT <= ROT_A_FF & ROT_B_FF;
		-- Ergibt 4 Zustände (Z1,Z2: 00, 01, 10, 11) welche je nach drehrichtung durchwechseln
		-- Linksrum:  00->01->11->10 
		-- Rechtsrum: 00->10->11->01
		-- Dadurch wird gewährleistet, dass Prellen eines der Signale, da man davon ausgehen kann dass nur eines
		-- gleichzeitig Prellen kann, dazu führt das kein Zustandswechsel stattfindet => entprellung
		-- Die auswertung der Richtung kann man nun in einem Zweiten Prozess (richtung) erfolgen
			case ROT is
				when "00" => 	Z1 <= '0';         
									Z2 <= Z2;
				when "01" => 	Z1 <= Z1;
									Z2 <= '0';
				when "10" => 	Z1 <= Z1;
									Z2 <= '1';
				when "11" => 	Z1 <= '1';
									Z2 <= Z2; 
				when others =>	Z1 <= Z1;
									Z2 <= Z2;
			end case;
		end if;
	end process filter; 
	
	-- Prozess zur Auswertung der Richtung
	richtung: process(clk)
	begin
		if clk'event and clk='1' then
		-- Alter Zustand von Z1 gespeichert => wechsel kann geprüft werden
			Z1_ENTP <= Z1;
      	-- Wenn ein Übergang von Z1 von 0 auf 1 erfolgt ist, so hat eine Drehung des Knopfes
		-- stattgefungden. Z2 gibt nun die drehrichtung an (Übergang von 01 -> 11 links, 00 -> 10 rechts)
		if Z1='1' and Z1_ENTP='0' then
				ROT_KLICK <= '1';	-- Knopf wurde bewegt
				ROT_LEFT <= Z2;		-- Wenn Z2 = 1 dann linksrum
       	else
				ROT_KLICK <= '0';	-- Wenn nicht Bewegt
				Z2 <= Z2;		-- dann lasses wie es ist
      	end if;
    	end if;
	end process richtung;
	
	-- Prozess zur erzeugung der Links-Rechts Signale
	auswertung: process(clk)
	begin
		if clk'event and clk='1' then
			-- Wenn Daten gelesen wurden, dann Register für ROT_L und ROT_R wieder rücksetzen...(dominant)				
			if rd = '0' and rot_read = '1' then
          		ROT_L <= '0';
					ROT_R <= '0';
		   else
				if ROT_KLICK = '1' and ROT_LEFT='1' and rot_read = '0' then 
          		ROT_L <= '1';
					ROT_R <= '0';
				elsif ROT_KLICK = '1' and ROT_LEFT='0' and rot_read = '0' then
          		ROT_L <= '0';
					ROT_R <= '1';
				end if;
			end if;
		end if;
	end process auswertung;

	datenprozess: process (clk)
	begin
		if clk'event and clk = '1' then
			ready <= '0';
			if rd = '1' then
				ready <= '1';
				-- signal wird gessetz um LR Signal so lange zu speichern bis es abgerufen wurde
				rot_read <= '1';
			--end if;
			elsif rd = '0' and rot_read = '1' then
				rot_read <= '0';
			end if;	
			if wr = '1' then
				led_state <= data(7 downto 0);
				ready <= '1';
			end if;
		end if;
	end process datenprozess;
end RTL;

