------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:	VGA
--
-- PURPOSE:     VGA Top Module
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-28
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.cfg.all;

entity vga is
	port (
		clk: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;
		rd, wr: in std_logic;
		ready: out std_logic;
		-- VGA_IO
		CLK_50M: in std_logic;
		VGA_R: out std_logic_vector (3 downto 0);
		VGA_G: out std_logic_vector (3 downto 0);
		VGA_B: out std_logic_vector (3 downto 0);
		VGA_HSYNC: out std_logic;
		VGA_VSYNC: out std_logic;
		--Eingaenge von ASCII
		do_ascii: in std_logic;
		ascii_data: in std_logic_vector (31 downto 0);
		ascii_adr_in: in std_logic_vector (31 downto 0);
		ascii_wr: in std_logic;
		ascii_ready: out std_logic	
	);
end vga;

architecture RTL of vga is

-- VGA_RAM einbinden
component vga_ram is
	port (
		clk: std_logic; -- normal
		CLK_50M: std_logic;
		adr: in std_logic_vector (31 downto 0);
		data: inout std_logic_vector (31 downto 0);
		adressed: out std_logic;      
		rd, wr: in std_logic;
		ready: out std_logic;
		vga_out: out std_logic_vector (31 downto 0);
		vga_adr: in std_logic_vector (31 downto 0);
		ascii_wr_in: in std_logic;
		ascii_data_in: in std_logic_vector (31 downto 0)
	);
end component;

-- VGA_CORE einbinden
component vga_core is
	port(
		CLK_50M: in std_logic;
		VGA_R: out std_logic_vector (3 downto 0);
		VGA_G: out std_logic_vector (3 downto 0);
		VGA_B: out std_logic_vector (3 downto 0);
		VGA_HSYNC: out std_logic;
		VGA_VSYNC: out std_logic;
		-- Verbindung zum RAM
		adr: out std_logic_vector (31 downto 0);
		data: in std_logic_vector (31 downto 0)
	);
end component;

-- ASCII Multiplexer einbinden
component ascii_mul is
	port(
	-- ASCII Eingang
		do_ascii: in std_logic;
		ascii_adr_in: in std_logic_vector (31 downto 0);
		ascii_wr: in std_logic;
		ascii_ready: out std_logic;
		-- Datebbus Eingang
		adr: in std_logic_vector (31 downto 0);
		rd: in std_logic;
		wr: in std_logic;
		ready: out std_logic;
		-- VGARAM Ausgang
		vga_adr: out std_logic_vector (31 downto 0);
		vga_rd: out std_logic;
		vga_wr: out std_logic;
		vga_ready: in std_logic	
	);
end component;

for all: vga_ram use entity WORK.vga_ram(syn);
for all: vga_core use entity WORK.vga_core(RTL);
for all: ascii_mul use entity WORK.ascii_mul(RTL);

signal s_clk_50m: std_logic;
signal s_clk: std_logic;
signal s_vga_ram_adr: std_logic_vector (31 downto 0);
signal s_vga_ram_data: std_logic_vector (31 downto 0);
signal s_vga_adr: std_logic_vector (31 downto 0);
signal s_vga_data: std_logic_vector (31 downto 0);
signal s_vga_rd: std_logic;
signal s_vga_wr: std_logic;
signal s_vga_ready: std_logic;
signal s_ascii_wr: std_logic;

begin
vga_ram_impl : vga_ram
	port map(
		clk		=> s_clk,
		CLK_50M		=> s_CLK_50M,
		adr		=> s_vga_adr,
		data		=> data,
		adressed 	=> adressed,
		rd		=> s_vga_rd,
		wr		=> s_vga_wr,
		ready		=> s_vga_ready,
		vga_out		=> s_vga_ram_data,
		vga_adr		=> s_vga_ram_adr,
		ascii_wr_in	=> s_ascii_wr,
		ascii_data_in	=> ascii_data
	);
  
vga_core_impl : vga_core
	port map(
		CLK_50M		=> s_CLK_50M,
		VGA_R		=> VGA_R,
		VGA_G 		=> VGA_G,
		VGA_B		=> VGA_B,
		VGA_HSYNC	=> VGA_HSYNC,
		VGA_VSYNC	=> VGA_VSYNC,
		-- Verbindung zum RAM
		adr		=> s_vga_ram_adr,
		data		=> s_vga_ram_data
	);
	
ascii_mul_impl :ascii_mul
	port map(
		-- ASCII Eingang
		do_ascii	=> do_ascii,
		ascii_adr_in	=> ascii_adr_in,
		ascii_wr	=> s_ascii_wr,
		ascii_ready	=> ascii_ready,
		-- Datebbus Eingang
		adr		=> adr,
		rd		=> rd,
		wr		=> wr,
		ready		=> ready,
	-- VGARAM Ausgang
		vga_adr		=> s_vga_adr,
		vga_rd		=> s_vga_rd,
		vga_wr		=> s_vga_wr,
		vga_ready	=> s_vga_ready
	);

  -- Glue logic...
s_clk <= clk;
s_clk_50m <= CLK_50M;
s_ascii_wr <= ascii_wr;

end RTL;

