------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:	VGA_CORE
--
-- PURPOSE:	VGA Timing Module
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-19
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity vga_core is
	port(
		CLK_50M: in std_logic;
		VGA_R: out std_logic_vector (3 downto 0);
		VGA_G: out std_logic_vector (3 downto 0);
		VGA_B: out std_logic_vector (3 downto 0);
		VGA_HSYNC: out std_logic;
		VGA_VSYNC: out std_logic;
		-- Verbindung zum RAM
		adr: out std_logic_vector (31 downto 0);
		data: in std_logic_vector (31 downto 0)
	);
end vga_core;

architecture rtl of vga_core is

--signal clk25: std_logic;
signal clockdividor: std_logic;
signal hcounter: integer range 0 to 800;
signal vcounter: integer range 0 to 521;
signal linedoppler: integer range 0 to 1;
signal pixcounter: integer range 0 to 39;
signal linecounter: integer range 0 to 239;
signal wordcounter: integer range 0 to 15;

signal RGB: std_logic_vector(11 downto 0);
signal pixel: std_logic_vector(3 downto 0);

-- ColorLookUpTable Funktion
function clut(input : in std_logic_vector(3 downto 0)) return std_logic_vector is
begin
	case (conv_integer(input)) is
				--"RRRR"&"GGGG"&"BBBB"
		when 15 => return "0000"&"0000"&"0000";	--schwarz
		when 14 => return "0100"&"0000"&"0000";	--dunkelrot
		when 13 => return "0000"&"0100"&"0000";	--dunkelgruen 
		when 12 => return "0100"&"0100"&"0000";	--dunkelgelb
		when 11 => return "0000"&"0000"&"0100";	--dunkelblau
		when 10 => return "0100"&"0000"&"0100";	--dunkelmagenta
		when  9 => return "0000"&"0100"&"0100";	--dunkelcyan
		when  8 => return "0011"&"0011"&"0011";	--dunkelgrau
		when  7 => return "0111"&"0111"&"0111";	--hellgrau
		when  6 => return "1111"&"0000"&"0000";	--rot
		when  5 => return "0000"&"1111"&"0000";	--gruen
		when  4 => return "1111"&"1111"&"0000";	--gelb
		when  3 => return "0000"&"0000"&"1111";	--blau
		when  2 => return "1111"&"0000"&"1111";	--magenta
		when  1 => return "0000"&"1111"&"1111";	--cyan
		when  0 => return "1111"&"1111"&"1111";	--wei�
		when others => return "000000000000";
	end case;
end function clut;
begin
--generate a 25Mhz clock
process (CLK_50M)
begin
  if CLK_50M'event and CLK_50M='1' then
		clockdividor <= not clockdividor;	--Macht aus den 50MHZ Takt die 25MHZ Pixeltakt
  end if;
end process;

process (CLK_50M)
	variable x: integer range 0 to 639;
	variable y: integer range 0 to 479;
begin
	-- hcounter counts from 0 to 799
	-- vcounter counts from 0 to 520
	-- x coordinate: 0 - 639 (x = hcounter - 144, i.e., hcounter -Tpw-Tbp)
	-- y coordinate: 0 - 479 (y = vcounter - 31, i.e., vcounter-Tpw-Tbp)
	x := hcounter - 144;
	y := vcounter - 71; --Bei y = 480 statt 400 muss dieser Wert auf 31 gestellt werden
	if CLK_50M'event and CLK_50M = '1' then
		if clockdividor = '1' then
			if x < 640 and y < 400 then
				adr <= conv_std_logic_vector(pixcounter+(40*linecounter), 13); -- Adressberechnung
			end if;
			-- if x < 640 and x > 0 and y < 400 then --Hier wuerde die erste Zeile ausgeblendet...
			if x < 640 and y < 400 then
				RGB <= clut(pixel);
			else
				-- if not traced, set it to "black" color
				RGB <= "000000000000";
			end if;
			-- Pixelberechnung
			case wordcounter is
				when 0 =>
					pixel <= data (31 downto 28);
				when 15 =>
					pixel <= data (31 downto 28);
				when 14 =>
					pixel <= data (27 downto 24);
				when 13 =>
					pixel <= data (27 downto 24);
				when 12 =>
					pixel <= data (23 downto 20);
				when 11 =>
					pixel <= data (23 downto 20);
				when 10 =>
					pixel <= data (19 downto 16);
				when 9 =>
					pixel <= data (19 downto 16);
				when 8 =>
					pixel <= data (15 downto 12);
				when 7 =>
					pixel <= data (15 downto 12);
				when 6 =>
					pixel <= data (11 downto 8);
				when 5 =>
					pixel <= data (11 downto 8);
				when 4 =>
					pixel <= data (7 downto 4);
				when 3 =>
					pixel <= data (7 downto 4);
				when 2 =>
					pixel <= data (3 downto 0);
				when 1 =>
					pixel <= data (3 downto 0);	
				when others =>                              -- error
					pixel <= "0000";
			end case;

			if x < 640 and y < 400 then
				wordcounter <= wordcounter + 1;
				if wordcounter = 15 then
					pixcounter <= pixcounter + 1;
					wordcounter <= 0;
				end if;
			end if;

			if x > 639 then
				wordcounter <= 0;
				pixcounter <= 0;
			end if;
			if x = 640 then
				linedoppler <= linedoppler + 1;
			end if;
			if x = 640 and linedoppler = 1 then
				linedoppler <= 0;
				linecounter <= linecounter + 1;
			end if;
			if x > 639 and y > 479 then
				linecounter <= 0;
				linedoppler <= 0;
			end if;
				
			-- Here is the timing for horizontal synchronization.
			-- (Refer to p. 24, Xilinx, Spartan-3 Starter Kit Board User Guide)
			-- Pulse width: Tpw = 96 cycles @ 25 MHz
			-- Back porch: Tbp = 48 cycles
			-- Display time: Tdisp = 640 cycles
			-- Front porch: Tfp = 16 cycles
			-- Sync pulse time (total cycles) Ts = 800 cycles

			if hcounter > 0 and hcounter < 97 then
				VGA_HSYNC <= '0';
			else
				VGA_HSYNC <= '1';
			end if;
			-- Here is the timing for vertical synchronization.
			-- (Refer to p. 24, Xilinx, Spartan-3 Starter Kit Board User Guide)
			-- Pulse width: Tpw = 1600 cycles (2 lines) @ 25 MHz
			-- Back porch: Tbp = 23200 cycles (29 lines)
			-- Display time: Tdisp = 38400 cycles (480 lines)
			-- Front porch: Tfp = 8000 cycles (10 lines)
			-- Sync pulse time (total cycles) Ts = 416800 cycles (521 lines)
			if vcounter > 0 and vcounter < 3 then
				VGA_VSYNC <= '0';
			else
				VGA_VSYNC <= '1';
			end if;
			-- horizontal counts from 0 to 799
			hcounter <= hcounter+1;
			if hcounter = 800 then
				vcounter <= vcounter+1;
				hcounter <= 0;
			end if;
			-- vertical counts from 0 to 519
			if vcounter = 521 then
				vcounter <= 0;
				linedoppler <= 0;
				pixcounter <= 0;
				linecounter <= 0;
				wordcounter <= 0;
			end if;
		end if;
	end if;
end process;
--Glue logic
VGA_R <= RGB(11 downto 8);
VGA_G <= RGB(7 downto 4);
VGA_B <= RGB(3 downto 0);
end;

