------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      DBGSHELL
--
-- PURPOSE:     Debug Shell from VISCY for HiCoVec
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- Prof. Gundolf Kiefer
-- Hochschule Augsburg
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Mueller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Ruehm
--
-- 
-- DATE CREATE: 2008-12-28
-- LAST CHANGE: 2010-02-24 FROM: Andreas Mueller
-- VERSION:     0.1
-----------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity dbgshell is
	port( 
		clk,                        -- shell clock
		se, sdi: in std_logic;      -- scan enable, shift data in
		sdo: out std_logic;         -- shift data out
		-- CPU signals...
		cpu_reset: out std_logic;
		cpu_rd, cpu_wr: in std_logic;
		cpu_ready: out std_logic;	
		cpu_adr: in std_logic_vector (31 downto 0);
		cpu_data: inout std_logic_vector (31 downto 0);
		-- BUS signals...
		bus_rd, bus_wr: out std_logic;
		bus_ready: in std_logic;	
		bus_adr: out std_logic_vector (31 downto 0);
		bus_data: inout std_logic_vector (31 downto 0)
	);
end dbgshell;

architecture RTL of dbgshell is

signal capture_cpu, capture_io: std_logic;
signal r_reset, r_rd, r_wr, r_ready: STD_LOGIC;
signal r_adr, r_data: STD_LOGIC_VECTOR(31 downto 0);
signal bus_wr_int: std_logic;
--signal delayed_bus_ready, delayed_cpu_rd, delayed_cpu_wr: std_logic;
--signal delayed_bus_ready: std_logic;

begin
	-- Scan chain...
	sdo <= capture_cpu;

	shell: process (clk)
	begin
		if clk'event and clk = '1' then
			if se = '1' then
				-- Shift mode...
				capture_cpu <= capture_io;
				capture_io <= r_reset;
				r_reset <= r_rd;
				r_rd <= r_wr;
				r_wr <= r_ready;
				r_ready <= r_adr(31);
				r_adr <= r_adr(30 downto 0) & r_data(31);
				r_data <= r_data(30 downto 0) & sdi;
			else
				-- System mode...
				if capture_cpu = '1' then
					r_rd <= cpu_rd;
					r_wr <= cpu_wr;
					r_adr <= cpu_adr;
					if cpu_wr = '1' then r_data <= cpu_data; end if;
				end if;
				if capture_io = '1' then
					r_ready <= bus_ready;
					if cpu_wr = '0' and r_wr = '0' then r_data <= bus_data; end if;
				end if;
				-- delayed input of 'rw', 'wr' and 'ready' signals to compensate for clock skewing...
				--delayed_bus_ready <= bus_ready;
				--delayed_cpu_rd <= cpu_rd;
				--delayed_cpu_wr <= cpu_wr;
				-- delayed output of 'rw', 'wr' and 'ready' signals to compensate for clock skewing...
				cpu_ready <= r_ready;
				bus_rd <= r_rd;
				bus_wr_int <= r_wr;
			end if;
		end if;
	end process;

	-- CPU connection...
	cpu_reset <= r_reset;
	cpu_data <= r_data when cpu_rd = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";

	-- Bus connection...
	bus_wr <= bus_wr_int;
	bus_adr <= r_adr;
	bus_data <= r_data when r_wr = '1' or bus_wr_int = '1' else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";

end RTL;
