------------------------------------------------------------------
-- PROJECT:     Der Hicovec Prozessor in Aktion
--
-- ENTITY:      SCOTCH System TB
--
-- PURPOSE:     Testbench von SCOTCH
-- 
-- System to
-- Connect,
-- Operate ,
-- Transfer data and
-- Compute with the
-- Hicovec 
--
-- TEAM:
--
-- Jakob Golus
-- David Lucinkiewicz
-- Andreas Müller
-- Florian Richter	
-- Sebastian Rigorth
-- Soeren Rühm
--
-- 
-- DATE: 2008-12-11
-- VERSION:     0.1
-----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.cfg.all;

entity scotch_tb is
end scotch_tb;

architecture behavior of cpu_tb is
-- CPU implementieren
	component scotch_system is
	port(
	-- CLK und RS232
--		clk_50m : in  std_logic;
--		RS232_rx : in  std_logic;
--		RS232_tx : out std_logic;
	--I/O Schnittstelle
		IO_addressed : in std_logic_vector (0 to slaves-1);
		IO_ready : in std_logic_vector (0 to slaves-1);
		IO_rd : out std_logic_vector (0 to slaves-1);
		IO_wr : out std_logic_vector (0 to slaves-1);
		IO_data : inout std_logic_vector (31 downto 0);
		IO_address : out std_logic_vector (31 downto 0);
		IO_clk : out std_logic;
	--Prozessor debug Leitungen
		dbg_a : out std_logic_vector (31 downto 0); 
		dbg_x : out std_logic_vector (31 downto 0); 
		dbg_y : out std_logic_vector (31 downto 0); 
		dbg_ir : out std_logic_vector (31 downto 0); 
		dbg_ic : out std_logic_vector (31 downto 0);
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh : out std_logic_vector (3 downto 0);  
        -- Internal Chip Debugging Connectors
		clk_hicovec_in : in  std_logic;
		clk_io_in : in  std_logic;
		clk_shell_in : in  std_logic;
		se_shell : in  std_logic;
		sdi_shell : in  std_logic;
		sdo_shell : out  std_logic

  );
	end component;
	
	or all: scotch_system use entity WORK.scotch_system(RTL);
-- Signale
	signal IO_addressed : std_logic_vector (0 to slaves-1) := ( 0 => '1', others => '0'); 
	signal IO_ready : std_logic_vector (0 to slaves-1);
	signal IO_rd : std_logic_vector (0 to slaves-1);
	signal IO_wr : std_logic_vector (0 to slaves-1);
	signal IO_data : std_logic_vector (31 downto 0);
	signal IO_address : std_logic_vector (31 downto 0);
	signal IO_clk : std_logic;
	--Prozessor debug Leitungen
	signal dbg_a : std_logic_vector (31 downto 0); 
	signal dbg_x : std_logic_vector (31 downto 0); 
	signal dbg_y : std_logic_vector (31 downto 0); 
	signal dbg_ir : std_logic_vector (31 downto 0); 
	signal dbg_ic : std_logic_vector (31 downto 0);
	-- 0: carry 1: zero 2: ir-ready 3: halted
	signal dbg_czrh : std_logic_vector (3 downto 0);  
        -- Internal Chip Debugging Connectors
	signal clk : std_logic := '0';
	signal se_shell : std_logic  := '0';
	signal sdi_shell : std_logic := '0';
	signal sdo_shell : std_logic := '0';

-- Parameter
	constant clk_period: 	time := 20 ns;
	constant mem_delay: 	time := 40 ns;
-- Speicherinhalt
	type t_memory is array (0 to 263) of std_logic_vector(31 downto 0);
	signal s_memory: t_memory := (
      16#0000# => "10000001000000000000000000010001",
	  16#0001# => "00000000000000000000000000000000",
	  16#0002# => "00000000000000000000000000000000",
      16#0003# => "00000000000000000000000000000000",
      16#0011# => "00000000000000000000000000001111",	 
      16#0051# => "00000000000000000000000000000000",		  
      others => "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU"
	);

begin
-- Instantiate the Unit Under Test (UUT)
	uut: scotch_system 
		port map (
		IO_addressed => IO_addressed,
		IO_ready => IO_ready,
		IO_rd => IO_rd,
		IO_wr => IO_wr,
		IO_data => IO_data,
		IO_address => IO_address,
		IO_clk => IO_clk,
	--Prozessor debug Leitungen
		dbg_a => dbg_a,
		dbg_x => dbg_x,
		dbg_y => dbg_y,
		dbg_ir => dbg_ir,
		dbg_ic => dbg_ic,
	-- 0: carry 1: zero 2: ir-ready 3: halted
		dbg_czrh => dbg_czrh,
        -- Internal Chip Debugging Connectors
		clk_hicovec_in => clk,
		clk_io_in => clk,
		clk_shell_in => clk,
		se_shell => se_shell,
		sdi_shell => sdi_shell,
		sdo_shell => sdo_shell
		);
-- Prozess um den Memory zu simulieren
	memory: process
	begin
		data <= "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
		ready <= '0';
		wait on IO_rd(0), IO_wr(0);
		if IO_rd = '1' then
			wait for mem_delay;
			IO_data <= s_memory (conv_integer ('0' & IO_address));
			IO_ready(0) <= '1';
			wait until IO_rd(0) = '0';
			wait for mem_delay;
			data <= "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
			IO_ready(0) <= '0';
		elsif IO_wr(0) = '1' then
			wait for mem_delay;
			s_memory (conv_integer ('0' & IO_address)) <= IO_data;
			IO_ready(0) <= '1';
			wait until IO_wr(0) = '0';
			wait for mem_delay;
			IO_ready(0) <= '0';
		end if;
	end process;
-- Takt simuliern
	tb: process
	procedure run_cycle is
	begin
		clk <= '0';
		wait for clk_period / 2;
		clk <= '1';
		wait for clk_period / 2;
	end procedure;
-- Testbench Programm
	variable i: integer;	-- Variable für Zähler
	
	begin
-- RESET
		run_cycle;
		reset <= '1';
		run_cycle;
		reset <= '0';

	-- Programm ablaufen lassen
		i := 0;
		while i < 2000 loop
			run_cycle;
			i := i + 1;
		end loop;
	wait;     -- wait forever (stop simulation)
	end process;
end;
