/*! createFile.c
 *  In this file are the functions which create the output file
 *
 *  @author Eggenmueller Bernd, Co author Schmidt Juergen
 *	$Revision: 1.0 $
 *	@date 25.04.2007
 */

#include <string.h>
#include <stdlib.h>
#include "functions.h"
#include "commit.h"
#include "errors_and_warnings.h"

/*! This function writes the sections for the output file into an Array and returns the number of sections
 *
 *	@param int** sectionArray array where the sections will be saved
 *	@param MERKERLIST* firstMerker pointer to the first Merker in the Merkerlist
 *	@param int lineInd how much lines the file includes
 *	@param COMMENTLIST* commentl pointer to the first Comment in the Commentlist
 *	@param char* info string of the Info-Section
 *	@param CODELIST* firstCode pointer to the first Code in the Codelist
 *	@param int start line in which start is declered
 *	@param int inhalt shows what should into tho object file
 *
 *	@return int number of the Sections
 */ 
int getSections(int **sectionArray, MERKERLIST *firstMerker,int lineInd,COMMENTLIST *commentl,char *info,CODELIST *firstCode, int start,int inhalt)
{
	int sectionAnz=0;	//Anzahl der Sections um genügend Speicher zu reservieren
	int reg=0;		//length of register section, 1 for register section exists
	int lab=0;	 	//length of the Label section as n*16 Bit, if 0 no such section exists
	int line=0;		//length of line section as n*16 Bit, if 0 no such section exists
	int code=0;		//length of all code sections if 0 no such section exists
	int comment=0;		//length of comment section, if 0 no such section exists
	int infoLen=0;		//length of info section, if 0 no such section exists
	MERKERLIST *helpM=firstMerker;
	CODELIST *helpC=firstCode;
	COMMENTLIST *helpCom=commentl;
	int *helpArray=NULL;
	int i=0;
	//length of label section
	if((inhalt&0x1)==0x1)
	{
		for(;helpM!=NULL;helpM=helpM->next)
		{	
			lab+=3; 			// Labellänge u. Speicheradresse, vorher: lab+=2  ->
			lab+=(strlen(helpM->name)+1)/2; // jeder Buchstabe braucht 8 Bit		 -> wird x mal eingetragen
		}
		if(lab!=0)
		{
			lab++;				 // Labelanzahl, wird einmal eingetragen
			sectionAnz++;
		}
	}
	//length of register section
	if(start>0)
	{
		sectionAnz++;
		reg=8;       // vorher: reg= 9,  32bit * A, X, Y u. 32Bit mal PC somit 8 * 16 bit
	}
	//length of line section
	if((inhalt&0x2)==0x2)
	{
		if(lineInd!=0)
		{
			line=line+(3*lineInd); // vorher: (2*lineInd)
			sectionAnz++;
		}
	}
	//length of code section
	for(;helpC!=NULL;helpC=helpC->next)	
	{
		sectionAnz++;
		code=code+(helpC->length)*2+2; // vorher: +1 , da Startadresse 32 bit, vorher:ohne *2
	}

	//length of comment section
	if((inhalt&0x4)==0x4)
	{
		for(;helpCom!=NULL;helpCom=helpCom->next)
		{
			comment=comment+2+((strlen(helpCom->comment)+1)/2);		
		}
		if(comment!=0)
		{
			comment++;
			sectionAnz++;
		}
	}
	//length of info section
	if((inhalt&0x8)==0x8)
	{
		if(info!=NULL)
		{
			sectionAnz++;
			infoLen=(strlen(info)+1)/2;
		}
	}

	helpArray=(int*)malloc(2*sizeof(int)*sectionAnz);
	(*sectionArray)=helpArray;
	if(lab!=0)
	{
		helpArray[i]=0x12;
		i++;
		helpArray[i]=lab;
		i++;
	}
	if(line!=0)
	{
		helpArray[i]=0x13;
		i++;
		helpArray[i]=line;
		i++;
	}
	if(comment!=0)
	{
		helpArray[i]=0x5;
		i++;
		helpArray[i]=comment;
		i++;
	}
	if(infoLen!=0)
	{
		helpArray[i]=0x6;
		i++;
		helpArray[i]=infoLen;
		i++;
	}
	if(reg!=0)
	{
		helpArray[i]=0x17;
		i++;
		helpArray[i]=reg;
		i++;
	}
	helpC=firstCode;
	for(;i/2<sectionAnz;)
	{
		helpArray[i]=0x11;
		i++;
		helpArray[i]=helpC->length*2+2;
		i++;
		helpC=helpC->next;
	}
	return sectionAnz;
}
/*
Sections wurden erweitert, alle die 32 Bit Adressen enthalten werden mit 0x1a
angesprochen wobei
a= 1 Code
   2 Labels
   3 Line
   7 Register

*/
  

/*! This function writes the different Sections into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int* secArray array where the sections are saved
*	@param int secAnz number of the secdtions
 *	@param MERKERLIST* firstMerker pointer to the first Merker in the Merkerlist
 *	@param int lineInd how much lines the file includes
 *	@param short int* lineTab table of the lines and memory adresses which belongs together
 *	@param COMMENTLIST* commentl pointer to the first Comment in the Commentlist
 *	@param char* info string of the Info-Section
 *	@param CODELIST* firstCode pointer to the first Code in the Codelist
 *	@param short int* memory array where the code is saved
 *	@param int start line in which start is declered
 *
 *	@return void
 */ 
void createSections(FILE **fp,int *secArray,int secAnz,MERKERLIST *firstMerker,int lineInd,short int *lineTab,COMMENTLIST *commentl,char *info,CODELIST *firstCode,unsigned int *memory, int start)
{
	CODELIST *helpC=firstCode;
	int i;	
	for(i=0;i<secAnz*2;i=i+2)
	{	switch(secArray[i])
		{	case 0x11:	writeCodeSection(fp,firstCode,helpC,memory);
					helpC=helpC->next;	
					break;
			case 0x12:	writeLabelSection(fp,firstMerker);	
					break;
			case 0x13:	writeLineSection(fp,lineInd,lineTab);	
					break;
			case 0x5:	writeCommentSection(fp,commentl);	
					break;
			case 0x6:	writeInfoSection(fp,info);	
					break;
			case 0x17:	writeRegisterSection(fp,start,lineTab);
					break;
			default:	Err_NoSuchSec(i);
		}
	}	
}

/*! This function writes the different Sections into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int* wCount distance to the beginn of the file in words (16Bit)
 *	@param int* secArray array where the sections are saved
 *	@param int secAnz number of the secdtions
 *
 *	@return void
 *
 * Reihenfolge der Abarbeitung beachten, erst wird 0x2 gelesen, denn die Labels sind das erste was nach dem Sektionenverzeichnis kommt.
 * secAnz * 5 + 1 --> Größe des Sektionenverzeichnisses (5 -> art, A_adresse h, A_adresse l, E_adresse h, E_adresse l) 
*/
void createSectionTab(FILE **fp,int *wCount,int *secArray, int secAnz)
{
	int i,help;

	(*wCount)=(*wCount)+secAnz*5+1;
	fwriteIntBigEndian(secAnz,fp);
		
	for(i=0;i<secAnz*2;i=i+2)
	{	switch(secArray[i])
		{	case 0x11:	fwriteIntBigEndian(0x11,fp);			
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			case 0x12:	fwriteIntBigEndian(0x12,fp);
					help=(*wCount)>>16;                //
					fwriteIntBigEndian(help,fp);       // H u. L Byte tauschen
					fwriteIntBigEndian((*wCount),fp);  //
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			case 0x13:	fwriteIntBigEndian(0x13,fp);
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			case 0x5:	fwriteIntBigEndian(5,fp);
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			case 0x6:	fwriteIntBigEndian(6,fp);
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			case 0x17:	fwriteIntBigEndian(0x17,fp);
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)=(*wCount)+secArray[i+1]-1;
					help=(*wCount)>>16;
					fwriteIntBigEndian(help,fp);
					fwriteIntBigEndian((*wCount),fp);
					(*wCount)++;
					break;
			default:	Err_NoSuchSec(secArray[i]);
		}
	}	
}

/*! This function writes the register-section into the file
 *
 *	@param FILE** fp pointer to the output file
 *	@param MERKERLIST* firstMerker pointer to the first Merker in the Merkerlist
 *
 *	@return void
 */
void writeRegisterSection(FILE **fp, int start, short int *lineTab)
{	
	int regAnz=3;
	int i,help;
        help=(lineTab[start-1])>>16;
	fwriteIntBigEndian(help,fp);
	fwriteIntBigEndian(lineTab[start-1],fp);
	
	for(i=0;i<regAnz;i++)
	 {
	 fwriteIntBigEndian(0,fp);
	 fwriteIntBigEndian(0,fp);
         }
}

/*! This function writes the info-section into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int start line in which start is declered
 *	@param short int* lineTab table of the lines and memory adresses which belongs together
 *
 *	@return void
 */
void writeInfoSection(FILE **fp,char *info)
{
	writeString(fp,info);
}

/*! This function writes the comment-section into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param COMMENTLIST* commentl pointer to the first Comment in the Commentlist
 *	
 *	@return void
 */
void writeCommentSection(FILE **fp,COMMENTLIST *commentl)
{
	int anzComment=0;
	COMMENTLIST *helpC;
	for(anzComment=0,helpC=commentl;helpC!=NULL;helpC=helpC->next,anzComment++);
	fwriteIntBigEndian(anzComment,fp);
	for(helpC=commentl;helpC!=NULL;helpC=helpC->next)
	{
		fwriteIntBigEndian(strlen(helpC->comment),fp);
		fwriteIntBigEndian(helpC->line,fp);
		writeString(fp,helpC->comment);
	}
}

/*! This function writes the label-section into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param MERKERLIST* firstMerker pointer to the first Merker in the Merkerlist
 *
 *	@return void
 */
void writeLabelSection(FILE **fp,MERKERLIST *firstMerker)
{
	int anzLab=0;
	int i=0;
	unsigned char zeichen;
	int help; 
	MERKERLIST *helpM;
	for(anzLab=0,helpM=firstMerker;helpM!=NULL;helpM=helpM->next,anzLab++);
	fwriteIntBigEndian(anzLab,fp);
	for(helpM=firstMerker;helpM!=NULL;helpM=helpM->next)
	{
		fwriteIntBigEndian(strlen(helpM->name),fp);
		help=(helpM->adress)>>16;		//hier 32 bit reinschreiben erst H dann L
		fwriteIntBigEndian(help,fp);
		fwriteIntBigEndian(helpM->adress,fp); 
		writeString(fp,helpM->name);
	}
	
}

/*! This function writes the line-section into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int lineInd how much lines the file includes
 *	@param short int* lineTab table of the lines and memory adresses which belongs together
 *
 *	@return void
 */
void writeLineSection(FILE **fp, int lineInd, short int *lineTab)
{
	int i,help;
	for(i=0;i<lineInd;i++)
	{
		fwriteIntBigEndian(i+1,fp);
		help = (lineTab[i])>>16;     // hier 32 bit reinscheiben erst H dann L
		fwriteIntBigEndian(help,fp);
		fwriteIntBigEndian(lineTab[i],fp);
	}
}

/*! This function writes the code-section into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int* secArray array where the sections are saved
 *	@param CODELIST* helpC pointer to an element of the Codelist
 *	@param CODELIST* firstCode pointer to the first Code in the Codelist
 *	@param short int* memory array where the code is saved
 *
 *	@return void
 */		
void writeCodeSection(FILE **fp,CODELIST *firstCode,CODELIST *helpC,unsigned int *memory)
{
	int i=0;
	int j=0;
	int tmp;

	CODELIST *help=firstCode;
	for(i=0;help!=helpC;help=help->next)
	{
		i=i+(help->length);
	}
	tmp = (helpC->startAdress)>>16;
 	fwriteIntBigEndian(tmp,fp);
	fwriteIntBigEndian(helpC->startAdress,fp);
	//fwriteInt32Bit(helpC->startAdress,fp);
	for(j=i;j<(i+helpC->length);j++)
	{
		tmp = (memory[j])>>16;
		fwriteIntBigEndian(tmp,fp);
		fwriteIntBigEndian(memory[j],fp);
		//fwriteInt32Bit(memory[j],fp);
	}
}

/*! This function writes the header into the File
 *
 *	@param FILE** fp pointer to the output file
 *	@param int magic_number the magic number of the compiler version
 *
 *	@return void
 */
void createHeader(FILE **fp,int magic_number)
{
	fwriteIntBigEndian(magic_number,fp);
	fwriteIntBigEndian(0,fp);
}
