/*! expression.c
 *  In this file are the functions which are used for calculating expressions
 *
 *	@author Eggenmueller Bernd , Schmidt Juergen
 *	$Revision: 1.0 $
 *	@date 25.04.2007
 */
#include <string.h>
#include <stdlib.h>
#include "commit.h"
#include "functions.h"
#include "errors_and_warnings.h"

/*! This function walks through the list of the expressions written in firstExpr, calculate them and write them into the memory-array
 *
 *	@param WERT_RETURN *firstExpr pointer
 *	 @param unsigned int *memory pointer
 * 	@param MERKERLIST *firstMerk pointer
 *
 *	@return void
 */


/*
!Achtung!

die load Variable zeigt die ART der expr. an, also wie sie letzendlich in den Speicher geschrieben werden muss,
dh. in welche Bits.

0: Bit 0 - Bit 15
1: Bit 21 - Bit 28
2: alle Bits, denn Konstante (DC)

dagegen die flag Variable in der COMMAND_RETURN Struktur, zeigt an, um welchen Befehl es sich handelt:
abfrage oder low8 Variablen 
0 : alle anderen Befehle ohne expr.
1 : assembler direktive kann auch expr. enthalten z.B. DC 80+30
2:  Befehle mit expr. z.B. LD ST ALU JMP VMOV
	                


*/
void calculateExpressions(WERT_RETURN *firstExpr,unsigned int *memory,MERKERLIST *first_Merker)
{
	WERT_RETURN *help;
	int buffer; 	// 32-Bit Buffer
	//walk through the list of expressions
	for(help=firstExpr;help!=NULL;help=help->next)
	{
		// Die Direktwerte sollten mit 0 initialisiert werden
		if(help->load==0)  //Bit 0-15 Befehl mit Direktwert     
		{	
			memory[help->memory]+=(0xffff & calculate(help->expression,first_Merker));
			//printf("%x\n", memory[help->memory]); // for debug only
		}
		else if(help->load==1) //Bit 21 - 28 Vektorbefehl mit Direktwert
		{
			buffer = calculate(help->expression,first_Merker);
			//printf("%x\n",buffer); //for debug only
			memory[help->memory]+=((buffer & 0xff) <<20);
			//printf("%x\n", memory[help->memory] );
		}
		else if(help->load==2)// Bit 0 - 31 Konstante
		{
			
			memory[help->memory]=(calculate(help->expression,first_Merker));
			
		}
		//Unexpected error, normally help->load must be 0,1 or 2
		else
		{
			Err_Out();
		}
	}
}

/*! This function calculate the expression and calls it self recursive
 *
 *	@param MERKERLIST* first pointer to the first Merker in the Merkerlist
 *	@param EXPRESSION_RETURN* expr pointer to one part of a whole expression
 *
 *	@return int returns if there is an error 1 or not 0
 */
int calculate(EXPRESSION_RETURN *expr, MERKERLIST *first)
{
	MERKERLIST *help;
	
	//select the operation
	switch(expr->operation)
	{	
		//returns the value
		case E_VAL:	return expr->value;
		
		//returns the memory place of label which is written in the expression
		case E_LAB:	for(help=first;help!=NULL;help=help->next)
				{
					if(strcasecmp(expr->label,help->name)==0)
					{
						//printf("%x\n", help->adress); // for debug only
						return help->adress;
					}
				}
				Err_LabelNotExist(expr->label);
				
		//returns the calculation of the expression and calls the function recursive
		case E_ADDOP:	return (calculate(expr->left,first)+calculate(expr->right,first));
		case E_SUBOP:	return (calculate(expr->left,first)-calculate(expr->right,first));
		case E_MULOP:	return (calculate(expr->left,first)*calculate(expr->right,first));
		case E_DIVOP:	return (calculate(expr->left,first)/calculate(expr->right,first));
		case E_BITSHIFT_L:	return (calculate(expr->left,first)<<calculate(expr->right,first));
		case E_BITSHIFT_R:	return (calculate(expr->left,first)>>calculate(expr->right,first));
		case E_CLAMP:	return (calculate(expr->left,first));
		case E_NOTOP:	return (~calculate(expr->left,first));
		case E_ANDOP:	return (calculate(expr->left,first)&calculate(expr->right,first));
		case E_OROP:	return (calculate(expr->left,first)|calculate(expr->right,first));
		default:	Err_Out();
	}
}
