/*
*  Filename: getchain.c
*  Author: Michael Schaeferling
*  Institute: Hochschule Augsburg, Fakultaet fuer Informatik
*  Date of last change: 09-03-12
*  About: This file is part of the OnChipDebugger (via RS232).
*         This example demonstrates how to use the libraries ReadChain() function.
*  Version: 0.50
*/


#include <stdio.h>        /* printf */
#include <stdlib.h>       /* atoi */
#include <string.h>
#include "chipdebuglib.h"

#define OUTSTREAM "/dev/ttyUSB0"

int main(int argc, char* argv[])
{  
  int i;
  int chainLen = atoi(argv[2]);
   
  if (argc != 4){
    printf("Usage: getchain <chainID> <chainLength> <verbosity>\n");
    return 1;
  }
  else{
  char* chainBytes = (char*)calloc(chainLen + 1,sizeof(char)); /* +1 for c-string */
  
  /* for testing purpose initialize chainBytes with x */
  for (i = 0; i<chainLen + 1; i++)
    chainBytes[i] = 'x';
  chainBytes[chainLen] = '\0';

  printf("Length in main(): %d(total) / %d(string)\n", sizeof(chainBytes), chainLen);

  SetVerbosity(atoi(argv[3]));
  printf("Verbosity is set to: %i\n", GetVerbosity());

  Open(IOSTREAM);

  if (ReadChain(atoi(argv[1]), chainBytes) > -1){
    printf("Chain:\n");
    for (i = 0; i < strlen(chainBytes); i++)
      printf("0x%.2x ", (unsigned char)chainBytes[i]);
    printf("\n%s\n", chainBytes);
    Close();
    free(chainBytes);
    return 0;
  }
  else {
    Close();
    free(chainBytes);
    return -1;
  }
  }
}

