/*
*  Filename: testing.c
*  Author: Michael Schaeferling
*  Institute: Hochschule Augsburg, Fakultaet fuer Informatik
*  Date of last change: 09-03-23
*  About: This file is part of the OnChipDebugger (via RS232).
*         This example is for testing and combining some functions of the library.
*  Version: 0.50
*/


#include <stdio.h>        /* printf */
#include <stdlib.h>       /* atoi */
#include <string.h>
#include "chipdebuglib.h"

int main(int argc,char* argv[])
{
  if (argc != 4){
    printf("Usage: sendchain <chainID> <chain> <verbosity>\n");
    return 1;
  }
  else{
  SetVerbosity(atoi(argv[3]));
  printf("Verbosity is set to: %i\n", GetVerbosity());

  Open(IOSTREAM);

 while(getchar()){

  /* do some writing */
  if (WriteChain(atoi(argv[1]), argv[2]) == -1) {
    Close();
    printf("## Writechain failed.\n");
    return 0;
  }
  
  /* do some reading*/
  int chainLen = strlen(argv[2]);
  int i;
  char* chainBytes = (char*)calloc(chainLen + 1,sizeof(char)); /* +1 for c-string */

  /* for testing purpose initialize chainBytes with x */
  for (i = 0; i<chainLen + 1; i++)
    chainBytes[i] = 'x';
  chainBytes[chainLen] = '\0';

  if (ReadChain(atoi(argv[1]), chainBytes) > -1){
    printf("Chain:\n");
    for (i = 0; i < strlen(chainBytes); i++)
      printf("0x%.2x ", (unsigned char)chainBytes[i]);
    printf("\n%s\n", chainBytes);
  }
  free(chainBytes);
 }

  Close();
  return 0;
  }
}

