#!/usr/bin/python
# This file is part of the SCOTCH project
# The original file is part of the VISCY project.
# (C) 2007-2008 Gundolf Kiefer, Fachhochschule Augsburg, University of Applied Sciences

from string import *
from sys import *
import array
import os
import cmd

import chipdebug


version="1.0"



################### Configuration #########################


optSerialDevice="/dev/ttyS0"





################### Helpers ###############################


def toBinary (val, len):
  return "".join(map(lambda y:str((val >> y) & 1), range(len-1, -1, -1)))


def fromBinary (x):
  return sum(map(lambda z: int(x[z]) and 2**(len(x) - z - 1), range(len(x)-1, -1, -1)))


def toInt (s):
  if s[0:2] == "0x": return int(s[2:], 16);
  else: return int(s);





################### (Object) file I/O #####################


def readObjAndSrcFile (objFileName):
  "returns (memory, comments), both as dictionaries by adress"
  
  # Read the object file...
  f = open (objFileName, "rb")
  objData = array.array('H')
  try: objData.fromfile (f, 999999)
  except EOFError: pass
  print "Read object file '%s'." % objFileName

  # Try to read source file...
  try:
    srcFileName = objFileName.rsplit('.', 1)[0] + ".asm"
    srcLines = open(srcFileName, "rt").readlines ()
    print "Read source file '%s'." % srcFileName
  except IOError:
    srcLines = []

  # Convert to little endian...
  for n, v in enumerate (objData):
    #print objData[n]
    objData[n] = ((v & 0xff) << 8) | (v >> 8)
    print "%5i. %04x" % (n, v)  
    
  # Init 'outMemory' and 'outComments'...
  outMemory = {}
  outComments = {}
  
  # Parse header...
  magnum = objData[0]
  if magnum != 0xf00e: raise Exception, "Wrong magic number - not an object file?"
  #getting amount of sections
  sections = objData[2]
  print "Sections = %i" % sections
  
  # Parse sections...
  for secNum in range(sections):
    #calculate correct secNum-startadress: one sectiondir-entry is 5*16Bit, therefor 5*SecNum and 3 for the header ...
    idx = 3 + 5 * secNum
    secType, secStartHi, secStart, secEndHi, secEnd = objData[idx : idx+5]   # idx : idx+5 , also von : bis mehrfachzuweisung
    #bring adress in high-word
    secStart += secStartHi << 16
    secEnd += secEndHi << 16
    print "Type = %i, Start = %i, End = %i " % (secType, secStart, secEnd),
  
    if secType == 1:   # 'Code 16 BIT VISCY'
      adr = objData[secStart]
      n = secEnd - secStart
      #print "-> Code: 0x%04x .. 0x%04x" % (adr, adr + n - 1)
      for i in range(n):outMemory[adr + i] = objData[secStart + 1 + i]
	
    elif secType == 3: # 'Zeilen 16 BIT VISCY'
      n = (secEnd - secStart + 1) / 2
      print "-> Line info: %i source lines" % n
      for i in range(n):
        lineNo, adr = objData[secStart + 2 * i : secStart + 2 * i + 2]
        try:
          outComments[adr] = rstrip (srcLines[lineNo-1])
          print "  0x%04x =%4i: %s" % (adr, lineNo, rstrip (srcLines[lineNo-1]))
        except: pass
	
    elif secType == 0x11: # 'Code 32 BIT HICOVEC'
      adr = objData[secStart]
      adr = adr << 16
      adr += objData[secStart+1]
      n = secEnd - (secStart +1)
      print "-> Code: 0x%04x .. 0x%04x" % (adr, adr + n - 1)

      z = 0
      for i in range(1,n,2):
	outMemory[adr + z] = objData[secStart + 1 + i]
	outMemory[adr + z] = (outMemory[adr + z]) << 16
	outMemory[adr + z] += objData[secStart + 2 + i]
    	z = z+1
	#print "%04x" % objData[secStart + i]
	
    elif secType == 0x13: # 'Zeilen 32 BIT HICOVEC'
      n = (secEnd - secStart + 1) / 2
      print "-> Line info: %i source lines" % n
      for i in range(n):
        lineNo, adr = objData[secStart + 2 * i : secStart + 2 * i + 2]
        try:
          outComments[adr] = rstrip (srcLines[lineNo-1])
          print "  0x%04x =%4i: %s" % (adr, lineNo, rstrip (srcLines[lineNo-1]))
        except: pass
	
    #else:
    #  print "-> ignoring"
     
  return (outMemory, outComments)



def printVhdl (memory, comments):
  if len(memory) == 0:
    print "Memory is completely empty!"
    return;

  # Add dummy memory entries for commented but undefined memory adresses...
  for adr in comments.iterkeys():
    if not adr in memory: memory[adr] = -1
  
  # Output VHDL...
  adrList = memory.keys ()
  adrList.sort()
  print "  -- Memory content (created by VOHDCA) ..."
  print "  type t_memory is array (%i to %i) of STD_LOGIC_VECTOR(32 downto 0);" % (adrList[0], adrList[-1])
  print "  signal mem_content: t_memory := ("
  for adr in adrList:
    val = memory[adr]
    if val < 0: word = "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU"
    else: word = toBinary (val, 32)
    print "      16#%04x# => \"%s\"," % (adr, word),
    if adr in comments: print "  -- %s" % comments[adr],
    print
  print "      others => \"UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU\""
  print "    );"





################### Scan chain communication ##############


### Low-level...


clkShell = 0x11
clkCpu = 0x22
clkIo = 0x44


class Pins:
  def __init__(self):
    self.initCpuReady()
    self.inMemCycle = 0   # for external use

  def initCpuReady(self):
    # initialize such that the CPU + IO is runnable
    self.captureCpu = "1"
    self.captureIo = "1"
    self.reset = "0"
    self.rd = "0"
    self.wr = "0"
    self.ready = "0"
    self.adr = "11111111111111111111111111111111"
    self.data = "00000000000000000000000000000000"


  def toStrCpuOnly(self):
    return "reset=" + self.reset + " rd=" + self.rd + " wr=" + self.wr        \
        + " ready=" + self.ready + " adr=" + self.adr + " data=" + self.data

  def toStr(self):
    return "captureCpu=" + self.captureCpu + " captureIo=" + self.captureIo     \
        + "   " + self.toStrCpuOnly()

  def read(self):
    chain = '-' * 70   # dummy init with correct length
    if chipdebug.ReadChain (0, chain) != 0: raise Exception, "Error reading scan chain!"
    self.captureCpu = chain[0]
    self.captureIo = chain[1]
    self.reset = chain[2]
    self.rd = chain[3]
    self.wr = chain[4]
    self.ready = chain[5]
    self.adr = chain[6:38]
    self.data = chain[38:70]
    #print "### Pins.read: " + self.toStr()

  def write(self):
    chain = self.captureCpu + self.captureIo + self.reset + self.rd + self.wr + self.ready + self.adr + self.data
    #print "read '%s', " % rdChain,
    #print "written '%s', " % wrChain,
    tmp = chipdebug.WriteChain (0, chain)
    if tmp   == -1: raise Exception, "WriteChain() failed! Chain has at least one bit"
    elif tmp == -2: raise Exception, "WriteChain() failed! Chain too long, max 16383 bits"
    elif tmp == -3: raise Exception, "WriteChain() failed! Max 4 chains (0-3) supported"
    elif tmp == -4: raise Exception, "WriteChain() failed! Not all bytes were sent"
    elif tmp == -5: raise Exception, "WriteChain() failed! Instruction not sucessfully executed by OnChipDebugger"
    elif tmp == -6: raise Exception, "WriteChain() failed! Unknown ack (exec) received"
    
    #print "### Pins.write: " + self.toStr()


def boardClk (cycles, mask):
  #print "### boardClk: %i, 0x%02x" % (cycles, mask)
  return chipdebug.Step (cycles, mask) == 0


def boardInit ():
  chipdebug.Open (optSerialDevice)
  chipdebug.SetVerbosity (0)


def boardDone ():
  chipdebug.Close()

def boardRunAuto (mask):    # run system autonomously
  chipdebug.Run (mask)

def boardStopAuto ():
  chipdebug.Stop ()

def boardRecover ():
  "recover after a break due to Ctrl-C"
  pins = Pins ()
  while True:
    try:
      boardDone ()
      boardInit ()
      pins.read ()
      pins.write ()
      #print "Board communication is fine again. :-)"
      break
    except: pass



### High-level (no console output)...


def boardReset ():
  pins = Pins()
  pins.reset = "1"
  pins.write()
  boardClk (3, clkCpu)


def boardSetMem (adr, val):
  clocksPerCheck = 3  # to accelerate read operations

  pins = Pins()
  pins.data = "01100110011001100110011001100110"
  pins.captureCpu = '0'
  # shift in "no read and no write"...
  pins.write()
  boardClk (clocksPerCheck, clkIo | clkShell)
  # wait until memory is ready to receive requests...
  
  pins.read()
  while pins.ready != '0':
    boardClk (clocksPerCheck, clkIo | clkShell)
    pins.read()
  
  # send write request...
  pins.wr = '1';
  pins.adr = toBinary (adr, 32)
  pins.data = toBinary (val, 32)
  pins.write()
  # wait until memory has completed the request ...
  while pins.ready != '1':
    boardClk (clocksPerCheck, clkIo | clkShell)
    pins.read()


def boardGetMem (adr):
  clocksPerCheck = 4  # to accelerat read operations

  pins = Pins()
  pins.data = "01100110011001100110011001100110"
  pins.captureCpu = '0'

  # shift in 'no read or write'...
  pins.write()
  boardClk (clocksPerCheck, clkIo | clkShell)
  # wait until memory is ready to receive requests...
  pins.read()
  while pins.ready != '0':
    boardClk (clocksPerCheck, clkIo | clkShell)
    pins.read()
  # send read request ...
  pins.rd = '1';
  pins.adr = toBinary (adr, 32)
  pins.write()
  # wait until memory has completed the request ...
  while pins.ready != '1':
    boardClk (clocksPerCheck, clkIo | clkShell)
    pins.read()
  return fromBinary (pins.data)



### High-level (with console output)...


def uploadObject(memory):
  keys = memory.keys()
  keys.sort()
  for adr in keys:
    boardSetMem (adr, memory[adr])
    stdout.write ('.')
    stdout.flush()
  print


def boardStepMonitored ():
  pins = Pins()

  pins.write()            # set 'capture...' bits appropriately
  boardClk (5, clkShell)  # let shell assume present signal lines from CPU/IO (not the values just shifted in)

  boardClk (1, clkShell | clkCpu | clkIo)  # run CPU for one clock

  pins.read()
  print pins.toStrCpuOnly ()


def boardRunMonitored (memory, comments):
  clkCount = 0
  noMemCount = 1
  pins = Pins()

  pins.write()            # set 'capture...' bits appropriately

  try:
    while noMemCount <= 20:
      boardClk (5, clkShell)  # let shell assume present signal lines from CPU/IO (not the values just shifted in)
      boardClk (1, clkShell | clkCpu | clkIo)  # run CPU for one clock
      pins.read()
      #print pins.toStrCpuOnly ()
      adrVal = fromBinary (pins.adr)
      dataVal = fromBinary (pins.data)
      if pins.wr == "1":
        if noMemCount > 0: print "\nw(0x%04x) = 0x%04x   %s" % (adrVal, dataVal, comments.get(adrVal, ""))
        noMemCount = 0
      elif pins.rd == "1" and pins.ready == "1":
        if noMemCount > 0: print "\nr(0x%04x) = 0x%04x   %s" % (adrVal, dataVal, comments.get(adrVal, ""))
        noMemCount = 0
      else:
        noMemCount += 1
      clkCount += 1
      stdout.write ('.')
      stdout.flush()
    print
  except KeyboardInterrupt:
    boardRecover()
    print "\nStopped"
  print "Clock cycles: %i" % clkCount


def boardRunFullSpeed ():
  pins = Pins()

  pins.write()            # set 'capture...' bits appropriately
  boardClk (4, clkShell)  # let shell take over present signal lines from CPU/IO (not the values just shifted in)

  # Using the OCD's 'Start' and 'Stop' commands...
  print "System is running at full speed. Press 'return' to stop."
  boardRunAuto (clkShell | clkCpu | clkIo)
  stdin.readline ()
  stdin.flush ()
  boardStopAuto ()
  boardRecover ()
  print "Stopped"

  # Alternative version not using the OCD's 'Start' and 'Stop' commands...
#  print "System is running at full speed. Press 'Ctrl-C' to stop."
#  try:
#    while True:
#      boardClk (10000, clkShell | clkCpu | clkIo)  # run CPU for many clocks
#  except KeyboardInterrupt:
#    boardRecover()
#    print "\nStopped"




############## Memory emulation mode (OBSOLETE!) ##########


# NOTE: The following function probably won't work


def boardRunEmulatedMem (memory, comments):

  def stepCpu (pins):

    # Clock shell once to capture eventual asynchronous CPU outputs
    if chipdebug.Step (1, 0x11) != 0: status = '?'  # raise Exception, "Error in 'Step'!"
    # Read CPU pins...
    pins.readChain ()
    # Create reaction on memory request from previous cycle...
    adrVal = fromBinary (pins.adr)
    if pins.wr == "1":
      dataVal = fromBinary (pins.data)
      memory[adrVal] = dataVal
      if not pins.inMemCycle: print "\nw(0x%04x) = 0x%04x   %s" % (adrVal, dataVal, comments.get(adrVal, ""))
      pins.inMemCycle = 1
      pins.ready = "1"
    elif pins.rd == "1":
      if not adrVal in memory: memory[adrVal] = 0xf0f0
      dataVal = memory[adrVal]
      pins.data = toBinary(dataVal, 16)
      #print "### dataVal = %04x, %s" % (dataVal, pins.data)
      if not pins.inMemCycle: print "\nr(0x%04x) = 0x%04x   %s" % (adrVal, dataVal, comments.get(adrVal, ""))
      pins.inMemCycle = 1
      pins.ready = "1"
    else:
      pins.inMemCycle = 0
    # Write CPU pins...
    status = '.'
    pins.writeChain ()
    # Clock CPU...
    if chipdebug.Step (1, 0x33) != 0: status = '?'  # raise Exception, "Error in 'Step'!"
    #print pins.toStr()
    # Conclude...
    pins.ready = "0"
    stdout.write (status)
    stdout.flush()

  pins = Pins()
  stepCpu (pins)
  stepCpu (pins)
  pins.reset = "0"
  timer = 0
  while timer < 20:
    stepCpu (pins)
    timer = timer + 1
    if pins.wr == "1" or pins.rd == "1": timer = 0
  print




################### Command interpreter ###################


class MyCmd(cmd.Cmd):

  def __init__(self):
    cmd.Cmd.__init__(self)
    self.prompt = "(VOHDCA) "
    self.intro = ""
    self.memory = {}
    self.comments = {}

  # Helpers...
  def completeFileName(self, text, line, begidx, endidx):
    name = text
    if line[-1] == ' ': dir = ""
    else: dir = line.rsplit(" ", 1) [-1]
    if len(name) > 0: dir = dir[:-len(name)]
    if dir == "": dir = "."
    #print [ text, line, begidx, endidx, dir, name ]
    ret = []
    try:
      textlen = len(text)
      for f in os.listdir(dir) + [ ".", ".." ]:
        if f[:textlen] == text:
          if os.access (dir + "/" + f + "/", os.F_OK): ret += [ f + "/" ]
          else: ret += [ f + " " ]
    except: pass
    return ret

  def error(self, msg):
    print "*** %s" % msg
    #self.do_help(self.lastcmd);

  # Overloaded pre-defined methods...
  def completenames(self, text, line, begidx, endidx):
    l = cmd.Cmd.completenames(self, text, line, begidx, endidx)
    #print l, begidx, endidx, "EOF"[endidx:]
    try: l.remove("EOF")
    except: pass
    return [x + " " for x in l]

  def onecmd(self, str):
    try:
      return cmd.Cmd.onecmd(self, str)
    except KeyboardInterrupt:
      boardRecover()
      print "\nStopped"
    except Exception, msg:
      self.error(msg)
      return False

  def do_help(self, arg):
    print
    if arg: cmd.Cmd.do_help(self, arg)
    else:
      for x in dir(self):
        if x[:5] == "help_": exec ("self." + x + "()")
    print

  # Command: Exit ...
  def do_exit(self, arg):
    return True
  def help_exit(self):
    print "exit              - exit the tool"
  def do_EOF(self, arg):
    print
    return self.do_exit(arg)

  # Command: Open...
  def do_open(self, arg):
    (self.memory, self.comments) = readObjAndSrcFile (arg)
  def help_open(self):
    print "open <file name>  - open an object file"
  def complete_open(self, text, line, begidx, endidx):
    #print "complete_open"
    return self.completeFileName (text, line, begidx, endidx)

  # Command: VHDL dump...
  def do_dump(self, arg):
    printVhdl(self.memory, self.comments)
  def help_dump(self):
    print "dump              - dump the memory contents (VHDL syntax)"

  # Command: reset...
  def do_reset(self, arg):
    boardReset ()
  def help_reset(self):
    print "reset             - reset the CPU"

  # Command: get ...
  def do_get(self, arg):
    args = arg.split(" ")
    adr0 = toInt(args[0])
    if len(args) > 1: num = toInt (args[1])
    else: num = 8
    for adr in range (adr0, adr0 + num):
      data = boardGetMem (adr)
      #self.memory[adr] = data
      #print "###", adr, self.memory[adr]
      print "0x%04x: 0x%04x" % (adr, data),
      if adr in self.comments: print "  -- %s" % self.comments[adr],
      print
  def help_get(self):
    print "get <adr> [<n>]   - display <n> memory words starting from <adr>"

  # Command: set ...
  def do_set(self, arg):
    args = arg.split(" ");
    adr = toInt(args[0]);
    val = toInt(args[1]);
    #self.memory[adr] = val;
    boardSetMem (adr, val)
  def help_set(self):
    print "set <adr> <val>   - write <val> to memory adress <adr>"

  # Command: upload ...
  def do_upload(self, arg):
    uploadObject(self.memory)
  def help_upload(self):
    print "upload            - write the loaded object file to the board memory"

  # Command: run_emumem...
  def do_run_emumem(self, arg):
    boardRunEmulatedMem (self.memory, self.comments)

  # Command: step...
  def do_step(self, arg):
    boardStepMonitored ()
  def help_step(self):
    print "step              - run system for one clock cycle and display CPU pins"

  # Command: run_monitored...
  def do_run_monitored(self, arg):
    boardRunMonitored (self.memory, self.comments)
  def help_run_monitored(self):
    print "run_monitored     - run system slowly and print information on memory accesses"

  # Command: run_fullspeed...
  def do_run_fullspeed(self, arg):
    boardRunFullSpeed ()
  def help_run_fullspeed(self):
    print "run_fullspeed     - run system at full speed"




################### Main ##################################


def usage ():
  print ("Usage: vohdca [options]\n"
         "\nOptions:\n"
         "  -h              : this help\n"
         "  -d <device>     : set serial device (default: " + optSerialDevice + ")\n"
         "  -c <command(s)> : execute commands upon startup\n")
  exit ()


from getopt import *


print "VOHDCA - Viscy2l Optimized for Hicovec to Debug, Compute and Assemble - v. ", version, "\n\n(C) 2008 - 2009\n","Jakob Golus\n","David Lucinkiewicz\n","Andreas Mueller\n","Florian Richter\n","Sebastian Rigorth\n","Soeren Ruehm\n","University of Applied Sciences Augsburg\n\n","Original viscy2l: (c) 2007-2008 Gundolf Kiefer, University of Applied Sciences Augsburg\n"


# Read options...
firstCmds = []
try:
  opts, args = getopt (argv[1:], "hd:c:")
  for opt, value in opts:
    if opt == "-d": optSerialDevice = value;
    elif opt == "-c": firstCmds = value.split(";")
    #elif opt == "-h":
    else: usage ()
except GetoptError:
  usage ()

# opens serial interface
boardInit ()
# instances prompt environment
myCmd = MyCmd()
noExit = True

# Run first commands...
for c in firstCmds:
  try:
    print myCmd.prompt + c.strip()
    noExit = not myCmd.onecmd (c)
    if not noExit: break
  except Exception:
    break

# Re-run the main command loop as long it has been ended by an exception (e.g. KeyboardInterrupt, Ctrl-C)
while noExit:
  try:
    myCmd.cmdloop()
    noExit = False
  except KeyboardInterrupt:
    print

boardDone ()
raw_input



#chipdebug.Open(optSerialDevice);
#objFileName = argv[1]
#(memory, comments) = readObjAndSrcFile (objFileName)
#autoRunMemEmu ("/dev/ttyS0", memory, comments)
#printVhdl (memory, comments)
